@@ -572,7 +572,7 @@ check_files2() {
 
 check_files_by_md5() {
 	csum=`${CAT} "${package_dir}"/catalog/dfiles/files | ${SED} -e 's@/$@@' | ${SORT} | ${MD5SUM} | sed -e s/^'(stdin)= '//`
-	fsum=`find "${package_dir}" | ${SED} -e 's@/$@@' | ${SORT} | ${MD5SUM}`
+	fsum=`find "${package_dir}" | ${SED} -e 's@/$@@' | ${SORT} | ${MD5SUM} | sed -e s/^'(stdin)= '//`
 	# OLD slow way:  fsum=`${TAR} cf - ${package_dir} | ${TAR} tf - | ${SED} -e 's@/$@@' | ${SORT} | ${MD5SUM} | sed -e s/^'(stdin)= '//`
 	case "$csum" in
 		"") return 1
