@@ -51,6 +51,7 @@
 #include "swproglib.h"
 #include "swverify_lib.h"
 #include "atomicio.h"
+#include "swlex_supp.h"
 
 #define SWREMOVE_UPVERBOSE	0
 #define PROGNAME                "swverify"
@@ -174,7 +175,7 @@ fprintf(fp, "%s",
 "     -W C   same as -W emit-signed-file [FILE].\n"
 "     -W emit-digest-file [FILE]  Writes the digested archive data to stdout.\n"
 "     -W S   same as -W emit-digest-file.\n"
-"     -W sig-number=N Verify the Nth signature, (0 is last, 1 is first). \n"
+"     -W sig-number=N Verify the Nth signature, (0 is last -1 is all). \n"
 "     -W owner=OWNER    Specify OWNER\n"
 "     -W group=GROUP    Specify GROUP\n"
 "     -W get-sig-if=outputfile [FILE] Same as -G=OUTPUTFILE \n"
@@ -198,12 +199,14 @@ fprintf(fp, "%s",
 "      --check-scripts=true|false    Not implemented\n"
 "      --check-volatile=true|false\n"
 "      --check-owners=true|false\n"
-"      --show-info    show the matching comparison info to stdout\n"
+"      --show-info               show the matching comparison info to stdout\n"
 "      --catalog-info-name=NAME  write comparison info to filename NAME\n"
-"      --system-info-name=NAME  write comparison info to filename NAME\n"
-"      --dereference-symlinks  when obtaining file system info\n"
-"      --ignore-slack-install  ignore Slackware control files when comparing\n"
+"      --system-info-name=NAME   write comparison info to filename NAME\n"
+"      --dereference-symlinks    when obtaining file system info\n"
+"      --ignore-slack-install    ignore Slackware control files when comparing\n"
 "      --without-summary-report  ignore Slackware control files when comparing\n"
+"      --output-form=FORM        diff|rpm|all, diff is the current and historic default \n"
+"      --no-prelink              no checking failed digests via prelink\n"
 "\n"
 "  Verification Options for verifying a directory:\n"
 "\n"
@@ -287,7 +290,7 @@ fprintf(fp, "%s",
 " Swbis Extended Options:\n"
 "        File : <libdir>/swbis/swbisdefaults\n"
 "               ~/.swbis/swbisdefaults\n"
-"        Command Line Syntax : -x option=option_argument\n"
+"        Command Line Syntax : -W option=option_argument\n"
 "  swverify.swbis_no_getconf		 = true # getconf is used\n"
 "  swverify.swbis_shell_command	         = detect\n"
 "  swverify.swbis_no_remote_kill	 = false\n"
@@ -504,6 +507,7 @@ main(int argc, char **argv)
 	int e_argc = 1;
 	int w_argc = 1;
 	int swi_event_fd = -1;
+	int prelink_fd = -1;
 	pid_t ss_pid = 0;
 	pid_t target_write_pid = 0;
 	pid_t target_pump_pid = 0;
@@ -565,6 +569,8 @@ main(int argc, char **argv)
 	int * p_do_progressmeter;
 	int make_master_raw = 1;
 	int is_local_stdin_seekable = 0;
+	int tmpret = 0;
+	int tmp_status = 0;
 	int nullfd;
 	int testnum = 0;
 	int v_retval = 0;       /* verification status */
@@ -624,6 +630,7 @@ main(int argc, char **argv)
 	STROB * target_line_buf;
 	STROB * isc_script_buf;
 	STROB * tmpcommand;
+	STROB * verid_buf;
 	STROB * target_catalog_dir;
 	CPLOB * w_arglist;
 	CPLOB * e_arglist;
@@ -661,6 +668,8 @@ main(int argc, char **argv)
 		{"version", 0, 0, 'Y'},
 		{"help", 0, 0, '\012'},
 		{"noop", 0, 0, 154},
+		{"output-format", 1, 0, 155},
+		{"no-prelink", 0, 0, 156},
 		{"show-options", 0, 0, 158},
 		{"show-options-files", 0, 0, 159},
 		{"testnum", 1, 0, 164},
@@ -900,6 +909,7 @@ main(int argc, char **argv)
 	btmp2 = strob_open(10);		/* General use String object. */
 
 	tmpcommand = strob_open(10);
+	verid_buf = strob_open(10);
 	w_arglist = cplob_open(1);	/* Pointer list object. */
 	e_arglist = cplob_open(1);	/* Pointer list object. */
 	swicol = swicol_create();
@@ -981,14 +991,21 @@ main(int argc, char **argv)
 	cplob_add_nta(w_arglist, strdup(argv[0]));
 	cplob_add_nta(e_arglist, strdup(argv[0]));
 
+	/* default output format */				
+	G->g_output_formM = SWVERIFY_OUTF_DIFF;
+
 	while (1) {
 		int option_index = 0;
 
-		c = ugetopt_long(argc, argv, "df:t:prvs:X:x:W:", 
+		c = ugetopt_long(argc, argv, "Adf:t:prvs:X:x:W:", 
 					main_long_options, &option_index);
 		if (c == -1) break;
 
 		switch (c) {
+		case 'A':
+			wopt_forward_agent = CHARTRUE;
+			set_opta(opta, SW_E_swbis_forward_agent, wopt_forward_agent);
+			break;
 		case 'd':
 			G->g_do_distribution = 1;
 			break;
@@ -1222,6 +1239,20 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 		switch (c) {
 		case 154: /* noop */
 			break;
+		case 155:
+			if (strncmp(optarg, "rpm", 3) == 0) {
+				G->g_output_formM = SWVERIFY_OUTF_RPM;
+			}
+			if (strncmp(optarg, "all", 3) == 0 || strncmp(optarg, "both", 4) == 0) {
+				G->g_output_formM = SWVERIFY_OUTF_ALL;
+			}
+			if (strncmp(optarg, "diff", 4) == 0) {
+				G->g_output_formM = SWVERIFY_OUTF_DIFF;
+			}
+			break;
+		case 156:
+			G->g_no_prelinkM = 1;
+			break;
 		case 158:
 			wopt_show_options = CHARTRUE;
 			break;
@@ -1627,6 +1658,7 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 	swgp_signal(SIGTERM, safe_sig_handler);
 	swgp_signal(SIGUSR1, safe_sig_handler);
 
+	swlex_set_input_policy(SWLEX_INPUT_UTF_TR);
 	/*
 	 * = = = = = = = = = = = = = = = = = = = = = = = = = = = 
 	 *          Process the Software Selections
@@ -2048,8 +2080,36 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 
 			if (ret < 0) {
 				/* Error, start message indicated failure */
+				E_DEBUG("ABC");
+
+				/*
+				 * This is (may be) the failure path for ssh authentication
+				 * failure.
+				 */
+		
+				/*
+				 * See if the ssh command has finished.
+				 */
+				if (swlib_wait_on_pid_with_timeout(ss_pid, &tmp_status, 0 /*flags*/, 1/*verbose_level*/, 3 /* Seconds */) > 0) {
+					/*
+					 * Store the result in the status array.
+					 */
+					SWLIB_ASSERT(
+						swlib_update_pid_status(ss_pid, tmp_status,
+							G->g_pid_array, G->g_status_array, G->g_pid_array_len) == 0);
+				}
+				E_DEBUG("ABCD");
+				if (WIFEXITED(tmp_status)) {
+					/*
+					 * This is the exit status of ssh (or rsh)
+					 */
+					tmpret = WEXITSTATUS(tmp_status);
+					swpl_agent_fail_message(G, current_arg, tmpret);
+				} else {
+					sw_e_msg(G, "SW_INTERNAL_ERROR for target %s in file %s at line %d\n",
+						current_arg, __FILE__, __LINE__);
+				}
 				LC_RAISE(SIGTERM);
-				/* Signal Check */
 				swc_check_for_current_signals(G, __LINE__, wopt_no_remote_kill);
 				sw_e_msg(G, "read_start_ctl_message error (loc=start)\n");
 				LCEXIT(sw_exitval(G, target_loop_count, target_success_count));
@@ -2257,6 +2317,51 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 				raise(SIGINT);
 			}
 
+			/*
+			 * Get the prelink(8)'ed file list for this host
+			 */
+
+			ret = 0;
+			if (f_retval == 0)
+				ret = swpl_run_get_prelink_filelist(G, swicol, target_fdar[1], &prelink_fd);
+
+			if (ret != 0) {
+				abort_script(swicol, G, target_fdar[1]);
+        			swpl_send_abort(swicol, target_fdar[1], G->g_swi_event_fd, "");
+				f_retval++;
+				raise(SIGINT);
+			} else {
+				/* 
+ 				 * Run some sanity checks on the prelink_fd
+				 */ 			
+				char * xxpr_text;
+				int xxpr_ret;
+				
+				SWLIB_ASSERT(prelink_fd > 0);
+				xxpr_text = uxfio_get_fd_mem(prelink_fd, (int *)NULL);
+				uxfio_lseek(prelink_fd, (off_t)0, SEEK_SET);
+				xxpr_ret = swlib_pipe_pump(nullfd, prelink_fd);
+				G->g_prelink_fdM = prelink_fd;
+				G->g_prelink_fd_memM = xxpr_text;
+				if (xxpr_ret != (int)strlen(xxpr_text)) {
+					/* Sanity check */
+					/*
+					fprintf(stderr, "PRELINK: xxpr_ret = [%d]\n", xxpr_ret);
+					fprintf(stderr, "PRELINK: strlen(xxpr_text) = [%d]\n", (int)strlen(xxpr_text));
+					*/
+					G->g_prelink_fdM = -1;
+					G->g_prelink_fd_memM = NULL;
+					sw_e_msg(G, "Internal error, prelink buffer lengths not equal: [%d] [%d]\n",
+							xxpr_ret, (int)strlen(xxpr_text));
+				}
+				if (strlen(xxpr_text) == 0) {
+					/* Normal for system without prelink'ed binaries */
+					uxfio_close(prelink_fd);
+					G->g_prelink_fdM = -1;
+					G->g_prelink_fd_memM = NULL;
+				}
+			}
+
 			swc_check_for_current_signals(G, __LINE__, wopt_no_remote_kill);
 
 			E_DEBUG("");
@@ -2399,13 +2504,20 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 					if (compare_status != 0) {
 						v_retval++;
 						swc_stderr_fd2_set(G, STDERR_FILENO);
-							sw_l_msg(G, SWC_VERBOSE_1, "%s at sig_level=%s for %s: status=1\n",
-							swevent_code(SW_SOC_INTEGRITY_NOT_CONFIRMED), wopt_sig_level, sc->sqM->swspec_stringM);
+							sw_l_msg(G, SWC_VERBOSE_1,
+								"%s at sig_level=%s for %s: %s: status=1\n",
+								swevent_code(SW_SOC_INTEGRITY_NOT_CONFIRMED),
+									wopt_sig_level,
+									sc->sqM->swspec_stringM,
+									swverid_print(sr->swspecM, verid_buf));
 						swc_stderr_fd2_restore(G);
 					} else {
 						swc_stderr_fd2_set(G, STDERR_FILENO);
-						sw_l_msg(G, SWC_VERBOSE_2, "%s at sig_level=%s for %s: status=0\n",
-							swevent_code(SW_SOC_INTEGRITY_CONFIRMED), wopt_sig_level, sc->sqM->swspec_stringM);
+						sw_l_msg(G, SWC_VERBOSE_1, "%s at sig_level=%s for %s: %s: status=0\n",
+							swevent_code(SW_SOC_INTEGRITY_CONFIRMED),
+								wopt_sig_level,
+								sc->sqM->swspec_stringM,
+								swverid_print(sr->swspecM, verid_buf));
 						swc_stderr_fd2_restore(G);
 					}
 				}  /* loop over responses */
