@@ -32,7 +32,7 @@ g_pax_write_command g_pax_write_commands[] = {
 		"tar cbf 1 -"
 	},
 	{"gtar", 
-		"gtar cbf 1 -"
+		"gtar cbf 1 - --format=pax"
 	},
 	{"swbistar",
 		SWBISLIBEXECDIR "/swbis/swbistar"
