@@ -68,11 +68,27 @@ construct_controlsh_script(GB * G, STROB * buf, SWI * swi)
 {
 	char * tag;
 	STROB * tmp;
+	STROB * tmp2;
 	STROB * taglist;
 
 	taglist = strob_open(100);
 	tmp = strob_open(100);
+	tmp2 = strob_open(100);
 	strob_strcpy(buf, "");
+
+	/*
+	 * Construct the ch'dir command to SW_ROOT
+	 */
+
+	if (G->g_no_script_chdirM) {
+		strob_sprintf(tmp2, STROB_NO_APPEND, "\n");
+	} else {
+		strob_sprintf(tmp2, STROB_NO_APPEND, 
+			"# here chdir  cd $SW_ROOT\n"
+			"cd \"$SW_ROOT\" || exit 1\n"
+		);
+	}
+
 	swicat_write_auto_comment(buf, SW_A_CONTROL_SH);
 	swicat_env(buf, swi, NULL, NULL);
 
@@ -114,6 +130,12 @@ construct_controlsh_script(GB * G, STROB * buf, SWI * swi)
 		, strob_str(taglist)
 		);
 
+	strob_sprintf(buf, STROB_DO_APPEND,
+		"# Set the interpreter default value\n"
+		"INTERPRETER=\"sh\"\n"
+		"export INTERPRETER\n"
+		);
+
 	strob_sprintf(buf, STROB_DO_APPEND,
 		"case \"$SWBCS_MATCH\" in\n"
 		);
@@ -136,7 +158,75 @@ construct_controlsh_script(GB * G, STROB * buf, SWI * swi)
 		CSHID
 		"# Now run the script\n"
 		"\n"
-		"sh \"${SW_CONTROL_DIRECTORY}/${SWBCS_SCRIPT_NAME}\"\n"
+		"# Now set the INTERPRETER\n"
+		"#\n"
+		"# SWPATH is set above\n"
+		"export CONFPATH\n"
+		"CONFPATH=\"$SWPATH\"\n"
+		"case \"$CONFPATH\" in\n"
+		"\"\")\n"
+		"CONFPATH=\"$PATH\"\n"
+		";;\n"
+		"esac\n"
+
+		"# /bin/bash is the first choice\n"
+		"case \"${INTERPRETER}\" in\n"
+		"\"\"|sh)\n"
+		"#\n"
+		"# Find PGM in CONFPATH\n"
+		"export PGM=bash\n"
+		"# Here is a shell implementation of which(1)\n"
+		"location=`IFS=: ; set --; set -- \"$CONFPATH\"; for i in $*; do test -x \"$i/$PGM\"; res=$?; case \"$res\" in 0) echo \"$i\"; exit 0;;  esac; done; exit 1; `\n"
+		"case \"$?\" in\n"
+		"0)\n"
+		"INTERPRETER=\"$location\"/$PGM\n"
+		";;\n"
+		"esac\n"
+		";;\n"
+		"esac\n"
+
+		"# sh as found in CONFPATH is the next choice\n"
+		"case \"${INTERPRETER}\" in\n"
+		"\"\"|sh)\n"
+		"#\n"
+		"# Find PGM in CONFPATH\n"
+		"export PGM=sh\n"
+		"# Here is a shell implementation of which(1)\n"
+		"location=`IFS=: ; set --; set -- \"$CONFPATH\"; for i in $*; do test -x \"$i/$PGM\"; res=$?; case \"$res\" in 0) echo \"$i\"; exit 0;;  esac; done; exit 1; `\n"
+		"case \"$?\" in\n"
+		"0)\n"
+		"INTERPRETER=\"$location\"/$PGM\n"
+		";;\n"
+		"esac\n"
+		";;\n"
+		"esac\n"
+
+		"# /usr/xpg4/bin/sh is the last choice\n"
+		"case \"${INTERPRETER}\" in\n"
+		"\"\"|sh)\n"
+		"test -x /usr/xpg4/bin/sh\n"
+		"case \"$?\" in\n"
+		"0)\n"
+		"INTERPRETER=/usr/xpg4/bin/sh\n"
+		";;\n"
+		"esac\n"
+		";;\n"
+		"esac\n"
+
+		"case \"${INTERPRETER}\" in\n"
+		"\"\"|sh)\n"
+		"# No interpreter found\n"
+		"echo \"swbis: control.sh (tag=$SW_CONTROL_TAG): interpreter [${INTERPRETER}] not found\" 1>&2\n"
+		"exit 1\n"
+		";;\n"
+		"esac\n"
+
+		"#\n"
+		"# All the checks are done, Now execute the script\n"
+		"#\n"
+		"%s\n"
+		"export SW_CONTROL_DIRECTORY\n"
+		"\"${INTERPRETER}\" \"${SW_CONTROL_DIRECTORY}/${SWBCS_SCRIPT_NAME}\"\n"
 		"swbcs_ret=$?\n"
 		"case \"$swbcs_ret\" in\n"
 		"	0) ;;\n"
@@ -145,11 +235,13 @@ construct_controlsh_script(GB * G, STROB * buf, SWI * swi)
 		"	3) ;;\n"
 		"	*) swbcs_ret=1 ;;\n"
 		"esac\n"
-		"exit $swbcs_ret\n"
+		"exit $swbcs_ret\n",
+		strob_str(tmp2)
 		);
 
 	strob_close(taglist);
 	strob_close(tmp);
+	strob_close(tmp2);
 	return 0;
 }
 
@@ -191,6 +283,20 @@ write_tar_controlsh_file(GB * G, SWI * swi, int ofd,
 	return ret;
 }
 
+
+static
+int
+copy_attributes(GB * G, SWHEADER * infoheader, struct new_cpio_header * file_hdr)
+{
+	int ret;
+	ret = swheader_fileobject2filehdr(infoheader, file_hdr);
+
+	/* FIXME, also copy the extended attributes into file_hdr */
+
+	return ret;
+}
+
+
 static
 int
 write_tar_session_options_file(GB * G, SWI * swi, int ofd,
@@ -656,7 +762,7 @@ write_catalog_archive_member(SWI * swi, int ofd,
 	catalog_tar_hdr = (struct tar_header *)malloc(TARRECORDSIZE);
 	xformat = swi->xformatM;
 	buf = strob_open(100);
-	etar = etar_open(swi->xformatM->taruM);
+	etar = etar_open(swi->xformatM->taruM->taru_tarheaderflagsM);
 	etar_init_hdr(etar);
 
 	/*
@@ -1021,7 +1127,7 @@ static
 int
 send_files_with_no_storage(GB * G, SWI * swi, SWHEADER * infoheader, XFORMAT * xformat, int fileset_fd, int vofd,
 				int * pstatus, char * target_path, char * location,
-				char * directory, uintmax_t * pstatbytes)
+				char * directory, uintmax_t * pstatbytes, char * tar_format)
 {
 	int ret;
 	int retval;
@@ -1043,6 +1149,7 @@ send_files_with_no_storage(GB * G, SWI * swi, SWHEADER * infoheader, XFORMAT * x
 	ahs = ahs_open();
 	*pstatus = 0;
 
+	E_DEBUG2("tar_format=[%s]", tar_format);
 	swheader_store_state(infoheader, NULL);
 	swheader_reset(infoheader);
 	while((next_line=swheader_get_next_object(infoheader, (int)UCHAR_MAX, (int)UCHAR_MAX))) {
@@ -1087,6 +1194,7 @@ send_files_with_no_storage(GB * G, SWI * swi, SWHEADER * infoheader, XFORMAT * x
 				continue;
 			} else if (
 				*current_type == SW_ITYPE_d || /* directory */
+				*current_type == SW_ITYPE_h || /* hard link */
 				*current_type == SW_ITYPE_s || /* sym link */
 				*current_type == SW_ITYPE_c || /* char device */
 				*current_type == SW_ITYPE_b || /* block device */
@@ -1105,9 +1213,11 @@ send_files_with_no_storage(GB * G, SWI * swi, SWHEADER * infoheader, XFORMAT * x
 						break;
 					}
 
-					if (strlen(current_link_source) > TARLINKNAMESIZE) {
-						fprintf(stderr, "tar link name too long\n");
-						SWI_internal_error();
+					E_DEBUG("HERE");
+					if ( strcmp(tar_format, "ustar") == 0 &&
+					     strlen(current_link_source) > TARLINKNAMESIZE) {
+						sw_e_msg(G, "tar link name too long for ustar format, try -H gnu\n");
+						/* SWI_internal_error(); */
 						*pstatus = 4;
 						break;
 					}
@@ -1115,12 +1225,12 @@ send_files_with_no_storage(GB * G, SWI * swi, SWHEADER * infoheader, XFORMAT * x
 						swpl_sanitize_pathname(current_link_source);
 					}
 					taru_set_new_linkname(xformat->taruM, (struct tar_header *)NULL, current_link_source);
-					ahsStaticSetTarLinkname(
+					ahsStaticSetPaxLinkname(
 						ahs_vfile_hdr(xformat_ahs_object(xformat)),
 						 current_link_source);
 				} else {
 					taru_set_new_linkname(xformat->taruM, (struct tar_header *)NULL, "");
-					ahsStaticSetTarLinkname(ahs_vfile_hdr(xformat_ahs_object(xformat)), "");
+					ahsStaticSetPaxLinkname(ahs_vfile_hdr(xformat_ahs_object(xformat)), "");
 				}
 
 				if (pstatbytes) {
@@ -1221,7 +1331,6 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 	int allow_missing_files, SWICAT_E * up_e)
 {
 	int format;
-	int output_format;
 	int vofd;
 	int padamount;
 	int read_header_ret;
@@ -1263,10 +1372,12 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 	STROB * execution_phase_script;
 	STROB * current_path_expanded;
 	STROB * relocated_current_path;
+	STROB * relocated_current_linksource;
 	STROB * shell_lib_buf;
 	STROB * dir_owner_buf;
 	AHS * info_ahs2 = ahs_open();
 	char * obj;
+	char * tar_format;
 	int tarheaderflags;
 	struct new_cpio_header * file_hdr;
 	XFORMAT * xformat = swi->xformatM;
@@ -1301,6 +1412,9 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 	CISF_PRODUCT * cisf;
 	SWI_FILELIST * fl;
 
+	tar_format = swbisoption_get_opta(opta, SW_E_swbis_format);
+	E_DEBUG2("tar_format=[%s]", tar_format);
+
 	E_DEBUG2("fp: catalog_path=[%s]", catalog_path);
 	signal_flag = 0;
 	p_signal_flag = &signal_flag;
@@ -1321,6 +1435,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 	tmp = strob_open(10);
 	current_path_expanded = strob_open(10);
 	relocated_current_path = strob_open(10);
+	relocated_current_linksource = strob_open(10);
 	pathtmp = strob_open(10);
 	resolved_path = strob_open(10);
 	execution_phase_script = strob_open(10);
@@ -1331,7 +1446,6 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 	
 	swi->swicolM->verbose_levelM = swi->verboseM;
 	swicol_set_verbose_level(swi->swicolM, swi->verboseM);
-	output_format = xformat_get_output_format(xformat);
 	format = xformat_get_format(xformat);
 	ifd = xformat_get_ifd(xformat);
 	if (ifd < 0) {
@@ -1966,7 +2080,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 		*p_signal_flag == 0 &&
 		retval == 0
 	) {
-		E_DEBUG("HERE: Storage");
+		E_DEBUG("HERE: ******************************************** Storage");
 		if (xformat_is_end_of_archive(xformat)){
 			break;
 		}
@@ -1974,6 +2088,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 		E_DEBUG("");
 		xformat_get_name(xformat, namebuf);
 		name = strob_str(namebuf);
+		E_DEBUG2("***************************************** name=[%s]", name);
 		parseret = swpath_parse_path(swpath, name);
 		E_DEBUG("");
 		if (parseret < 0) {
@@ -2046,7 +2161,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 		E_DEBUG2("HERE: Testing if control directory: %s", l_current_path);
 	
 		if (l_current_path && strlen(l_current_path)) {
-			E_DEBUG("HERE: Storage");
+			E_DEBUG2("HERE: Storage: current_path_expanded is [%s]", strob_str(current_path_expanded));
 			if (obj == (char*)NULL) {
 				E_DEBUG("");
 				obj = swheader_get_object_by_tag(infoheader, SW_OBJ_file, strob_str(current_path_expanded));
@@ -2112,7 +2227,11 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 			E_DEBUG("");
 			ahs_init_header(info_ahs2);
 			E_DEBUG("");
-			ret = swheader_fileobject2filehdr(infoheader, ahs_vfile_hdr(info_ahs2));
+
+			/*
+ 			 * Copy the attributes from the INFO file to the file_hdr struct
+ 			 */
+			ret = copy_attributes(G, infoheader, ahs_vfile_hdr(info_ahs2));
 			if (ret) {
 				/*
 				* FIXME, handle this error better.
@@ -2215,24 +2334,58 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 				current_link_source = swheader_get_attribute(infoheader, SW_A_link_source, NULL);
 				current_link_source = swheaderline_get_value(current_link_source, NULL);
 				if (!current_link_source) { SWI_internal_error(); return -68; }
-
-				if (strlen(current_link_source) > TARLINKNAMESIZE) {
-					fprintf(stderr, "tar link name too long\n");
-					SWI_internal_error();
-					return -74;
-				}
-				if (strcmp(target_path, "/") == 0) {
-					/* swpl_safe_check_pathname(current_link_source); */
+				if (swlib_check_clean_relative_path(current_link_source) == 0) {
+					/*
+ 					 * If path is relative do not apply location modification
+ 					 * FIXME: is this the right thing to do ?
+ 					 *
+ 					 * FIXME: maybe need to determine if part of the 'directory' attribute
+ 					 * is part of the relative path. and also process any ../ to see if they
+ 					 * intersect any of the 'directory''s levels
+ 					 */
+					E_DEBUG2("relative linksource [%s]", current_link_source);
+					strob_strcpy(relocated_current_linksource, current_link_source);
 				} else {
-					swpl_sanitize_pathname(current_link_source);
+					/*
+ 					 * Link source is absolute
+ 					 */
+					if (
+						(!p_directory || strcmp(p_directory, "/") == 0) &&
+						(!p_location || strcmp(p_location, "/") == 0)
+					) {
+						/*
+ 						 * Both are nul or '/' (root default),
+ 						 * Do nothing, this is a special case to preserve the linksource
+ 						 * exactly as it is in the package
+ 						 */
+						E_DEBUG2("absolute linksource: [%s]", current_link_source);
+						strob_strcpy(relocated_current_linksource, current_link_source);
+					} else {
+						/*
+ 						 * apply relocation
+ 						 */
+						swlib_apply_location(relocated_current_linksource, 
+							current_link_source, p_location, p_directory);
+						E_DEBUG2("current_path after applying location: %s",
+							strob_str(relocated_current_linksource));
+						if (strcmp(target_path, "/") == 0) {
+							/* swpl_safe_check_pathname(current_link_source); */
+						} else {
+							swpl_sanitize_pathname(strob_str(relocated_current_linksource));
+						}
+						swlib_squash_all_leading_slash(strob_str(relocated_current_linksource));
+					}
 				}
-				taru_set_new_linkname(xformat->taruM, (struct tar_header *)NULL, current_link_source);
-				ahsStaticSetTarLinkname(
+
+				taru_set_new_linkname(xformat->taruM,
+						(struct tar_header *)NULL,
+						strob_str(relocated_current_linksource));
+				ahsStaticSetPaxLinkname(
 					ahs_vfile_hdr(xformat_ahs_object(xformat)),
-					 current_link_source);
+					strob_str(relocated_current_linksource));
 			} else {
 				taru_set_new_linkname(xformat->taruM, (struct tar_header *)NULL, "");
-				ahsStaticSetTarLinkname(ahs_vfile_hdr(xformat_ahs_object(xformat)), "");
+				ahsStaticSetPaxLinkname(ahs_vfile_hdr(xformat_ahs_object(xformat)), "");
 			}
 
 			if (pstatbytes) {
@@ -2253,7 +2406,8 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 			 */
 			if (fileset_fd != vofd) {
 				/*
-				 * This code path is used in preview mode.
+				 * This code path is used in preview mode and writing the
+				 * archive to stdout.
 				 */
 				xformat_set_ofd(xformat, fileset_fd);
 				E_DEBUG("");
@@ -2422,7 +2576,7 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 		   avert this error. */
 		ret2 = 1;
 		ret = send_files_with_no_storage(G, swi, infoheader, xformat, fileset_fd, vofd, &ret2, target_path,
-						p_location, p_directory, pstatbytes);
+						p_location, p_directory, pstatbytes, tar_format);
 		fileset_write_ret += ret;
 	} else {
 		ret2 = 0;
@@ -2433,6 +2587,9 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 		 * this is an error
 		 */
 		E_DEBUG("STORAGE FILE error");
+		if (allow_missing_files == 0) {
+			sw_e_msg(G, "storage section has missing files: error status %d\n", ret2);
+		}
 		files_missing_from_storage = 1;
 	}
 
@@ -2622,13 +2779,19 @@ arfinstall_tar(GB * G, SWI * swi, int ofd,
 	strob_close(dir_owner_buf);
 	strob_close(resolved_path);
 	strob_close(execution_phase_script);
+	strob_close(relocated_current_path);
+	strob_close(relocated_current_linksource);
 	taru_free_header(file_hdr);
 	ahs_close(info_ahs2);
+
 	if (allow_missing_files) {
 		E_DEBUG2("HERE returning %d", retval);
 		return retval;
 	} else {
 		E_DEBUG2("HERE returning %d", retval || files_missing_from_storage);
+		if (files_missing_from_storage) {
+			sw_e_msg(G, "missing files, use --allow-missing-files to allow\n");
+		}
 		return retval || files_missing_from_storage;
 	}
 }
@@ -2808,6 +2971,10 @@ write_target_install_script(GB * G,
 
 	if (strcmp(get_opta(G->optaM, SW_E_swbis_shell_command), "detect") == 0) {
 		swpl_bashin_detect(buffer_new);
+	} else if (strcmp(get_opta(G->optaM, SW_E_swbis_shell_command), "posix") == 0) {
+		swpl_bashin_posixsh(buffer_new);
+	} else {
+		swpl_bashin_testsh(buffer_new, get_opta(G->optaM, SW_E_swbis_shell_command));
 	}
 
 	strob_sprintf(buffer_new, STROB_DO_APPEND,
