@@ -6,6 +6,15 @@
 #ifndef uxformatcpio_200007_h
 #define uxformatcpio_200007_h
 
+extern "C" {
+#include "swuser_config.h"
+}
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <limits.h>
+
 #include "xformat.h"
 #include "uxformat.h"
 class uxFormatCpio: public uxFormat
