@@ -75,7 +75,7 @@ class swExDfiles: public swObjFiles
 	void tuneSignatureFile(void) {
 		char * sig;
 		sig=getSigFileBuffer();
-		tuneAttributeFile("signature", sig);
+		tuneAttributeFile(SW_A_signature, sig);
 	}
 
 	void tuneAttributeFile(char * name, char * buffer) {
@@ -129,7 +129,7 @@ class swExDfiles: public swObjFiles
 						setErrorCode(10451, NULL);
 					}
 
-					if (::strcmp(name, "signature") == 0) {	
+					if (::strcmp(name, SW_A_signature) == 0) {	
 						int padlen;
 						char * pad;
 						int sigend;
