@@ -145,7 +145,9 @@ swAttribute * swAttribute::make_newAttribute (int fd, char * keyword, char * val
 
 int swAttribute::write_fd_debug (int uxfio_fd, char * prefix) {
 	STROB * tmp = strob_open(10);
-	char * line = static_cast<char*>(NULL);
+	char * line;
+	char * line2;
+	char * line3;
 	char * val;
 	int ret;
 	int value_len;
@@ -154,9 +156,9 @@ int swAttribute::write_fd_debug (int uxfio_fd, char * prefix) {
 
 	ret = ::swlib_writef(uxfio_fd, tmp,
 	"%s[%s] keyword=[%s] value=[%s] len=[%d] offset=[%d] inode=[%d] status=[%d]\n",  prefix,
-		(line=get_parserline()) ? line : "ERROR: get_parserline returned null.", 
-		(line=get_keyword()) ? line : "", 
-		((line=val) ? line : ""),
+		((line=get_parserline())) ? line : "ERROR: get_parserline returned null.", 
+		((line2=get_keyword())) ? line2 : "", 
+		((line3=val) ? line3 : ""),
 		/* get_length() */ 0,
 		get_p_offset(),
 		get_ino(),
