@@ -7,7 +7,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 
@@ -25,8 +25,15 @@
 #ifndef swdeffile_19980601jhl_h
 #define swdeffile_19980601jhl_h
 
+extern "C" {
+#include "swuser_config.h"
+}
+
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
+#include <limits.h>
+
 #include "swdefinition.h"
 
 class swDefFile: public swDefinition {
