@@ -356,6 +356,151 @@ int swMetaData::swmetadata_local_close(SWVARFS * swvarfs, int u_fd) {
 	return ret;
 }
 
+mode_t
+swMetaData::getModeFromFileType(mode_t mode, int ch) {
+	switch (ch) {
+		case SW_ITYPE_f:
+			mode &= ~CP_IFMT; mode |= CP_IFREG;
+			break;
+		case SW_ITYPE_d:
+			mode &= ~CP_IFMT; mode |= CP_IFDIR;
+			break;
+		case SW_ITYPE_c:
+			mode &= ~CP_IFMT; mode |= CP_IFCHR;
+			break;
+		case SW_ITYPE_b:
+			mode &= ~CP_IFMT; mode |= CP_IFBLK;
+			break;
+		case SW_ITYPE_p:
+			mode &= ~CP_IFMT; mode |= CP_IFIFO;
+			break;
+		case SW_ITYPE_s:
+			mode &= ~CP_IFMT; mode |= CP_IFLNK;
+			break;
+		case SW_ITYPE_h:
+			mode &= ~CP_IFMT; // FIXME, what else to do?
+			break;
+		default:
+			fprintf(stderr, "swpackage: internal error in getModeFromFileType\n");
+			return '\0';
+	}
+	return mode;
+}
+
+int
+swMetaData::fileStats2statbuf( struct stat * statbuf, char array[][file_permsLength], int array_is_set[])
+{
+	int ret;
+	int ret2;
+	unsigned long ulmode;
+	unsigned long ul;
+	mode_t mode;
+	time_t tm;
+	gid_t guid;
+	uid_t uuid;
+
+	ret = 0;
+	memset(statbuf, '\0', sizeof(struct stat));
+	ulmode = 0;
+
+	// Set the mode
+	if (array_is_set[modeE]) {
+		taru_otoul (array[modeE], &ulmode);
+		mode = (mode_t)ulmode;
+		statbuf->st_mode = mode;
+	} else {
+		statbuf->st_mode = 0;
+		ret++;		
+	}
+
+	//
+	// Set the file type
+	if (array_is_set[typeE] && ret == 0) {
+		mode = getModeFromFileType(mode, (int)((unsigned char)(*array[typeE])));
+		statbuf->st_mode |= mode;
+	} else {
+		ret++;
+	}
+	
+	//
+	// Set the uid
+	//
+	if (array_is_set[uidE] && ret == 0) {
+		ul = 0;
+		if (taru_datoul(array[uidE], &ul))
+			ret++;
+		statbuf->st_uid = (uid_t)ul;
+	} else if (array_is_set[ownerE] && ret == 0) {
+		ret2 = taru_get_uid_by_name(array[ownerE], &uuid);
+		if (ret2) {
+			fprintf(stderr, "swpackage: owner name not found: %s\n", array[ownerE]);
+			ret++;
+		} else {
+			statbuf->st_uid = uuid;
+		}
+	} else {
+		ret++;
+	}
+
+	//
+	// Set the gid
+	//
+	if (array_is_set[gidE] && ret == 0) {
+		ul = 0;
+		if (taru_datoul(array[gidE], &ul))
+			ret++;
+		statbuf->st_gid = (gid_t)ul;
+	} else if (array_is_set[groupE] && ret == 0) {
+		ret2 = taru_get_gid_by_name(array[groupE], &guid);
+		if (ret2) {
+			fprintf(stderr, "swpackage: group name not found: %s\n", array[groupE]);
+			ret++;
+		} else {
+			statbuf->st_gid = guid;
+		}
+	} else {
+		ret++;
+	}
+
+	//
+	// Set the mtime;
+	//
+	if (array_is_set[mtimeE] && ret == 0) {
+		if (taru_datoul(array[mtimeE], &ul))
+			ret ++;
+		statbuf->st_mtime = (time_t)ul;
+		statbuf->st_ctime = statbuf->st_mtime;
+		statbuf->st_atime = statbuf->st_mtime;
+	} else if (array_is_set[mtimeE] == 0 && ret == 0) {
+		statbuf->st_mtime = time(NULL);
+		statbuf->st_ctime = statbuf->st_mtime;
+		statbuf->st_atime = statbuf->st_mtime;
+	} else {
+		ret ++;
+	}
+
+
+	// FIXME? this abandons hard link information
+	statbuf->st_nlink = 0;
+
+	//
+	// Set the size
+	//
+	if (*array[typeE] == SW_ITYPE_f && array_is_set[sizeE] && ret == 0) {
+		ul = 0;
+		if (taru_datoul(array[sizeE], &ul))
+			ret++;
+		statbuf->st_size = (off_t)ul;
+	} else if (*array[typeE] != SW_ITYPE_f) {
+		statbuf->st_size = (off_t)(0);
+	} else {
+		ret++;
+	}
+
+	return ret;
+}
+
+
 char
 swMetaData::getFileTypeFromMode(mode_t mode) {
 	int ch;
@@ -397,7 +542,7 @@ swMetaData::getFileTypeFromMode(mode_t mode) {
 
 /* Private */ void
 swMetaData::setTarLinkname(char * dest, char * linkname, int size) {
-	if (strlen(linkname) > 100) {
+	if (strlen(linkname) > file_permsLength) {
 		fprintf(stderr, "link source name too long: link_name=[%s]\n", linkname);
 		fprintf(stderr, "fatal... exiting with status 1\n");
 		_exit(1);
@@ -586,14 +731,14 @@ int swMetaData::swmetadata_decode_file_stats(SWVARFS * swvarfs,
 		p_linksource = array[linksourceE];
 		p_swbi = array[swbiE]; 	// Internal attribute used as a flag.
 		
-		array_is_set[linksourceE] = 0;
-		if (*p_type == SW_ITYPE_s) {
+		// array_is_set[linksourceE] = 0;
+		if (*p_type == SW_ITYPE_s && array_is_set[linksourceE] == 0) {
 			//
 			// Get the link source.
 			//
 			SWMETADATA_DEBUG("");
 			array_is_set[linksourceE] = 1;
-			if (swvarfs_u_readlink(swvarfs, source, p_linksource, (size_t)(file_permsLength)) <= 0) {
+			if (swvarfs_u_readlink(swvarfs, source, p_linksource, (size_t)(file_permsLength-1)) <= 0) {
 				fprintf(stderr, "error reading link for %s : error or link source too long.\n", source);
 				ret = -1;
 				*p_linksource = '\0';
@@ -777,7 +922,7 @@ int swMetaData::swmetadata_decode_file_stats(SWVARFS * swvarfs,
 		if (*p_owner == '\0') {
 			uid_t uid = st->st_uid;
 			SWMETADATA_DEBUG("p_owner == nil");
-			taru_get_user_by_uid(uid,  p_owner);
+			taru_get_tar_user_by_uid(uid,  p_owner);
 			snprintf(p_uid,  file_permsLength, "%d",(int)uid);
 			p_uid[file_permsLength-1] = '\0';
 			array_is_set[ownerE] = 1;
@@ -802,7 +947,7 @@ int swMetaData::swmetadata_decode_file_stats(SWVARFS * swvarfs,
 		if (*p_group == '\0') {
 			gid_t gid = st->st_gid;
 			SWMETADATA_DEBUG("p_group == nil");
-			taru_get_group_by_gid(gid, p_group);
+			taru_get_tar_group_by_gid(gid, p_group);
 			snprintf(p_gid,  file_permsLength, "%d",(int)gid);
 			p_gid[file_permsLength-1] = '\0';
 			array_is_set[groupE] = 1;
