@@ -51,7 +51,6 @@ extern "C" {
 }
 
 // #define SWPACKAGEFILENEEDDEBUG 1
-#define SWPACKAGEFILENEEDDEBUG 1
 #undef SWPACKAGEFILENEEDDEBUG 
 
 #ifdef SWPACKAGEFILENEEDDEBUG
@@ -183,6 +182,10 @@ class swPackageFile :  public swAttribute
 	int initialize_file_stats_array(swDefinition * swdef,
 				char array[][file_permsLength],
 					int array_is_set[]); 
+	
+	int check_no_stat_optimization(swDefinition * swdef,
+				char array[][file_permsLength],
+					int array_is_set[]); 
 
 	virtual int add (char * keyword, char * value, swMetaData * newnode, int level) {
 		int c;
@@ -519,6 +522,10 @@ class swPackageFile :  public swAttribute
 		uxformatM->xFormat_set_xformat(xf);
 	}
 
+	void xFormat_init_vfile_hdr(void) {
+	    return uxformatM->xFormat_init_vfile_hdr();
+	}
+
 	struct new_cpio_header * xFormat_vfile_hdr(void) {
 		return uxformatM->xFormat_vfile_hdr();
 	}
@@ -746,7 +753,7 @@ class swPackageFile :  public swAttribute
 				}
 				source_fd = xFormat_u_open_file(strob_str(tmpM));  // This sets the global stat/header buf.
 				if (source_fd < 0)  {
-					return -4;
+					return -40;
 				}
 			} else {
 				source_fd = -1;
@@ -783,7 +790,7 @@ class swPackageFile :  public swAttribute
 				1 /*logical ON, means vremoved attr. will be ignored*/,
 				tmp_control_path);
 			if (ret)
-				return -4;	
+				return -40;	
 			
 			swfile_set_package_filename(package_path);
 			source = swdef->find(SW_A_source); // This line fixed a core dump.
@@ -792,8 +799,10 @@ class swPackageFile :  public swAttribute
 				fprintf(stderr, "%s: error expanding escapes for package_path ( %s ) ret=%s\n",
 					swlib_utilname_get(), package_path, swlib_imaxtostr(ret, NULL));
 			}
+			SWPACKAGEFILE_DEBUG("Running xFormat_write_file");
 			ret = swPackageFile::xFormat_write_file(static_cast<struct stat *>(NULL), package_path, source_fd);
 			if (source_fd >= 0) xFormat_u_close_file(source_fd);
+			SWPACKAGEFILE_DEBUG2("Running xFormat_write_file ret = %d", ret);
 		} else {
 			//
 			// Stat the source.
@@ -806,7 +815,9 @@ class swPackageFile :  public swAttribute
 				fprintf(stderr, "%s: error expanding escapes for package_path ( %s ) ret=%s\n",
 					swlib_utilname_get(), package_path, swlib_imaxtostr(ret, NULL));
 			}
+			SWPACKAGEFILE_DEBUG("Running xFormat_write_file");
 			ret = swPackageFile::xFormat_write_file(static_cast<struct stat*>(NULL), package_path, strob_str(tmpM)); 
+			SWPACKAGEFILE_DEBUG2("Running xFormat_write_file ret = %d", ret);
 		}
 		return ret;
 	}
@@ -1075,7 +1086,6 @@ class swPackageFile :  public swAttribute
 		return swpathM->swp_get_basename();
 	}
 	
-	
 	virtual char *  swp_form_path (STROB * buf) {
 		return swpathM->swp_form_path(buf);
 	}
