@@ -1,4 +1,4 @@
-/* swlib_rpmfile.h
+/* rpmar_rpmfile.h
  */
 
 /*
@@ -6,6 +6,9 @@
    This file may be copied under terms of GNU GPL.
 */
 
+/* Not currently used */
+#define RPMINCLUDE_RPMFILE_H
+
 
 #ifndef RPMINCLUDE_RPMFILE_H
 #define RPMINCLUDE_RPMFILE_H
@@ -41,4 +44,3 @@ int rpmfile_fix_rpminfo_tarheader ( unsigned char * oldheader, unsigned char * n
 /* int rpmfile_rpminfo_size ( int read_write , int value ); */
 
 #endif
-
