@@ -85,7 +85,7 @@
 {1, 0, 0, "swbis_bzip2",		"false",		NULL, SWC_U_P},
 {1, 0, 0, "swbis_numeric_owner",	"false",		NULL, SWC_U_P},
 {1, 0, 0, "swbis_absolute_names",	"false",		NULL, SWC_U_P},
-{0, 0, 0, "swbis_format",		"ustar",		NULL, SWC_U_P},
+{0, 0, 0, "swbis_format",		"ustar",		NULL, SWC_U_P|SWC_U_I},
 {1, 0, 0, "swbis_no_analysis_phase",	"false",		NULL, SWC_U_X},
 {0, 0, 0, "swbis_archive_reader", 	"tar",			NULL, SWC_U_X},
 {1, 0, 0, "swbis_no_audit",	 	"true",			NULL, SWC_U_C},
@@ -104,7 +104,7 @@
 {0, 0, 0, "swbis_remote_shell_client",	"/usr/bin/ssh",		NULL, SWC_U_C|SWC_U_I|SWC_U_L},
 {1, 0, 0, "swbis_install_volatile",	"true",			NULL, SWC_U_I},
 {0, 0, 0, "swbis_volatile_newname",	"",			NULL, SWC_U_I},
-{1, 0, 0, "swbis_any_format",		"false",		NULL, SWC_U_L},
+{1, 0, 0, "swbis_any_format",		"false",		NULL, SWC_U_L|SWC_U_I|SWC_U_P|SWC_U_C},
 {1, 0, 0, "swbis_forward_agent",	"true",			NULL, SWC_U_L|SWC_U_C|SWC_U_I|SWC_U_V|SWC_U_G},
 {1, 0, 0, "swbis_ignore_scripts",	"false",		NULL, SWC_U_I|SWC_U_G},
 {0, 0, 0, "swbis_sig_level",		"0",			NULL, SWC_U_R|SWC_U_I|SWC_U_L|SWC_U_V|SWC_U_G},
