@@ -32,6 +32,23 @@
 #include "swparse.h"
 #include "strob.h"
 
+
+int
+swdef_write_attribute_to_buffer(STROB * buf, char *keyword, char *value, int level, int attribute_type)
+{
+	int iret, ret;
+
+	ret = swdef_write_keyword_to_buffer(buf, keyword, level, attribute_type);
+	if (attribute_type != SWPARSE_MD_TYPE_OBJ) {
+		iret = swdef_write_value_to_buffer(buf, value);
+		if (iret < 0)
+			return -1;
+		ret += iret;
+	}
+	return ret;
+}
+
+
 int
 swdef_write_attribute(char *keyword, char *value, int level, int len, 
 					int attribute_type, int uxfio_fd)
