@@ -18,6 +18,9 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
+#define FILENEEDDEBUG 1
+#undef FILENEEDDEBUG
+
 #include "swuser_config.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -215,6 +218,7 @@ swlib_pty_fork2(int *ptrfdm, int ofd, int ifd, int efd,
 	pid_t	c_pid;
 	char	pts_name[20];
 	char *  tmpc;
+	char * pts_type;
 	char    slave_pty_name[50];
 	int     op[2];
 	int     ip[2];
@@ -222,8 +226,15 @@ swlib_pty_fork2(int *ptrfdm, int ofd, int ifd, int efd,
 	int 	testfds = -1;
 	int	start = 0;
 	int have_openpty = 0;
+	int have_posix_openpt = 0;
 	int	ret;
 
+#ifdef HAVE_POSIX_OPENPT
+	have_posix_openpt = 1;
+#else
+	have_posix_openpt = 0;
+#endif
+
 #ifdef HAVE_OPENPTY
 	have_openpty = 1;
 #else
@@ -231,16 +242,58 @@ swlib_pty_fork2(int *ptrfdm, int ofd, int ifd, int efd,
 #endif
 
 	slave_pty_name[0] = '\0';
-	if (have_openpty) {
-		fprintf(stderr, "%s: Warning: this program's usage of openpty is insecure and may allow eavesdropping\n",
-			swlib_utilname_get());
+
+	if (have_posix_openpt) {
+		E_DEBUG("");
+		pts_type = "posix_openpt";
+		ret = posix_openpt(O_RDWR|O_NOCTTY);
+		if (ret < 0) {
+			fprintf(stderr, "%s: posix_openpt failed: %s\n",
+				swlib_utilname_get(), strerror(errno));
+			return -1;
+		}
+		fdm = ret;
+		E_DEBUG2("fdm=%d", fdm);
+		tmpc = ptsname(fdm);
+		if (tmpc == NULL) {
+			fprintf(stderr, "%s: ptsname failed\n", swlib_utilname_get());
+			return -1;
+		}
+		E_DEBUG2("ptsname=%s", tmpc);
+		strncpy(slave_pty_name, tmpc, sizeof(slave_pty_name)-1);
+		slave_pty_name[sizeof(slave_pty_name)-1] = '\0';
+	} else if (have_openpty) {
+		/* SVR4 acquires controlling terminal on open() */
+		pts_type = "openpty";
+		swlib_doif_writef(swlib_get_verbose_level(),  SWC_VERBOSE_7,
+			(struct sw_logspec *)(NULL), STDERR_FILENO,
+                               "using system openpty()\n");
+		ret = my_openpty(&fdm, &fds);
+		if (ret < 0) {
+			fprintf(stderr, "%s: openpty failed\n",
+				swlib_utilname_get());
+			return -1;
+		}
+		tmpc = ttyname(fds);		
+		if (tmpc == NULL) {
+			fprintf(stderr, "%s: ttyname failed\n",
+				swlib_utilname_get());
+			return -1;
+		}
+		strncpy(slave_pty_name, tmpc, sizeof(slave_pty_name)-1);
+		slave_pty_name[sizeof(slave_pty_name)-1] = '\0';
+		swlib_doif_writef(swlib_get_verbose_level(),  SWC_VERBOSE_7,
+			(struct sw_logspec *)(NULL), STDERR_FILENO,
+                               "opened master pty [%s] using openpty\n", ttyname(fdm));
+		swlib_doif_writef(swlib_get_verbose_level(),  SWC_VERBOSE_7,
+			(struct sw_logspec *)(NULL), STDERR_FILENO,
+                               "opened slave pty [%s] using openpty\n", slave_pty_name);
 	} else {
 		/*
 		 * This is for systems that don't have openpty()
 		 */
-		fprintf(stderr, "%s: Warning: this program's pty usage is insecure and may allow eavesdropping\n",
-			swlib_utilname_get());
 
+		pts_type = "SVR4";
 		while(testfds < 0) {
 			if (swlib_get_verbose_level() >= SWC_VERBOSE_7) {
 				fprintf(stderr, "%s: pty name search index: %d\n", 
@@ -263,6 +316,14 @@ swlib_pty_fork2(int *ptrfdm, int ofd, int ifd, int efd,
 			start++;
 		}
 	}
+
+	fprintf(stderr, 
+		"%s: @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n"
+		"%s: Warning: this program's usage of pty's (%s) may be insecure.\n"
+		"%s: This feature of swcopy is meant for testing only.\n"
+		"%s: @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n" ,
+		swlib_utilname_get(), swlib_utilname_get(), pts_type, swlib_utilname_get(), swlib_utilname_get());
+
 	if (ifd < 0) ifd = STDIN_FILENO;
 	if (ofd < 0) ofd = STDOUT_FILENO;
 	if (efd < 0) efd = STDERR_FILENO;
@@ -276,31 +337,24 @@ swlib_pty_fork2(int *ptrfdm, int ofd, int ifd, int efd,
 		if (setsid() < 0) {
 			fprintf(stderr, "setsid error");
 		}
-			/* SVR4 acquires controlling terminal on open() */
-		if (have_openpty) {
-			swlib_doif_writef(swlib_get_verbose_level(),  SWC_VERBOSE_7,
-				(struct sw_logspec *)(NULL), STDERR_FILENO,
-                                "using system openpty()\n");
-			ret = my_openpty(&fdm, &fds);
-			if (ret < 0) {
-				fprintf(stderr, "%s: openpty failed\n",
-					swlib_utilname_get());
+		if (have_posix_openpt) {
+			if (grantpt(fdm) < 0 || unlockpt(fdm) < 0) {
+				fprintf(stderr, "%s: pts gramtpt or unlockpt failed\n", swlib_utilname_get());
 				return -1;
 			}
-			tmpc = ttyname(fds);		
-			if (tmpc == NULL) {
-				fprintf(stderr, "%s: ttyname failed\n",
-					swlib_utilname_get());
+			fds = open(slave_pty_name, O_RDWR);
+			if (fds < 0) {
+				fprintf(stderr, "%s: open failed on %s, reason: %s\n",
+					swlib_utilname_get(), slave_pty_name, strerror(errno));
+
+				close(fdm);
 				return -1;
 			}
-			strncpy(slave_pty_name, tmpc, sizeof(slave_pty_name)-1);
-			slave_pty_name[sizeof(slave_pty_name)-1] = '\0';
-			swlib_doif_writef(swlib_get_verbose_level(),  SWC_VERBOSE_7,
-				(struct sw_logspec *)(NULL), STDERR_FILENO,
-                                "opened master pty [%s] using openpty\n", ttyname(fdm));
-			swlib_doif_writef(swlib_get_verbose_level(),  SWC_VERBOSE_7,
-				(struct sw_logspec *)(NULL), STDERR_FILENO,
-                                "opened slave pty [%s] using openpty\n", slave_pty_name);
+			E_DEBUG2("fds=%d", fds);
+		} else if (have_openpty) {
+			/*
+			 */
+
 		} else {
 			/*
 			 * This is for systems that don't have openpty()
@@ -321,6 +375,7 @@ swlib_pty_fork2(int *ptrfdm, int ofd, int ifd, int efd,
 #if	defined(TIOCSCTTY) && !defined(CIBAUD)
 				/* 44BSD way to acquire controlling terminal */
 				/* !CIBAUD to avoid doing this under SunOS */
+		E_DEBUG("calling  TIOCSCTTY on fds");
 		if (ioctl(fds, TIOCSCTTY, (char *) 0) < 0) {
 			fprintf(stderr, 
 				"TIOCSCTTY error: %s: %s\n",
@@ -328,6 +383,7 @@ swlib_pty_fork2(int *ptrfdm, int ofd, int ifd, int efd,
 						strerror(errno));
 		}
 #endif
+		E_DEBUG("HERE 4");
 		if (slave_termios != NULL) {
 			if (tcsetattr(fds, TCSANOW, slave_termios) < 0)
 				fprintf(stderr, 
@@ -335,6 +391,7 @@ swlib_pty_fork2(int *ptrfdm, int ofd, int ifd, int efd,
 						pts_name, 
 						strerror(errno));
 		}
+		E_DEBUG("HERE 5");
 		if (slave_winsize != NULL) {
 			if (ioctl(fds, TIOCSWINSZ, slave_winsize) < 0)
 				fprintf(stderr, 
@@ -342,6 +399,7 @@ swlib_pty_fork2(int *ptrfdm, int ofd, int ifd, int efd,
 						pts_name,
 						strerror(errno));
 		}
+		E_DEBUG("HERE 6");
 
 		c_pid = swndfork(sigmask, NULL);
 		if (c_pid < 0) {
@@ -360,6 +418,7 @@ swlib_pty_fork2(int *ptrfdm, int ofd, int ifd, int efd,
 				"write error : %s \n", strerror(errno));
 			}
 			close(pcpipe[1]);
+			E_DEBUG("at parent");
 			if (dup2(fds, 0) != 0)
 				fprintf(stderr, "dup2 error to ifd = %d", ifd);
 			if (dup2(fds, 1) != 1)
@@ -377,16 +436,18 @@ swlib_pty_fork2(int *ptrfdm, int ofd, int ifd, int efd,
 			close(op[0]);
 			close(ip[1]);
 			if (make_raw)
+
 			if (swlib_tty_raw(fdm) < 0) {
 				fprintf(stderr, 
 				"tty_raw error : source_fdar[0]\n");
 				exit(2);
 			}
+			E_DEBUG("at swgp_stdioPump");
 			ret = swgp_stdioPump(
-				op[1],
-				fdm,
-				fdm,
-				ip[0],
+				op[1], /* stdout */
+				fdm,   /* output from remote host */
+				fdm,   /* input to remote host */
+				ip[0], /* stdin */
 				0 /* verbose */,
 				1 /* be_silent */,
 				0, NULL, NULL, NULL, &sb);
