@@ -17,17 +17,10 @@
 #define SWFORK_FORK				"fork"
 #define SWFORK_NO_FORK				"no-fork"
 
-/* int swlib_ptym_open(char *pts_name, int *start); */
-/* int swlib_ptys_open(int fdm, char *pts_name); */
 pid_t swlib_fork(char * type, int *ptrfdm, int ofd, int ifd, int efd, 
 	const struct termios *slave_termios, 
 	const struct winsize *slave_winsize,
 		pid_t * pump_pid, int make_raw, sigset_t * blockset);
-/*
-pid_t swlib_pty_fork(int *ptrfdm, int ofd, int ifd, int efd,
-	const struct termios *slave_termios, 
-		const struct winsize *slave_winsize, sigset_t * blockset);
-*/
 pid_t swlib_pty_fork2(int *ptrfdm, int ofd, int ifd, int efd, 
 	const struct termios *slave_termios, 
 		const struct winsize *slave_winsize, int * pump_pid, int make_raw, sigset_t * blockset);
