@@ -371,7 +371,7 @@ swgpg_set_status_array(SWGPG_VALIDATE * w)
 	strob_strcpy(w->status_arrayM, "");
         i = 0;
         while((sig=strar_get(w->list_of_sigsM, i)) != NULL) {
-                ret = swgpg_determine_signature_status(strar_get(w->list_of_status_blobsM, i));
+                ret = swgpg_determine_signature_status(strar_get(w->list_of_status_blobsM, i), -1);
                 swgpg_set_status(w, i, ret);
                 i++;
         }
@@ -467,7 +467,7 @@ swgpg_remove_fifo(void)
 }
 
 int
-swgpg_determine_signature_status(char * gpg_status_lines)
+swgpg_determine_signature_status(char * gpg_status_lines, int which_sig)
 {
 	STROB * tmp;
 	char * line;
@@ -476,7 +476,13 @@ swgpg_determine_signature_status(char * gpg_status_lines)
 	int nodata;
 	int nokey;
 
-	E_DEBUG("");
+	E_DEBUG("----------------------------------");
+	E_DEBUG("----------------------------------");
+	
+	E_DEBUG2("%s", gpg_status_lines);
+
+	E_DEBUG("----------------------------------");
+	E_DEBUG("----------------------------------");
 	tmp = strob_open(100);
 
 	good_count=0;
@@ -487,23 +493,29 @@ swgpg_determine_signature_status(char * gpg_status_lines)
 	E_DEBUG("");
         while(line) {
 		/* fprintf(stderr, "%s\n", line); */
-		E_DEBUG("");
+		E_DEBUG2("LINE=[%s]", line);
 		if (strstr(line, GPG_STATUS_PREFIX) == line) {
 			if (strstr(line, GPG_STATUS_PREFIX GPG_STATUS_GOODSIG)) {
+				E_DEBUG("good_count++");
 				good_count++;
 			} else if (strstr(line, GPG_STATUS_BADSIG)) {
+				E_DEBUG("bad_count++");
 				bad_count++;
 			} else if (strstr(line, GPG_STATUS_NO_PUBKEY)) {
+				E_DEBUG("nokey++");
 				nokey++;
 			} else if (strstr(line, GPG_STATUS_EXPSIG)) {
+				E_DEBUG("bad_count++");
 				bad_count++;
 			} else if (strstr(line, GPG_STATUS_NODATA)) {
+				E_DEBUG("no_data++");
 				nodata++;
 			} else {
 				; /* allow all other lines ?? */
-			} 
+			}
 		} else {
 			;
+			E_DEBUG("do_nothing");
 			/* now stderr is mixed this output so just
 			   ignore other lines */
 			/* return SWGPG_SIG_ERROR; */
@@ -512,16 +524,23 @@ swgpg_determine_signature_status(char * gpg_status_lines)
         }
 	E_DEBUG("");
 	strob_close(tmp);
-	if (good_count == 1 && bad_count == 0 && nokey == 0 && nodata == 0) {
+	if (
+		(which_sig < 0 && good_count >= 1 && bad_count == 0 && nokey == 0 && nodata == 0) ||
+		(which_sig >= 0 && good_count == 1 && bad_count == 0 && nokey == 0 && nodata == 0)
+	) {
+		E_DEBUG("return SWGPG_SIG_VALID");
 		return SWGPG_SIG_VALID;
 	} else if (good_count == 0 && bad_count == 0 && nokey > 0) {
+		E_DEBUG("return SWGPG_SIG_NO_PUBKEY");
 		return SWGPG_SIG_NO_PUBKEY;
 	} else if (good_count == 0 && bad_count == 0 && nodata > 0) {
+		E_DEBUG("return SWGPG_SIG_NODATA");
 		return SWGPG_SIG_NODATA;
 	} else {
+		E_DEBUG("return SWGPG_SIG_NOT_VALID");
 		return SWGPG_SIG_NOT_VALID;
 	}
-	E_DEBUG("");
+	E_DEBUG("return SWGPG_SIG_ERROR");
 	return SWGPG_SIG_ERROR;
 }
 
@@ -541,6 +560,15 @@ swgpg_run_checksig2(char * sigfilename, char * thisprog,
 	SHCMD * cmd[3];
 	int u_verbose = uverbose;
 	STROB * tmp = strob_open(30);
+	int atoiret=1;
+	int which_sig;
+
+	if (which_sig_arg) {
+		which_sig = swlib_atoi(which_sig_arg, &atoiret);
+		if (atoiret) return -3;  /* never should happen */
+	} else {
+		which_sig = -1;
+	}
 
 	cmd[0] = shcmd_open();
 	cmd[1] = shcmd_open();
@@ -605,7 +633,7 @@ swgpg_run_checksig2(char * sigfilename, char * thisprog,
 	swlib_exec_filter(cmd+1, sbfd[0], tmp);
 	E_DEBUG("");
 
-	ret = swgpg_determine_signature_status(strob_str(tmp));
+	ret = swgpg_determine_signature_status(strob_str(tmp), which_sig);
 	retval = ret;
 	E_DEBUG2("signature status is %d", ret);
 
@@ -637,6 +665,7 @@ out:
 	if (cmd[1]) shcmd_close(cmd[1]);
 	close(sbfd[0]);
 	strob_close(tmp);
+	E_DEBUG2("retval=%d", retval);
 	return retval;
 }
 
@@ -706,7 +735,7 @@ swgpg_run_gpg_verify(SWGPG_VALIDATE * swgpg, int signed_bytes_fd, char * signatu
 
 	E_DEBUG2("swlib_exec_filter ret1=%d", ret1);
 
-	ret2 = swgpg_determine_signature_status(strob_str(output));
+	ret2 = swgpg_determine_signature_status(strob_str(output), -1);
 	E_DEBUG2("swgpg_determine_signature_status ret2=%d", ret2);
 
 	ret3 = waitpid(pid, &status, 0);
