@@ -31,9 +31,11 @@ int swdef_write_value_to_buffer (STROB * ob, char * value);
 int swdef_write_keyword (char * keyword, int level, int attribute_type, int uxfio_fd);
 int swdef_write_keyword_to_buffer (STROB * ob, char * keyword, int level, int attribute_type);
 int swdef_write_attribute (char * keyword, char * value, int level, int len, int attribute_type, int uxfio_fd);
+int swdef_write_attribute_to_buffer(STROB * buf, char *keyword, char *value, int level, int attribute_type);
 
 char *  swheaderline_get_type_pointer		(char *output_line);
 int     swheaderline_write			(char * line, int uxfio_fd);
+int     swheaderline_write_to_buffer		(STROB * buf, char * line);
 int     swheaderline_write_debug		(char * line, int uxfio_fd);
 int     swheaderline_get_level			(char * outputline);
 void    swheaderline_set_level			(char * outputline, int level);
