@@ -454,7 +454,18 @@ swicat_e_open_catalog_tarball(int cfd)
 	E_DEBUG2("swvarfs=%p", (void*)swvarfs);
 	if (swvarfs == NULL) return NULL;
 
-        while ((path=swvarfs_get_next_dirent(swvarfs, &st)) != NULL && strlen(path)) {
+        while (
+		(path=swvarfs_get_next_dirent(swvarfs, &st)) != NULL &&
+		strlen(path) &&
+		/*
+ 		 * taru_read_in_tar_header2() sets this TRAILER!!! on detecting NUL blocks
+ 		 * in a tar archive.  Out of defererence to cpio, continue this.
+ 		 * Somehow it worked, but when taru_read_in_tar_header2() was made
+ 		 * to read both trailer nul blocks (c.March2014) a hang occured here
+ 		 * and hence discovery of this long standing bug.
+ 		 */
+		strcmp(CPIO_INBAND_EOA_FILENAME, path)
+	) {
 		E_DEBUG2("new entry: %s", path);
 		e = swicat_e_create();
 		vplob_add(list, e);
@@ -545,13 +556,17 @@ swicat_e_isf_parse(SWICAT_E * e, int *pstatus, AHS * ahs)
 	int datalen;
 	int ret;
 
+	E_DEBUG("BEGIN");
 	tmp = strob_open(12);
 	if (pstatus) *pstatus = 0;
 	ret = swicat_e_find_attribute_file(e, SW_A_INSTALLED, tmp, ahs);
+	E_DEBUG2("swicat_e_find_attribute_file ret=[%d]", ret);
 	if (ret != 0) {
 		/* fatal error */
+		E_DEBUG("ret != 0");
 		SWLIB_ERROR("");
 		if (pstatus) *pstatus = 1;
+		E_DEBUG("returning NULL");
 		return NULL;
 	} 
 
@@ -561,7 +576,9 @@ swicat_e_isf_parse(SWICAT_E * e, int *pstatus, AHS * ahs)
 					      we have to put it back */
 	uxfio_lseek(ifd, (off_t)0, SEEK_SET);
 
+	E_DEBUG2("running sw_yyparse on fd=%d", ofd);
 	ret = sw_yyparse(ifd, ofd, SW_A_INSTALLED, 0, SWPARSE_FORM_MKUP_LEN);
+	E_DEBUG2("sw_yyparse: ret=[%d]", ret);
 	if (ret) {
 		fprintf(stderr,
 		"%s: error parsing %s\n", swlib_utilname_get(), SW_A_INSTALLED);
@@ -586,14 +603,12 @@ swicat_e_make_file_list(SWICAT_E * e, SWUTS * swuts, SWI * swi)
 {
 	int ret;
 	STROB * tmp;
-	STROB * relocated_path;
 	SWI_FILELIST * flist;
 	char * location;
 
 	E_DEBUG("");
 	flist = NULL;
 	tmp = strob_open(12);
-	relocated_path = strob_open(12);
 
 	if (swi == NULL) {
 		SWLIB_ERROR("");
@@ -604,7 +619,11 @@ swicat_e_make_file_list(SWICAT_E * e, SWUTS * swuts, SWI * swi)
 		SWLIB_ERROR("");
 		goto out;
 	}
-	
+
+	/*
+ 	 * find the location attribute, which is stored as a separate file
+ 	 * because it is a version attribute
+ 	 */	
 	E_DEBUG("");
 	ret = swicat_e_find_attribute_file(e, SW_A_location, tmp, NULL);
 	if (ret < 0) {
@@ -617,11 +636,11 @@ swicat_e_make_file_list(SWICAT_E * e, SWUTS * swuts, SWI * swi)
 		strob_strcpy(tmp, "/");
 	}
 	location = strdup(strob_str(tmp));
+
 	flist = swi_make_file_list(swi, swuts, location, SW_FALSE);
 out:
 	unfix_offset(e);
 	strob_close(tmp);
-	strob_close(relocated_path);
 	return flist;
 }
 
@@ -939,8 +958,11 @@ swicat_e_form_catalog_path(SWICAT_E * e, STROB * buf, char * isc_path, int activ
 	if (swlib_check_clean_path(p)) return NULL;
 
 	if (isc_path) {
-		risc = swlib_return_relative_path(isc_path);
-		strob_strcpy(buf, risc);
+		/* risc = swlib_return_relative_path(isc_path); */
+		/* the path has had the file:/// URL syntax processed, hence
+		 * if it is absolute it really should be absolute
+		 */
+		strob_strcpy(buf, isc_path);
 	} else {
 		strob_strcpy(buf, "");
 	}
