@@ -101,6 +101,10 @@
 
 #define SWBIS_PGM_SH		"sh"
 #define SWBIS_PGM_BIN_SH	"/bin/sh"
+#define SWBIS_PGM_BIN_KSH	"/bin/ksh"
+#define SWBIS_PGM_BIN_MKSH	"/bin/mksh"
+#define SWBIS_PGM_BIN_BASH	"/bin/bash"
+#define SWBIS_PGM_XPG4_SH	"/usr/xpg4/bin/bash"
 #define SWBIS_LOGGER_SIGTERM		SIGUSR2
 #define SWBIS_TAINTED_CHARS		"'\"|*?;&<>`$[]"
 #define SWBIS_WS_TAINTED_CHARS		"\n\r\t "
@@ -130,6 +134,11 @@
 #define SWBN_SIGNATURE	"signature"
 #define SWBN_SIG_HEADER	"sig_header"
 
+#define SWBIS_A_ustar	"ustar"
+#define SWBIS_A_pax	"pax"
+#define SWBIS_A_posix	"posix"
+#define SWBIS_A_gnu	"gnu"
+
 #define SW_ROOT_DIR "/"
 #define SWBIS_SYNCT_EOF			CPIO_INBAND_EOA_FILENAME "\r\n\r\n"
 
@@ -157,18 +166,18 @@
 #include	<sys/ioctl.h>	/* 44BSD requires this too */
 #include "swevents.h"
 
+#define SYNCT_EOF_CONDITION_0	 0
+#define SYNCT_EOF_CONDITION_1	 1
+#define SYNCT_EOF_CONDITION_2	 2   /* mangled trailer */
 
 typedef struct {
+	int do_debugM;  /* normally not used, should be -1 */
+	int debugfdM;   /* normally not used, should be -1 */ 
 	unsigned char * bufM;
 	unsigned char * tailM;
 	char * trailer_startM;
 	int countM;
-	/* int firstM; */
-	/* int remainsM; */
-        char * mtM;   /* TRAILER string */
-        /* char * mtpM; */  /* position in trailer string */
-        /* int mt_nullsM; */ /* Number of NULs interspersed trailer string */
-	/* int mt_spanM; */  /* block span of trailer string */
+        char * mtM;   /* TRAILER!!!\r\n\r\n string */
 } SYNCT;
 
 /* void 		* swlib_realloc(void * ptr, size_t newsize, size_t oldsize); */
@@ -226,7 +235,8 @@ void 		swlib_assertion_fatal(int assertion_result, char * reason, char * file, i
 void		swlib_swprog_assert(int assertion_result, int status, char * version, char * reason, char * file, int line, char * function);
 int 		swlib_write_signing_files(XFORMAT * package, int ofd, int which_file /* 0=catalog  1=storage */, int do_adjunct_md5);
 int		swlib_atoi(const char *nptr, int * result);
-unsigned long int swlib_atoul(const char *nptr, int * result);
+int 		swlib_atoi2(const char *nptr, char ** fp_endptr, int * result);
+unsigned long int swlib_atoul(const char *nptr, int * result, char ** endptr);
 int 		swlib_unix_dircat(STROB * dest, char * dirname);
 int 		swlib_dir_compare(char * s1, char * s2, int compare_flag /* See SWC_FC_* */);
 int		swlib_basename_compare(char * s1, char * s2);
@@ -240,6 +250,7 @@ char * 		swlib_basename(STROB * dest, char * source);
 int		swlib_kill_all_pids(pid_t * pid, int num, int signo, int debug);
 int 		swlib_wait_on_all_pids(pid_t * pid, int num, int * status, int flags, int debug);
 int 		swlib_wait_on_all_pids_with_timeout(pid_t * pid, int num, int * status, int flags, int debug, int tm);
+int		swlib_wait_on_pid_with_timeout(pid_t pid, int * status, int flags, int verbose_level, int fp_tmo);
 int 		swlib_update_pid_status(pid_t keypid, int value, pid_t * pid, int * status, int len);
 int		swlib_sha1(int uxfio_fd, char *digest);
 int		sha_stream (int ifd, void *resblock);
@@ -303,6 +314,9 @@ int 		swlib_unix_dirtrunc_n(STROB * buf, int n);
 void 		swlib_squash_illegal_tag_chars(char * s);
 intmax_t 	swlib_digs_copy(int ofd, int ifd, intmax_t count, FILE_DIGS * digs, int);
 void 		std_quicksort (void *const pbase, size_t total_elems, size_t size, int  (*)(const void *, const void *));
-mode_t		 swlib_filestring_to_mode(char * str, int do_include_type);
-void swlib_append_synct_eof(STROB * buf);
+mode_t		swlib_filestring_to_mode(char * str, int do_include_type);
+void		swlib_append_synct_eof(STROB * buf);
+pid_t		swlib_get_pax_header_pid(void);
+void		swlib_set_pax_header_pid(pid_t pd);
+
 #endif
