@@ -54,6 +54,8 @@ struct fileChain {
 
 #define SWVARFS_S_IFDIR	0040000
 #define SWVARFS_S_IFREG	0100000
+#define SWVARFS_VSTAT_LSTAT		"lstat"
+#define SWVARFS_VSTAT_STAT		"stat"
 
 struct dirContext {		/* Stack for directory-tree walking routine. */
 	struct stat * statbuf;
@@ -115,6 +117,7 @@ typedef struct {
 	int (*f_statM)(char * path, struct stat *);
 	HLLIST *link_recordM;		/* Hard link list object */
 	TARU * taruM;
+	int n_loop_symlinksM;		/* number of allowable symlinks */
 } SWVARFS;
 
 /* ++++  Public API  +++++ */
@@ -151,6 +154,7 @@ void 		swvarfs_debug_dump_filechain(SWVARFS * swvarfs);
 /*D char *		swvarfs_dump_string_s(SWVARFS * swvarfs, char * prefix); */
 int 		swvarfs_debug_list_files(SWVARFS * swvarfs, int ofd);
 void		swvarfs_set_stat_syscall(SWVARFS * swvarfs, char * s);
+char *		swvarfs_get_stat_syscall(SWVARFS * swvarfs);
 HLLIST * 	swvarfs_get_hllist	(SWVARFS * swvarfs);
 UINFORMAT * 	swvarfs_get_uinformat	(SWVARFS *swvarfs);
 int 		swvarfs_get_tarheader_flags(SWVARFS * xux);
