@@ -2,7 +2,7 @@
  */
 
 /*
- * Copyright (C) 1999,2007,2008  James H. Lowe, Jr. 
+ * Copyright (C) 1999,2007,2008,2014  James H. Lowe, Jr. 
  * 
  * COPYING TERMS AND CONDITIONS:
  *
@@ -57,6 +57,10 @@
 
 static void usage (char * progname, FILE*);
 
+
+static int G_OFD;
+
+
 YYSTYPE yylval;
 int yydebug=0;
 int swlex_definition_file = SW_PSF;
@@ -75,7 +79,7 @@ void
 copyright_info(FILE * fp)
 {
 	fprintf(fp,  "%s",
-	"Copyright (C) 2003,2004,2005,2007 Jim Lowe\n"
+	"Copyright (C) 2003,2004,2005,2007,2014 Jim Lowe\n"
 	"This software is distributed under the terms of the GNU General Public License\n"
 	"and comes with NO WARRANTY to the extent permitted by law.\n"
 	"See the file named COPYING for details.\n");
@@ -110,11 +114,14 @@ void  usage (char * progname, FILE * fp) {
     fprintf (fp, "   -x, --use-recursive-fileset Use \"file *\" instead of individual file definitions.\n");
     fprintf (fp, "   -r, --no-leading-path   use \".\" as the leading archive path\n");
     fprintf (fp, "   -B buffertype   mem or file  Mem is default.\n");
-    fprintf (fp, "   -H format     archive format: ustar, newc, crc, odc\n");
+    fprintf (fp, "   -H format     archive format: ustar, pax, newc, crc, odc (pax is default)\n");
     fprintf (fp, "   -p, --psf-only   Write the PSF to stdout.\n");
     fprintf (fp, "   -o, --info-only   Write the INFO file for the rpm archive.\n");
     fprintf (fp, "   -r, --no-leading-path.\n");
     fprintf (fp, "   -L, --smart-leading-path  Use leading path for srpm and not rpm.\n");
+    fprintf (fp, "   --construct-missing-files  Applies to RPM translation. Construct zero length\n");
+    fprintf (fp, "                   files for header entries that have no files in the archive.\n");
+    fprintf (fp, "   -v verbose messages about the translation.\n");
     fprintf (fp, 
 		 "\n"
 		 "Howto use with GNU swpackage:\n"
@@ -193,7 +200,7 @@ write_slack(TOPSF * topsf)
 	uxfio_lseek(psffd, 0, SEEK_SET);
 	E_DEBUG("");
 
-	nfd = STDOUT_FILENO;
+	nfd = G_OFD;
 	ret = uxfio_fcntl(topsf->fd_, UXFIO_F_ARM_AUTO_DISABLE, 1);
 	if (ret < 0) {
 		E_DEBUG("error");
@@ -212,7 +219,7 @@ write_slack(TOPSF * topsf)
 	/* Next, write the PSF file */
 
 	taru = taru_create();
-	etar = etar_open(taru);
+	etar = etar_open(taru->taru_tarheaderflagsM);
 	etar_init_hdr(etar);
 	etar_set_size_from_fd(etar, psffd, (int*)NULL);
 	etar_set_pathname(etar, "PSF");
@@ -259,7 +266,7 @@ write_plain_source_tarball(TOPSF * topsf)
 	uxfio_lseek(psffd, 0, SEEK_SET);
 	E_DEBUG("");
 
-	nfd = STDOUT_FILENO;
+	nfd = G_OFD;
 	ret = uxfio_fcntl(topsf->fd_, UXFIO_F_ARM_AUTO_DISABLE, 1);
 	if (ret < 0) {
 		E_DEBUG("error");
@@ -279,7 +286,7 @@ write_plain_source_tarball(TOPSF * topsf)
 
 	E_DEBUG("");
 	taru = taru_create();
-	etar = etar_open(taru);
+	etar = etar_open(taru->taru_tarheaderflagsM);
 
 	/* Next, write the PSF file */
 	etar_init_hdr(etar);
@@ -338,7 +345,7 @@ write_plain_source_tarball(TOPSF * topsf)
 	}
 
 	E_DEBUG("");
-	xformat_set_ofd(xformat, STDOUT_FILENO);
+	xformat_set_ofd(xformat, G_OFD);
 
 	E_DEBUG("");
 	while (
@@ -367,7 +374,7 @@ write_plain_source_tarball(TOPSF * topsf)
 		}
 	}
 	xformat_close(xformat);
-	etar_write_trailer_blocks(NULL, STDOUT_FILENO, 2);
+	etar_write_trailer_blocks(NULL, G_OFD, 2);
 
 	/*
 	ret = swlib_pipe_pump(nfd, topsf->fd_);
@@ -399,7 +406,7 @@ write_deb(TOPSF * topsf)
 	AHS * ahs;
 	char * name;
 
-	ofd = STDOUT_FILENO;
+	ofd = G_OFD;
 	tmpname = strob_open(32);
 	psffd = swlib_open_memfd();
 	ret = topsf_write_psf(topsf, psffd, 1 /* not used */);
@@ -431,7 +438,7 @@ write_deb(TOPSF * topsf)
 
 	/* write the PSF file */
 	taru = taru_create();
-	etar = etar_open(taru);
+	etar = etar_open(taru->taru_tarheaderflagsM);
 	etar_init_hdr(etar);
 	etar_set_size_from_fd(etar, psffd, (int*)NULL);
 	etar_set_pathname(etar, "PSF");
@@ -477,7 +484,7 @@ write_deb(TOPSF * topsf)
 	data_xformat = xformat_open(-1, -1, arf_ustar);
 	ret = xformat_open_archive_by_fd(data_xformat, data_fd,
 		UINFILE_DETECT_OTARFORCE|UINFILE_DETECT_DEB_CONTEXT, (mode_t)(0));
-	xformat_set_ofd(data_xformat, STDOUT_FILENO);
+	xformat_set_ofd(data_xformat, G_OFD);
 	
 	while ((ret=xformat_read_header(data_xformat)) > 0) {
 		if (xformat_is_end_of_archive(data_xformat)) {
@@ -533,6 +540,7 @@ main(int argc, char **argv)
 	int info_only = 0;
 	int psf_only = 0;
 	int tarpipe[2];
+	int verbose;
 	char * owner;
 	char * group;
 	char * progname;
@@ -552,9 +560,11 @@ main(int argc, char **argv)
 	int do_deb_data = 0;
 	char * debug_level = NULL;
 	unsigned long create_time = 0;
+	int construct_missing_files = 0;
 
 	progname = argv[0];
-
+	G_OFD = STDOUT_FILENO;
+	verbose = 1;
 	exclude_list = NULL;
 	yylval.strb = strob_open (8);
 	strcpy(swlex_filename,"none");
@@ -594,10 +604,12 @@ main(int argc, char **argv)
                {"slackware-pkg-name", 1, 0, 204},
                {"exclude", 1, 0, 205},
                {"exclude-system-dirs", 0, 0, 206},
+               {"construct-missing-files", 0, 0, 207},
+               {"verbose", 0, 0, 'v'},
                {0, 0, 0, 0}
              };
 
-             c = ugetopt_long (argc, argv, "ArB:pH:xD:oLM", long_options, &option_index);
+             c = ugetopt_long (argc, argv, "ArB:pH:xD:oLMv", long_options, &option_index);
              if (c == -1)
                  break;
 
@@ -615,6 +627,9 @@ main(int argc, char **argv)
                case 'r':
                  no_leading_path=1;        
 		 break;
+               case 'v':
+                 verbose++;        
+		 break;
                case 'x':
 		 /* Use recursive fileset spec for binary packages */
                  use_recursive_fileset_spec=1;
@@ -680,9 +695,10 @@ main(int argc, char **argv)
 			}
 			if (uinfile_get_type(uinformat) != DEB_FILEFORMAT) exit(1);
 			if (debfd < 0) exit(1);
-			ret = swlib_pipe_pump(STDOUT_FILENO, debfd);
+			ret = swlib_pipe_pump(G_OFD, debfd);
 			uxfio_close(debfd);
 			uinfile_close(uinformat);
+			close(G_OFD);
 			if (ret < 0)
 				exit(1);
 			else
@@ -711,9 +727,10 @@ main(int argc, char **argv)
 			if (uinfile_get_type(uinformat) != DEB_FILEFORMAT) exit(1);
 			/* this pumps the data.tar to stdout */
 			fd = ret;
-			ret = swlib_pipe_pump(STDOUT_FILENO, fd);
+			ret = swlib_pipe_pump(G_OFD, fd);
 			uxfio_close(fd);
 			uinfile_close(uinformat);
+			close(G_OFD);
 			if (ret < 0)
 				exit(1);
 			else
@@ -738,6 +755,9 @@ main(int argc, char **argv)
 			if (exclude_list == NULL) exclude_list = strar_open();
 			add_system_dirs(exclude_list);
 	         	break;
+               case 207:
+			construct_missing_files = 1;
+	         	break;
                default:
                  usage(progname, stderr);
 		 exit(1);
@@ -750,6 +770,15 @@ main(int argc, char **argv)
 		/* Open a regular file by name */
 		E_DEBUG("");
 		topsf = topsf_open(argv[optind], bufferflag, NULL);
+		if (optind+1 < argc) {
+			G_OFD = open(argv[optind+1], O_RDWR|O_CREAT|O_TRUNC, 0644);
+			if (G_OFD < 0) {
+				fprintf (stderr, "lxpsf: error opening %s: %s\n", argv[optind+1], strerror(errno));
+				exit(1);
+			}
+		}
+
+
 	} else {
 		E_DEBUG("");
 		topsf = topsf_open("-", bufferflag, NULL);	/* stdin */
@@ -758,8 +787,11 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
+	topsf->verboseM = verbose;
 	topsf->exclude_listM = exclude_list;
 
+	topsf->rpm_construct_missing_filesM = construct_missing_files;
+
 	if (owner)
 		topsf->ownerM = strdup(owner);
 	if (group)
@@ -811,7 +843,7 @@ main(int argc, char **argv)
 		topsf->debug_link_ = 1;
 	}
 
-	if (!strcmp(format,"ustar")) {
+	if (!strcmp(format, SWBIS_A_ustar) || !strcmp(format, SWBIS_A_pax) ) {
 		topsf->reverse_links_ = 1;
 	}
 
@@ -821,7 +853,7 @@ main(int argc, char **argv)
 
 	if (psf_only){
 		E_DEBUG("");
-		ret = topsf_write_psf(topsf, STDOUT_FILENO, 1 /* do indent */);
+		ret = topsf_write_psf(topsf, G_OFD, 1 /* do indent */);
 		E_DEBUG2("(topsf_write_psf) ret=%d", ret);
 		
 		if (uinfile_get_type(topsf->format_desc_) == SLACK_FILEFORMAT) {
@@ -832,7 +864,7 @@ main(int argc, char **argv)
 	} else if (info_only){
 		E_DEBUG("");
 		topsf->info_only_ = 1;
-		ret=topsf_write_info(topsf, STDOUT_FILENO, 1 /* do indent */);
+		ret=topsf_write_info(topsf, G_OFD, 1 /* do indent */);
 	} else {
 		if (uinfile_get_type(topsf->format_desc_) == DEB_FILEFORMAT) {
 			ret = write_deb(topsf);
@@ -872,10 +904,11 @@ main(int argc, char **argv)
 			/* now the file link lists are generated.  
 			 */
 
-			if (strcmp(format,"ustar") == 0) {
+			if (strcmp(format, SWBIS_A_ustar) == 0 || strcmp(format, SWBIS_A_pax) == 0) {
 				/*
 				* Convert output format to ustar.
 				*/
+				TARU * taru;
 				pipe(tarpipe);
 				pid = swfork((sigset_t*)(NULL));
 				if (pid < 0) {
@@ -889,10 +922,12 @@ main(int argc, char **argv)
 					_exit(ret);
 				} else {
 					close(tarpipe[1]);
-					ret = taru_process_copy_in(tarpipe[0], STDOUT_FILENO);	
+					taru = taru_create();
+					taru_set_tar_header_policy(taru, SWBIS_A_pax, NULL);
+					ret = taru_process_copy_in(taru, tarpipe[0], G_OFD);	
 				}
 			} else {
-				ret=topsf_copypass_swacfl_list(topsf, STDOUT_FILENO);
+				ret=topsf_copypass_swacfl_list(topsf, G_OFD);
 			}
 		} else if (uinfile_get_type(topsf->format_desc_) == PLAIN_TARBALL_SRC_FILEFORMAT) {
 			ret = write_plain_source_tarball(topsf);
@@ -903,6 +938,7 @@ main(int argc, char **argv)
 		E_DEBUG("");
 	}
 	E_DEBUG("");
+	if (G_OFD != STDOUT_FILENO) close(G_OFD);
 	topsf_close(topsf);	
 	exit(ret ? 1 : 0);
 }
