@@ -6,6 +6,8 @@
 #include <sys/wait.h>
 #include "taru.h"
 #include "swparser_global.h"
+#include "swevents_array.h"
+#include "swcommon_options.h"
 
 int 
 main  {
