@@ -2,7 +2,7 @@
  */
 
 /* 
- Copyright (C) 1995,1996,1997,1998,2000,2001,2005  James H. Lowe, Jr.
+ Copyright (C) 1995,1996,1997,1998,2000,2001,2005,2014  James H. Lowe, Jr.
  All rights reserved.
 
  This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,7 @@
 			/* MEM_CACHE defers free() of strob objects until the
 			   program exits, it maintains a list of active and unactive
 			   objects.  It appears to result in *no* performance gains */
-#undef S_MEM_CACHE
+#undef S_MEM_CACHE     /* default */
 
 static
 int 
@@ -60,7 +60,8 @@ sb_open(size_t initial_size)
 	strb->length_ = 0;
 	strb->str_ = (unsigned char*)NULL;
 	strb->in_use_ = 1;
-	strob_reopen(initial_size + 1, strb);
+	strb->fill_charM = '\0';
+	strob_reopen(strb, initial_size + 1);
 	return strb;
 }
 
@@ -131,7 +132,7 @@ bl_open(size_t initial_size)
 		if (s) bl_add(s);
 	} else {
 		s->in_use_ = 1;
-		strob_reopen(initial_size + 1, s);
+		strob_reopen(s, initial_size + 1);
 	}
 	return s;
 }
@@ -151,10 +152,20 @@ bl_close(STROB * s)
 
 static
 STROB *
-strob_reopen_if(size_t reqd_length, STROB * strb)
+strob_reopen_if_fill_with(STROB * strb, size_t reqd_length, int fill_char)
 {
 	if ((int)reqd_length > strb->reserve_){
-		return strob_reopen(reqd_length + strb->extra_, strb);
+		return strob_reopen_fill_with(strb, reqd_length + strb->extra_, fill_char);
+	}
+	return strb;
+}
+
+static
+STROB *
+strob_reopen_if(STROB * strb, size_t reqd_length)
+{
+	if ((int)reqd_length > strb->reserve_){
+		return strob_reopen(strb, reqd_length + strb->extra_);
 	}
 	return strb;
 }
@@ -163,7 +174,7 @@ static
 void *
 strob_set_up(STROB *strb, int n)
 {
-	if (!strob_reopen_if(n+1, strb))
+	if (!strob_reopen_if(strb, n+1))
 		return (char *)(NULL);
 
 	if (n > strb->length_) {
@@ -203,7 +214,7 @@ strob_release(STROB * strb)
 	strb->length_ = 0;
 	strb->str_ = (unsigned char*)NULL;
 	strb->in_use_ = 1;
-	strob_reopen(STROB_INITLENGTH + 1, strb);
+	strob_reopen(strb, STROB_INITLENGTH + 1);
 	strob_close(strb);
 #else
 	swbis_free(strb);
@@ -219,7 +230,7 @@ strob_set_reserve(STROB * strb, int res)
 }
 
 STROB *
-strob_reopen(size_t new_length, STROB * strb)
+strob_reopen_fill_with(STROB * strb, size_t new_length, int fill_char)
 {
 	unsigned char *tmpstr;
 
@@ -243,13 +254,17 @@ strob_reopen(size_t new_length, STROB * strb)
 	strb->reserve_ = new_length;
 	if (strb->reserve_ > strb->length_) {
 		memset(strb->str_ + strb->length_,
-				(int)'\0', strb->reserve_ - strb->length_);
-	} else {
-		strb->str_[new_length - 1]='\0';
+				(int)fill_char, strb->reserve_ - strb->length_);
 	}
+	strb->str_[new_length - 1]='\0';
 	return strb;
 }
 
+STROB *
+strob_reopen(STROB * strb, size_t new_length)
+{
+	return strob_reopen_fill_with(strb, new_length, (int)'\0');
+}
 
 o__inline__
 char *
@@ -276,7 +291,7 @@ o__inline__
 STROB *
 strob_trunc(STROB * strb)
 {
-	return strob_reopen(STROB_INITLENGTH + 1, strb);
+	return strob_reopen(strb, STROB_INITLENGTH + 1);
 }
 
 void
@@ -290,31 +305,22 @@ char *
 strob_strcpy_at_offset(STROB * strb, int offset, char *str)
 {
 	STROB *strb_ret;
-	strb_ret = strob_reopen_if(strlen(str) + offset + 1, strb);
+	strb_ret = strob_reopen_if(strb, strlen(str) + offset + 1);
 	if (!strb_ret)
 		return (char *)(NULL);
 
 	strb->length_ = offset + strlen(str);	
 	memmove(strb->str_ + offset, str, strlen(str) + 1);
-	/* return strcpy(strb->str_ + offset, str); */
 	return (char*)(strb->str_ + offset);
 }
 
 void
 strob_chr_index(STROB * strb, int index, int ch)
 {
-	strob_reopen_if(index + 2, strb);
-	/*
-		replaced with code below 2010-08-01
-	if (index >= (int)strlen((unsigned char*)(strb->str_))) {
-		memset(strb->str_ + strlen((unsigned char*)(strb->str_)),
-				'\0', index - strlen((unsigned char*)(strb->str_)) + 1);
-		*(strb->str_ + index + 1) = '\0';
-	}
-	*/
+	strob_reopen_if_fill_with(strb, index + 2, strb->fill_charM);
 
 	if (index >= strb->length_) {
-		memset(strb->str_ + strb->length_, '\0', index -  strb->length_ + 1);
+		memset(strb->str_ + strb->length_, strb->fill_charM, index -  strb->length_ + 1);
 		*(strb->str_ + index + 1) = '\0';
 	}
 
@@ -333,6 +339,19 @@ strob_get_char(STROB * strb, int index)
 	return (int) strb->str_[index];
 }
 
+char *
+strob_strcat_at_offset(STROB * strb, int offset, char *str)
+{
+	STROB *strb_ret;
+	strb_ret = strob_reopen_if(strb, strlen(str) + offset + 1 + strlen((char*)(strb->str_ + offset)));
+	if (!strb_ret)
+		return (char *)(NULL);
+
+	strb->length_ = offset + strlen(str);	
+	memmove(strb->str_ + offset + strlen((char*)(strb->str_ + offset)), str, strlen(str) + 1);
+	return (char*)(strb->str_ + offset);
+}
+
 /* -------------------------------------------------------------*/
 
 o__inline__
@@ -377,7 +396,7 @@ strob_strncat(STROB * strb, char *str, size_t len)
 	int ilen;
 
 	ilen = (int)len;
-	strb_ret = strob_reopen_if(strlen((char*)(strb->str_)) + ilen + 1, strb);
+	strb_ret = strob_reopen_if(strb, strlen((char*)(strb->str_)) + ilen + 1);
 	if (!strb_ret) {
 		return (char *)(NULL);
 	}
@@ -392,8 +411,8 @@ char *
 strob_strcat(STROB * strb, char *str)
 {
 	STROB *strb_ret;
-	strb_ret = strob_reopen_if(
-			strlen((char*)(strb->str_)) + strlen(str) + 1, strb);
+	strb_ret = strob_reopen_if(strb,
+			strlen((char*)(strb->str_)) + strlen(str) + 1);
 	if (!strb_ret)
 		return (char *)(NULL);
 	strb->length_=strlen((char*)(strb->str_)) + strlen(str);
@@ -427,7 +446,7 @@ char *
 strob_strncpy(STROB * strb, char *str, size_t n)
 {
 	char * s;
-	strob_reopen_if(n + 1, strb);
+	strob_reopen_if(strb, n + 1);
 	s = strncpy((char*)(strb->str_), str, n);
 	strb->str_[n] = '\0';
 	strb->length_=strlen((char*)(strb->str_));
@@ -451,7 +470,7 @@ strob_strlen(STROB * strb)
 char *
 strob_strchar(STROB * strb, int index)
 {
-	strob_reopen_if(index + 1, strb);
+	strob_reopen_if(strb, index + 1);
 	return (char*)(strb->str_ + index);
 }
 
@@ -574,8 +593,19 @@ strob_strstrtok(STROB * buf, char *s, const char * delim)
 	return retval;
 }
 
+void
+strob_set_fill_char(STROB * strb, int ch)
+{
+	strb->fill_charM = ch;
+}
 
-/* --- Unrestricted binary string interface ------------------------------*/
+int
+strob_get_fill_char(STROB * strb)
+{
+	return strb->fill_charM;
+}
+
+/* --- Unrestricted binary string interface ---- NOT WELL TESTED ----------*/
 
 
 void *
@@ -594,7 +624,7 @@ strob_append_hidden_null(STROB *strb)
 }
 
 void *
-strob_memcpy_at(STROB *strb, void * ct, size_t n, size_t offset)
+strob_memcpy_at(STROB *strb, size_t offset, void * ct, size_t n)
 {
 	if (!strob_set_up(strb, (int)n + (int)offset))
 		return (void*)(NULL);
@@ -632,7 +662,7 @@ strob_memset(STROB *strb, int c, size_t n)
 void
 strob_set_memlength(STROB * strb, int len)
 {
-	strob_reopen_if(len, strb);
+	strob_reopen_if(strb, len);
 	strb->length_=len;
 }
 
@@ -669,6 +699,45 @@ strob_catstr(STROB * strb, char *str)
 	return strob_strcat(strb, str);
 }
 
+int
+strob_vsprintf_at(STROB * sb, int at_offset, char * format, va_list ap)
+{
+	int added_amount=0; 
+	int up_incr = 128; 
+	char * start;
+	va_list aq;
+	int ret;
+	int len;
+	char * oldend = NULL;
+
+	if (at_offset > strob_get_reserve(sb)) {
+		strob_set_memlength(sb, at_offset + up_incr);
+	}
+
+	do {
+		if (oldend)
+			*oldend = '\0';
+
+		strob_set_memlength(sb, strob_get_reserve(sb) + added_amount);
+
+		start = strob_str(sb) + at_offset; 
+		len = strob_get_reserve(sb) - at_offset;
+
+		oldend = start;
+		added_amount += up_incr;
+#if defined va_copy
+                va_copy(aq, ap);
+#elif defined __va_copy
+                __va_copy(aq, ap);
+#else
+                memcpy(aq, ap, sizeof(va_list));
+#endif
+		ret=vsnprintf(start, len, format, aq);
+		va_end(aq);
+	} while (ret < 0 || ret >= len);
+	return ret;
+}
+
 int
 strob_vsprintf(STROB * sb, int do_append, char * format, va_list ap)
 {
@@ -740,3 +809,14 @@ strob_sprintf(STROB * sb, int do_append, char * format, ...)
 	va_end(ap);
 	return ret;
 }
+
+int
+strob_sprintf_at(STROB * sb, int at_offset, char * format, ...)
+{
+	int ret;
+	va_list ap;
+	va_start(ap, format);
+	ret = strob_vsprintf_at(sb, at_offset, format, ap); 
+	va_end(ap);
+	return ret;
+}
