@@ -1,6 +1,6 @@
 /* copyout.c:
 
-   Copyright (C) 1998, 1999 Jim Lowe
+   Copyright (C) 1998, 1999, 2014 Jim Lowe
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -39,6 +39,8 @@
 #include "inttostr.h"
 #include "strtoint.h"
 #include "swutilname.h"
+#include "etar.h"
+#include "atomicio.h"
 
 static
 intmax_t
@@ -55,12 +57,12 @@ taru_write_archive_member_data_(TARU * taru,
 	intmax_t ret = 0;
 	char * tmpp = ahsStaticGetTarLinkname(file_hdr0);
 
-	E_DEBUG("");
+	/* E_DEBUG(""); */
 
 	if ( (((file_hdr0->c_mode & CP_IFMT) == CP_IFREG) &&
 				(tmpp == (char*)NULL || !strlen(tmpp)))
 	) {
-		E_DEBUG("");
+		/* E_DEBUG(""); */
 		if (input_fd < 0) {
 			E_DEBUG("");
 			fd = open(ahsStaticGetTarFilename(file_hdr0),
@@ -76,9 +78,9 @@ taru_write_archive_member_data_(TARU * taru,
 		}
 
 		if (fout == NULL) {
-			E_DEBUG("");
+			/* E_DEBUG(""); */
 			ret = swlib_pump_amount8(out_file_des, input_fd, (intmax_t)(file_hdr0->c_filesize), adjuct_ofd, digs);
-			E_DEBUG2("ret=%s", swlib_imaxtostr(ret, NULL));
+			/* E_DEBUG2("ret=%s", swlib_imaxtostr(ret, NULL)); */
 
 			if (ret < 0) {
 				E_DEBUG("");
@@ -87,20 +89,21 @@ taru_write_archive_member_data_(TARU * taru,
 					swlib_utilname_get(), swlib_imaxtostr(ret, NULL));
 				return -1;
 			} else {
-				E_DEBUG("");
+				/* E_DEBUG(""); */
 				if (ret != (intmax_t)(file_hdr0->c_filesize)) {
 					E_DEBUG2("Error: ret=%s", swlib_imaxtostr(ret, NULL));
 					E_DEBUG2("Error: filesize=%s", swlib_umaxtostr(file_hdr0->c_filesize, NULL));
 					fprintf(stderr,
 					"%s: taru_write_archive_member_data_ internal error. ret=%s\n",
 						swlib_utilname_get(), swlib_imaxtostr(ret, NULL));
+					return -1;
 				} else {
-					E_DEBUG("");
+					/* E_DEBUG(""); */
 					/*
 					fprintf(stderr, "%s loaded OK\n", ahsStaticGetTarFilename(file_hdr0));
 					*/
 				}
-				E_DEBUG("");
+				/* E_DEBUG(""); */
 				if ((ret += taru_tape_pad_output(
 						pad_out_file_des,
 						(intmax_t)(file_hdr0->c_filesize),
@@ -109,7 +112,7 @@ taru_write_archive_member_data_(TARU * taru,
 					if (fd >= 0) close(fd);
 					return -1;
 				}
-				E_DEBUG("");
+				/* E_DEBUG(""); */
 				if (adjuct_ofd >= 0) {
 					if (taru_tape_pad_output(
 							adjuct_ofd,
@@ -121,15 +124,15 @@ taru_write_archive_member_data_(TARU * taru,
 					}
 				}
 			}
-			E_DEBUG("");
+			/* E_DEBUG(""); */
 		} else {
-			E_DEBUG("");
+			/* E_DEBUG(""); */
 			ret = (*fout)(out_file_des);
-			E_DEBUG2("ret=%d", ret);
+			/* E_DEBUG2("ret=%d", (int)ret); */
 		}
 		if (fd >= 0) close(fd);
 	} 
-	E_DEBUG2("ret=%s", swlib_imaxtostr(ret, NULL));
+	/* E_DEBUG2("ret=%s", swlib_imaxtostr(ret, NULL)); */
 	return ret;
 }
 
@@ -264,6 +267,259 @@ tarui_write_archive_member_(TARU * taru,
 	return ret + ret1;
 }
 
+static
+int
+get_ilen(int rlen)
+{
+	int i = 0;
+	int j = 0;
+	int r = rlen;
+	
+	while (r/10 >= 1) {
+		i++;
+		r=r/10;
+	}
+	i++;
+
+	/* i is now the approximate guessed length of the record
+ 	 * Now it must be determinied iteratively if this it is correct
+ 	 * when the length of the leading decimal length string is included.
+ 	 */  
+
+	r = rlen + i;
+	
+	/* r is the first guess at the total length */
+
+	while (i < 99 /*arbitrarily large number to stop infinite loop*/) {
+		j = 0;
+		while (r/10 >= 1) {
+			j++;
+			r=r/10;
+		}
+		j++;
+		if (i == j)
+			return i+rlen;
+		i++;
+		r = rlen + i;
+	}
+	/* should never get here */
+	return -1;
+}
+
+
+static
+int
+construct_exthdr_record(STROB * buf, int initial_offset, char * keyword, char * value, int len)
+{
+	int rlen;
+	int ret;
+
+	/*
+	"%d %s=%s\n", length, keyword, value
+	*/
+
+	rlen = 1/*space*/ + strlen(keyword) + len + 1/*equals_sign*/ + 1/*newline*/;
+
+	/* rlen is the length not including the rendering of the decimal length */
+
+	ret = get_ilen(rlen);
+	if (ret < 0) {
+		/* This should really never happen */
+		fprintf(stderr, "internal error in construct_exthdr_record loc=1\n");
+		exit(120);
+	}
+	rlen = ret;
+
+	/* rlen is now the record length */
+	strob_sprintf_at(buf, initial_offset, "%d %s=", rlen, keyword);
+
+	ret = strlen(strob_str(buf) + initial_offset);
+	strob_memcpy_at(buf, initial_offset + ret /*offset*/, (void *)value, (size_t)len);
+
+	/* sanity check */
+	if ((ret + len + 1/*newline*/ ) != rlen) {
+		/* This should really never happen */
+		fprintf(stderr, "internal error in construct_exthdr_record loc=2\n");
+		exit(121);
+	}
+
+	/* Now append the '\n' newline */
+
+	strob_memcpy_at(buf, initial_offset + rlen-1, (void *)"\n", 1);
+
+	return rlen;
+}
+
+static int
+write_exthdrs_xattr_records(TARU * taru, struct new_cpio_header * file_hdr, int data_len)
+{
+	int ret;
+	STROB * buf;
+	CPLOB * extlist;
+	EXATT  *p;
+	char * attr;
+	char * value;
+	int valuelen;
+	int index; 
+
+	buf = taru->exthdr_dataM;
+	/* strob_strcpy(buf, ""); */
+	extlist = (CPLOB*)(file_hdr->extattlistM);
+
+	index = 0;
+	while(extlist && (p=(EXATT*) cplob_val(extlist, index)) && p->is_in_useM) {
+		
+		/*
+ 		 * construct the dot delimted attribute name e.g. "SCHILLY.selinux.access"
+ 		 */
+		strob_sprintf(taru->localbufM, 0, "%s%s%s%s%s",
+				p->vendorM,
+				strlen(p->vendorM) ? "." : "",
+				p->namespaceM,
+				strlen(p->namespaceM) ? "." : "",
+				strob_str((STROB *)p->attrnameM));
+		value = strob_str(p->attrvalueM);
+		valuelen = p->lenM;
+		ret = construct_exthdr_record(buf, data_len, strob_str(taru->localbufM), value, valuelen);
+		if (ret < 0) return -1;
+		data_len += ret;
+		/* p->is_in_useM == 0; */
+		index++;
+	}
+	return data_len;
+}
+
+
+static int
+write_exthdrs_ustar_records(TARU * taru, struct new_cpio_header * file_hdr)
+{
+	int data_len;
+	int ret;
+	STROB * buf;
+
+	buf = taru->exthdr_dataM;
+	/* strob_strcpy(buf, ""); */
+
+	data_len = 0;
+	if (file_hdr->extHeader_needs_maskM & TARU_EHUM_LINKPATH) {
+		ret = construct_exthdr_record(buf, data_len, PAX_KW_linkpath,
+						ahsStaticGetTarLinkname(file_hdr),
+						strlen(ahsStaticGetTarLinkname(file_hdr)));
+		if (ret < 0)
+			return -1;
+
+		E_DEBUG2("LINKPATH: [%s]", ahsStaticGetTarLinkname(file_hdr));
+		/*
+ 		 * Now truncate the linkname to TARNAMESIZE,  this seems to be what GNU tar does
+ 		 */
+		strob_chr_index((STROB*)(file_hdr->c_tar_linkname),  TARNAMESIZE, '\0');
+		data_len += ret;
+	}
+
+	if (file_hdr->extHeader_needs_maskM & TARU_EHUM_PATH) {
+		ret = construct_exthdr_record(buf, data_len, PAX_KW_path,
+						ahsStaticGetTarFilename(file_hdr),
+						strlen(ahsStaticGetTarFilename(file_hdr)));
+		if (ret < 0)
+			return -2;
+
+		E_DEBUG2("PATH: [%s]", ahsStaticGetTarFilename(file_hdr));
+		/*
+ 		 * Now truncate the name to TARNAMESIZE,  this seems to be what GNU tar does
+ 		 */
+		strob_chr_index((STROB*)(file_hdr->c_name),  TARNAMESIZE, '\0');
+		data_len += ret;
+	}
+
+	if (file_hdr->extHeader_needs_maskM & TARU_EHUM_SIZE) {
+		strob_sprintf(taru->localbufM, 0, "%lu", (unsigned long)(file_hdr->c_filesize));
+		ret = construct_exthdr_record(buf, data_len, PAX_KW_size,
+				strob_str(taru->localbufM), strob_strlen(taru->localbufM));
+		if (ret < 0) return -3;
+
+		/*
+ 		 * Now set the filesize to a arbitrary number that fits
+ 		 */
+
+		file_hdr->c_filesize = TAR_MAX_SIZE;
+
+		data_len += ret;
+	}
+
+	if (file_hdr->extHeader_needs_maskM & TARU_EHUM_UID) {
+		strob_sprintf(taru->localbufM, 0, "%lu", (unsigned long)(file_hdr->c_uid));
+		ret = construct_exthdr_record(buf, data_len, PAX_KW_uid,
+				strob_str(taru->localbufM), strob_strlen(taru->localbufM));
+		if (ret < 0) return -4;
+		/*
+ 		 * Now set the value to a number that will fit
+ 		 */
+		file_hdr->c_uid = TAR_MAX_UID;
+		data_len += ret;
+	}
+
+	if (file_hdr->extHeader_needs_maskM & TARU_EHUM_GID) {
+		strob_sprintf(taru->localbufM, 0, "%lu", (unsigned long)(file_hdr->c_gid));
+		ret = construct_exthdr_record(buf, data_len, PAX_KW_gid,
+				strob_str(taru->localbufM), strob_strlen(taru->localbufM));
+		if (ret < 0) return -5;
+		/*
+ 		 * Now set the value to a number that will fit
+ 		 */
+		file_hdr->c_uid = TAR_MAX_GID;
+		data_len += ret;
+	}
+
+	if (file_hdr->extHeader_needs_maskM & TARU_EHUM_UNAME) {
+		ret = construct_exthdr_record(buf, data_len, PAX_KW_uname,
+			ahsStaticGetTarUsername(file_hdr), strlen(ahsStaticGetTarUsername(file_hdr)));
+		if (ret < 0) return -6;
+		/*
+ 		 * Now set the string that will fit in a ustar header by truncation
+ 		 */
+		strob_chr_index((STROB*)(file_hdr->c_username),  THB_FL_uname - 1, '\0');
+
+		data_len += ret;
+	}
+
+	if (file_hdr->extHeader_needs_maskM & TARU_EHUM_GNAME) {
+		ret = construct_exthdr_record(buf, data_len, PAX_KW_gname,
+			ahsStaticGetTarGroupname(file_hdr), strlen(ahsStaticGetTarGroupname(file_hdr)));
+		if (ret < 0) return -7;
+		/*
+ 		 * Now set the string that will fit in a ustar header by truncation
+ 		 */
+		strob_chr_index((STROB*)(file_hdr->c_groupname),  THB_FL_gname - 1, '\0');
+		data_len += ret;
+	}
+
+	if (file_hdr->extHeader_needs_maskM & TARU_EHUM_DEV_MAJ) {
+		strob_sprintf(taru->localbufM, 0, "%lu", (unsigned long)(file_hdr->c_dev_maj));
+		ret = construct_exthdr_record(buf, data_len, SWBIS_VENDOR "." PAX_KW_devmajor,
+				strob_str(taru->localbufM), strob_strlen(taru->localbufM));
+		if (ret < 0) return -8;
+		/*
+ 		 * Now set the value to a number that will fit
+ 		 */
+		file_hdr->c_dev_maj = TAR_MAX_DEVICE;
+		data_len += ret;
+	}
+
+	if (file_hdr->extHeader_needs_maskM & TARU_EHUM_DEV_MIN) {
+		strob_sprintf(taru->localbufM, 0, "%lu", (unsigned long)(file_hdr->c_dev_min));
+		ret = construct_exthdr_record(buf, data_len, SWBIS_VENDOR "." PAX_KW_devminor,
+				strob_str(taru->localbufM), strob_strlen(taru->localbufM));
+		if (ret < 0) return -9;
+		/*
+ 		 * Now set the value to a number that will fit
+ 		 */
+		file_hdr->c_dev_min = TAR_MAX_DEVICE;
+		data_len += ret;
+	}
+
+	return data_len;
+}
+
 int 
 taru_safewrite(int fd, void *vbuf, size_t amount)
 {
@@ -373,16 +629,24 @@ taru_write_archive_member(TARU * taru, char *filename,
 	int fd = -1;
 	intmax_t ret = 0;
 	int nullfd = -1;
+	char * p;
+	int vlen = 1024;
 
-	*linkname='\0';;
+	E_DEBUG("ENTERING");
+	E_DEBUG2("filename=[%s]", filename);
+	*linkname='\0';
 	if (statbuf) {
+		E_DEBUG("statbuf");
 		taru_statbuf2filehdr(file_hdr, statbuf, NULL,
 					filename, linkname);
 	} else if (file_hdr0) {
+		E_DEBUG("file_hdr0");
 		taru_filehdr2filehdr(file_hdr, file_hdr0);
 	} else {
+		E_DEBUG("else");
 		if(lstat (filename, &st) == 0) {
 			statbuf=&st;
+			E_DEBUG("lstat");
 			taru_statbuf2filehdr(file_hdr, statbuf,
 						NULL, filename, linkname);
 		} else {
@@ -417,6 +681,17 @@ taru_write_archive_member(TARU * taru, char *filename,
 				return 0;
 			}
 			break;
+		case CP_IFLNK:
+			/* FIXME arbitrary limit */
+			ahsStaticSetTarLinknameLength(file_hdr, vlen);
+			p = ahsStaticGetTarLinkname(file_hdr);
+			*(p+vlen-1) = '\0';
+			ret = swvarfs_u_readlink(NULL, filename, p, vlen-1);
+			if (ret < 0) {
+				fprintf(stderr, "swbis: %s: %s\n",
+						strerror(errno), filename);
+			}
+			break;
 		default:
 			fd = -1;
 			break;
@@ -504,15 +779,27 @@ taru_write_archive_member_header(TARU * taru,
 				int tarheaderflags)
 {
 	int do_gnu_long_link = 0;
+	int do_gnu_long_name = 0;
+	int do_gnu_long_link_name_bytes = 0;
+	int do_gnu_long_link_link_bytes = 0;
+	int do_exthdr_bytes = 0;
 	int nfound = 0, retval = 0;
+	int ret;
 	hllist_entry * link_record_buf = NULL;
-	char input_name[512];	/* Name of file read from stdin.  */
+	char * tmpp;
 	struct new_cpio_header * file_hdr;
 	dev_t v_dev;
 	ino_t v_ino;
 	int is_dir;
+	unsigned long c_type;
 	int do_strip_leading_slash = 0;
 	int linkrecord_active;
+	int is_too_long;
+	char link_name[128];	
+	int link_size;
+	char * sbase;
+	ETAR * etar;
+	int tar_iflags_numeric_uids = (tarheaderflags & TARU_TAR_NUMERIC_UIDS); 
 
 	E_DEBUG("BEGIN");
 	file_hdr = ahsStaticCreateFilehdr();
@@ -523,6 +810,8 @@ taru_write_archive_member_header(TARU * taru,
 			tarheaderflags & TARU_TAR_DO_STRIP_LEADING_SLASH;
 	}
 
+	E_DEBUG2("HERE tarheaderflags=%d", tarheaderflags);
+
 	/* 
 	 * Set Info from the file_hdr0
 	 */
@@ -565,101 +854,274 @@ taru_write_archive_member_header(TARU * taru,
 	}
 
 	is_dir = (file_hdr->c_mode & CP_IFMT) == CP_IFDIR;
+	c_type = (file_hdr->c_mode & CP_IFMT);
 
 	/*
 	* Check the name length.
 	*/
 	E_DEBUG2("name is [%s]", ahsStaticGetTarFilename(file_hdr));
 	file_hdr->c_namesize = strlen(ahsStaticGetTarFilename(file_hdr)) + 1;
-	if ((archive_format_out == arf_tar || archive_format_out == arf_ustar)
-	    && taru_is_tar_filename_too_long(
-				ahsStaticGetTarFilename(file_hdr),
-				tarheaderflags, 
-				&do_gnu_long_link,
-				is_dir)) {
-		E_DEBUG("");
-		fprintf(stderr, "error: file name too long: %s\n",
-				ahsStaticGetTarFilename(file_hdr));
-		ahsStaticDeleteFilehdr(file_hdr);
-		return -2;
-	}
+	
 
-	E_DEBUG("");
-	if (do_gnu_long_link) {
-		E_DEBUG("in do_gnu_long_link");
-		if ((do_gnu_long_link=taru_write_long_link_member
-				(taru, out_file_des,
-					ahsStaticGetTarFilename(file_hdr),
-					is_dir, tarheaderflags)) != 1024) {
-			fprintf(stderr, 
-			"internal error writing GNU extension LongLink: %s\n",
+	if (archive_format_out == arf_tar || archive_format_out == arf_ustar)
+	{
+		is_too_long =  taru_is_tar_filename_too_long(ahsStaticGetTarFilename(file_hdr),
+						tarheaderflags, &do_gnu_long_name, 0 /*is_dir*/);
+		if (is_too_long) {
+			E_DEBUG("");
+			fprintf(stderr, "%s: file name too long, not dumped: %s\n",
+					swlib_utilname_get(),
+					ahsStaticGetTarFilename(file_hdr));
+			ahsStaticDeleteFilehdr(file_hdr);
+			return -2;
+		}
+
+		/* determine if the linkname is too long */
+		is_too_long =  taru_is_tar_linkname_too_long(ahsStaticGetTarLinkname(file_hdr),
+								tarheaderflags, &do_gnu_long_link);
+		if (is_too_long) {
+			fprintf(stderr, "%s: link name too long, not dumped: %s\n",
+					swlib_utilname_get(),
 					ahsStaticGetTarFilename(file_hdr));
 			return -3;
 		}
-		swlib_strncpy(input_name,
-				ahsStaticGetTarFilename(file_hdr), TARNAMESIZE);
-		input_name[99] = '\0';
-		/*
-		* truncate the name in ahsStaticGetTarFilename()
-		*/
-		if (
-			(tarheaderflags & TARU_TAR_GNU_OLDGNUTAR) ||
-			(tarheaderflags & TARU_TAR_NAMESIZE_99) ||
-			0
-		) {
-			/*
-			 * This is what GNU tar 1.13.25 does
-			 * as well as swpackage formats:  oldgnu, gnutar
-			 */
-			ahsStaticSetTarFilenameLength(file_hdr, OLDGNU_TARNAMESIZE+1);
-			*(ahsStaticGetTarFilename(file_hdr) + OLDGNU_TARNAMESIZE) = '\0';
-		} else {
-			/*
-			 * This is what GNU tar 1.15.1 does
-			 * as well as swpackage formats:  gnu
-			 */
-			ahsStaticSetTarFilenameLength(file_hdr, TARNAMESIZE+1);
-			*(ahsStaticGetTarFilename(file_hdr) + TARNAMESIZE) = '\0';
-		}
-	} else {
+
+
+		E_DEBUG2("do_gnu_long_link=%d", do_gnu_long_link);
 		E_DEBUG("");
-		swlib_strncpy(input_name,
-				ahsStaticGetTarFilename(file_hdr), 512);
-	}
+		
+		if (tarheaderflags & TARU_TAR_PAXEXTHDR || tarheaderflags & TARU_TAR_PAXEXTHDR_MD) {
+			if (tar_iflags_numeric_uids == 0) {
+				/*
+ 				 * Fill in the user and group
+ 				 */
+				taru_set_filehdr_sysdb_nameid_by_policy("G"/* group */, file_hdr,
+						'\0' /*termch*/, tar_iflags_numeric_uids);
+				taru_set_filehdr_sysdb_nameid_by_policy("U"/* group */, file_hdr,
+						'\0' /*termch*/, tar_iflags_numeric_uids);
+			}
+			/*
+ 			 * Set the flags indicating an extended header record 
+ 			 * is required due to the length limitation in the ustar
+ 			 * header field
+ 			 */
+			file_hdr->extHeader_needs_maskM = taru_fill_exthdr_needs_mask(file_hdr, tarheaderflags);
+
+			/*
+ 			 * GNU LongLinks and PAX Ext headers are mutually exclusive.
+ 			 */
+			do_gnu_long_name = 0;
+			do_gnu_long_link = 0;
+		}
+
+		/*
+ 		 * GNU tar appears not to like a longLink link type 'K' without a type 'L'
+ 		 * i.e. don't create a 'K' without preceeding it with a 'L' type
+ 		 */
+		if (do_gnu_long_name | do_gnu_long_link) {
+			/*
+ 			 * GNU LongLink type
+ 			 */
+			E_DEBUG2("in do_gnu_long_name: %s", ahsStaticGetTarFilename(file_hdr));
+			if ((do_gnu_long_link_name_bytes=taru_write_long_link_member
+					(taru, out_file_des,
+						ahsStaticGetTarFilename(file_hdr), GNUTYPE_LONGNAME,
+						c_type, tarheaderflags)) != 1024) {
+				fprintf(stderr, 
+				"internal error writing GNU extension LongLink: %s\n",
+						ahsStaticGetTarFilename(file_hdr));
+				return -3;
+			}
+			/*
+			* truncate the name in ahsStaticGetTarFilename()
+			*/
+			if (
+				(tarheaderflags & TARU_TAR_GNU_OLDGNUTAR) ||
+				(tarheaderflags & TARU_TAR_NAMESIZE_99) ||
+				0
+			) {
+				/*
+				 * This is what GNU tar 1.13.25 does
+				 * as well as swpackage formats:  oldgnu, gnutar
+				 */
+				ahsStaticSetTarFilenameLength(file_hdr, OLDGNU_TARNAMESIZE+1);
+				*(ahsStaticGetTarFilename(file_hdr) + OLDGNU_TARNAMESIZE) = '\0';
+			} else {
+				/*
+				 * This is what GNU tar 1.15.1 does
+				 * as well as swpackage formats:  gnu
+				 */
+				ahsStaticSetTarFilenameLength(file_hdr, TARNAMESIZE+1);
+				*(ahsStaticGetTarFilename(file_hdr) + TARNAMESIZE) = '\0';
+			}
+		} else if (
+		((tarheaderflags & TARU_TAR_PAXEXTHDR ) || (tarheaderflags & TARU_TAR_PAXEXTHDR_MD)) &&
+				(file_hdr->extHeader_needs_maskM || file_hdr->extattlistM)
+				/* FIXME */
+			) {
+
+			/*
+ 			 * POSIX Pax Headers
+ 			 */
+			E_DEBUG("in pax headers");
+
+			/*
+ 			 * Set the User and group names per the policy
+ 			 */ 
+			taru_set_filehdr_sysdb_nameid_by_policy("G"/* group */, file_hdr,
+					'\0' /*termch*/, tar_iflags_numeric_uids);
+			taru_set_filehdr_sysdb_nameid_by_policy("U"/* group */, file_hdr,
+					'\0' /*termch*/, tar_iflags_numeric_uids);
+
+
+			/*
+ 			 * initialize the buffer where the extended records will be written.
+ 			 */
+			strob_strcpy(taru->exthdr_dataM, "");
+			
+			/*
+ 			 * Do Pax Exttended Header POSIX attributes for the fields that don't fit.
+ 			 */
+
+			do_exthdr_bytes = write_exthdrs_ustar_records(taru, file_hdr); 
+			if (do_exthdr_bytes < 0) {
+				fprintf(stderr, "internal error writing pax extended header POSIX attributes: %s: retval=%d\n",
+						ahsStaticGetTarFilename(file_hdr), do_exthdr_bytes);
+				return -3;
+			}
+
+			/*
+ 			 * Write the extended attributes
+ 			 */
+
+			do_exthdr_bytes = write_exthdrs_xattr_records(taru, file_hdr, do_exthdr_bytes);
+			if (do_exthdr_bytes < 0) {
+				fprintf(stderr, "internal error writing pax extended header xattr attributes: %s: retval=%d\n",
+						ahsStaticGetTarFilename(file_hdr), do_exthdr_bytes);
+				return -4;
+			}
+
+			/*
+ 			 * If there are no extended headers to written then skip the writes
+ 			 */
+
+			if (do_exthdr_bytes == 0)  goto NO_EA; /* goto */
+
+			/*
+ 			 * Now construct the header
+ 			 */
+
+			etar = taru->etarM;
+			etar->etar_tarheaderflagsM = taru->taru_tarheaderflagsM;	 
+			etar_init_hdr(etar);
+			strob_strcpy(taru->u_name_bufferM, "");
+			strob_sprintf(taru->u_name_bufferM, STROB_DO_APPEND, "./PaxHeaders.%d/", (int)swlib_get_pax_header_pid());
+			sbase = swlib_basename((STROB *)NULL, ahsStaticGetTarFilename(file_hdr));
+			strob_sprintf(taru->u_name_bufferM, STROB_DO_APPEND, "%s", sbase);
+			etar_set_uid(etar, 0);
+			etar_set_gid(etar, 0);
+			etar_set_uname(etar, "");
+			etar_set_gname(etar, "");
+			etar_set_mode_ul(etar, (unsigned int)(0644));
+			if (etar_set_pathname(etar, strob_str(taru->u_name_bufferM))) SWLIB_FATAL("name too long");
+			etar_set_typeflag(etar, XHDTYPE);
+			etar_set_size(etar, do_exthdr_bytes);
+			etar_set_chksum(etar);
+			
+			/*
+ 			 * Now write the header
+ 			 */
+
+			if (out_file_des >= UXFIO_FD_MIN)
+				ret = atomicio((ssize_t (*)(int, void *, size_t))uxfio_write, out_file_des,
+					(void*)(etar_get_hdr(etar)), (size_t)(TARRECORDSIZE));
+			else
+				ret = atomicio((ssize_t (*)(int, void *, size_t))write, out_file_des,
+					(void*)(etar_get_hdr(etar)), (size_t)(TARRECORDSIZE));
+
+
+			if (ret != TARRECORDSIZE) {
+				/* Big error */
+				fprintf(stderr, "%s: error writing extended header, out_file_des=%d\n", swlib_utilname_get(), out_file_des);
+				fprintf(stderr, "%s: error writing extended header, ret=%d\n", swlib_utilname_get(), ret);
+				return -11;
+			}
+
+			/*
+ 			 * Now write the data
+ 			 */
+
+			if (out_file_des >= UXFIO_FD_MIN)
+				ret = atomicio((ssize_t (*)(int, void *, size_t))uxfio_write, out_file_des,
+					(void*)strob_str(taru->exthdr_dataM), do_exthdr_bytes);
+			else
+				ret = atomicio((ssize_t (*)(int, void *, size_t))write, out_file_des,
+					(void*)strob_str(taru->exthdr_dataM), do_exthdr_bytes);
+
+
+
+			if (ret != do_exthdr_bytes) {
+				/* Big error */
+				fprintf(stderr, "%s: error writing extended data\n", swlib_utilname_get());
+				return -12;
+			}
+	
+			/*
+ 			 * Now pad the output to nearest block
+ 			 */
+
+			ret = taru_tape_pad_output(out_file_des, (intmax_t)ret, arf_ustar);
+			if (ret < 0) {
+				/* Big error */
+				fprintf(stderr, "%s: error writing extended data padding\n", swlib_utilname_get());
+				return -13;
+			}
+
+			NO_EA:
+			; /* required  to avert at compiler error of goto label at end of conditional block */
+		} else {
+			;
+			/* Just continue per normal */
+		}
+
+	} /* Tar Format */
 
 	E_DEBUG2("c_is_link = %d", file_hdr->c_is_tar_lnktype);
 	if (file_hdr->c_is_tar_lnktype == 1) {
 		;
 		E_DEBUG2("linkname is [%s]", ahsStaticGetTarLinkname(file_hdr));
 	}
+
 	E_DEBUG("");
 	switch (file_hdr->c_mode & CP_IFMT) {
 	case CP_IFREG:
 	{
 	E_DEBUG("CP_IFREG");
-	if ((archive_format_out == arf_oldascii ||
+		if ((archive_format_out == arf_oldascii ||
 		archive_format_out == arf_newascii ||
 		archive_format_out == arf_crcascii)
 	    && (file_hdr->c_nlink > 1)) {
 		E_DEBUG("");
-		if (defer){
-			defer_set_taru(defer, taru);
-			if (defer_is_last_link(defer,
-						file_hdr)) {
-			defer_writeout_zero_length_defers(defer,
-				file_hdr, out_file_des);
-			} else {
-			defer_add_link_defer(defer, file_hdr);
-				break;
+			/* CPIO format */
+			if (defer){
+				defer_set_taru(defer, taru);
+				if (defer_is_last_link(defer,
+							file_hdr)) {
+				defer_writeout_zero_length_defers(defer,
+					file_hdr, out_file_des);
+				} else {
+				defer_add_link_defer(defer, file_hdr);
+					break;
+				}
 			}
-		}
-		else {
-			fprintf(stderr,
-				"taru internal error: cpio hard link handling is disabled (loc=2).\n");
-		}
-	} else if (archive_format_out == arf_ustar ||
-		archive_format_out == arf_tar) {
-		if (linkrecord_active) {
+			else {
+				fprintf(stderr,
+					"taru internal error: cpio hard link handling is disabled (loc=2).\n");
+			}
+		} else if (archive_format_out == arf_ustar || archive_format_out == arf_tar) {
+			/* Tar Format */
+
+			if (linkrecord_active) {
 			   E_DEBUG("tar format");
 			   nfound=0;	
 			   if (file_hdr->c_nlink > 1) {
@@ -674,7 +1136,7 @@ taru_write_archive_member_header(TARU * taru,
 					E_DEBUG("");
 					
 					if (nfound) {
-						ahsStaticSetTarLinkname(file_hdr,
+						ahsStaticSetPaxLinkname(file_hdr,
 							link_record_buf->path_);
 
 					} else {
@@ -692,20 +1154,43 @@ taru_write_archive_member_header(TARU * taru,
 					E_DEBUG("not found");
 				}
 			    }
-			}
-			else {
+			} else {
 				E_DEBUG("");
-				if (!linkrecord)
+				/* dead code */
+				if (!linkrecord && 0 /*disable this warning 6/14/2014*/ )
 					fprintf(stderr, 
 					"taru internal error: tar hard link handling is disabled (loc=3).\n");
 			}
+			tmpp = ahsStaticGetTarLinkname(file_hdr);
+			if (tmpp && strlen(tmpp)) {
+				/* Hard link */
+				E_DEBUG2("in Hard link: %s", ahsStaticGetTarFilename(file_hdr));
+				if (do_gnu_long_link) {
+					E_DEBUG2("in do_gnu_long_link: %s", ahsStaticGetTarFilename(file_hdr));
+					if ((do_gnu_long_link_link_bytes=taru_write_long_link_member(taru,
+							out_file_des,
+							ahsStaticGetTarLinkname(file_hdr), GNUTYPE_LONGLINK,
+							c_type, tarheaderflags)) != 1024) {
+
+						fprintf(stderr, 
+							"internal error writing GNU extension LongLink for linkname: %s\n",
+							ahsStaticGetTarLinkname(file_hdr));
+						return -3;
+					}
+
+					/* truncate the linkname to 99 chars so the remaining tar operations 
+ 					 * do not complain.
+ 					 */
+					tmpp = ahsStaticGetTarLinkname(file_hdr);
+					*(tmpp+(TARLINKNAMESIZE-1)) = '\0';
+				}
+			}
 		} else {
 			E_DEBUG("");
-			/* 
-			is traversed.
-			*/
+			/* ?? dead */
 		}
-		E_DEBUG("Entering taru_write_out_header");
+
+		E_DEBUG2("Entering taru_write_out_header: %s", ahsStaticGetTarFilename(file_hdr));
 		retval = taru_write_out_header
 			(taru, file_hdr, out_file_des, 
 				archive_format_out,
@@ -726,13 +1211,14 @@ taru_write_archive_member_header(TARU * taru,
 			}
 			else {
 				E_DEBUG("");
-				if (!linkrecord)
+				/* dead code */
+				if (!linkrecord && 0 /* disable this warning 6/14/2014 */)
 				fprintf(stderr,
 					"taru internal error: tar hard link handling is disabled (loc=4).\n");
 			}
 		}
 		break;
-	}
+	} /*case CP_IFREG */
 	case CP_IFDIR:
 		E_DEBUG("CP_IFDIR");
 		file_hdr->c_filesize = 0;
@@ -774,7 +1260,7 @@ taru_write_archive_member_header(TARU * taru,
 			if (nfound) {
 				E_DEBUG("");
 				file_hdr->c_mode |= CP_IFREG;
-				ahsStaticSetTarLinkname(file_hdr,
+				ahsStaticSetPaxLinkname(file_hdr,
 						link_record_buf->path_);
 				retval = taru_write_out_header
 					(taru, file_hdr, 
@@ -800,8 +1286,7 @@ taru_write_archive_member_header(TARU * taru,
 		 archive_format_out,
 		(char *)(NULL),
 		tarheaderflags);
-	if (archive_format_out == arf_ustar ||
-			archive_format_out == arf_tar) {
+	if (archive_format_out == arf_ustar || archive_format_out == arf_tar) {
 			E_DEBUG("");
 			if (linkrecord_active) {
 				if (file_hdr->c_nlink > 1) {
@@ -827,26 +1312,50 @@ taru_write_archive_member_header(TARU * taru,
 	case CP_IFLNK:
 	E_DEBUG("CP_IFLNK");
 	{
-		char link_name[128];	
-		int link_size;
 		E_DEBUG("");
 		link_size = strlen(ahsStaticGetTarLinkname(file_hdr));
 		file_hdr->c_filesize = link_size;
 		if (archive_format_out == arf_tar ||
 				archive_format_out == arf_ustar) {
-			if (link_size + 1 > TARNAMESIZE) {
-				fprintf(stderr, "%s: symbolic link too long\n",
-					ahsStaticGetTarFilename(file_hdr));
-                               	link_name[TARNAMESIZE] = '\0';
-				return -4;
+			if (do_gnu_long_link == 0) {
+				E_DEBUG("do_gnu_long_link == 0");
+				if (link_size + 1 > TARNAMESIZE) {
+					fprintf(stderr, "%s: symbolic link too long\n",
+						ahsStaticGetTarFilename(file_hdr));
+       	                        	link_name[TARNAMESIZE] = '\0';
+					return TARU_HE_LINKNAME_4;  /* -4; */
+				} else {
+					retval = taru_write_out_header
+						(taru, file_hdr, out_file_des,
+							archive_format_out,
+							(char *)(NULL), 
+							tarheaderflags);
+				}
 			} else {
+				E_DEBUG("do_gnu_long_link == 1");
+				E_DEBUG("calling taru_write_long_link_member");
+				if ((do_gnu_long_link_link_bytes=taru_write_long_link_member(taru, out_file_des,
+					ahsStaticGetTarLinkname(file_hdr), GNUTYPE_LONGLINK,
+					c_type, tarheaderflags)) != 1024) {
+					fprintf(stderr, 
+					"internal error writing GNU extension LongLink: %s\n",
+							ahsStaticGetTarLinkname(file_hdr));
+					return -6;
+				}
+				/*
+ 				 * Truncate the name so as to make the remaining machinery believe
+ 				 * everything fits OK.
+ 				 */
+				*(ahsStaticGetTarLinkname(file_hdr)+99) = '\0';
+				file_hdr->c_filesize = 0;
 				retval = taru_write_out_header
-					(taru, file_hdr, out_file_des,
-						archive_format_out,
-						(char *)(NULL), 
-						tarheaderflags);
+					(taru, file_hdr, out_file_des, 
+						archive_format_out, 
+						(char *) (NULL), tarheaderflags);
+				if (retval < 0)
+					return -7;
 			}
-		} else {
+		} else /* cpio format */{
 			E_DEBUG("");
 			retval = taru_write_out_header
 				(taru, file_hdr, out_file_des,
@@ -866,10 +1375,10 @@ taru_write_archive_member_header(TARU * taru,
 	default:
 		E_DEBUG("");
 		fprintf(stderr, 
-		"%s: unknown file type in archive header : mode=%o FMT=%o \n",
-			input_name, 
+		"%s: unknown file type in archive header: mode=%o: FMT=%o: name=%s \n",
+			swlib_utilname_get(), 
 			(unsigned int)(file_hdr->c_mode),
-			(unsigned int)(file_hdr->c_mode & CP_IFMT)
+			(unsigned int)(file_hdr->c_mode & CP_IFMT), ahsStaticGetTarFilename(file_hdr)
 			);
 	}
 	if (file_hdr_return) {
@@ -878,7 +1387,7 @@ taru_write_archive_member_header(TARU * taru,
 	}
 	E_DEBUG("");
 	ahsStaticDeleteFilehdr(file_hdr);
-	return retval + do_gnu_long_link;
+	return retval + do_gnu_long_link_name_bytes + do_gnu_long_link_link_bytes;
 }
 
 int
@@ -1142,14 +1651,13 @@ taru_process_copy_out(TARU * taru, int input_fd, int out_file_des,
 	union tar_record tar_rec;
 	struct tar_header *tar_hdr = (struct tar_header *) &tar_rec;
 	char zeros_512[512];
-	char buf[512+512+512];
 	int block_number = 0;
 	int eoa = 0;
 	int ret;
 	intmax_t retx;
 	intmax_t retvalx;
+	int read_ret;
 	HLLIST * hllist;
-	int header_length;
 
 	E_DEBUG("");
 	file_hdr0 = ahsStaticCreateFilehdr();
@@ -1171,83 +1679,47 @@ taru_process_copy_out(TARU * taru, int input_fd, int out_file_des,
 	}
 
 	E_DEBUG("");
-	while (uxfio_sfa_read(input_fd, buf, 512) == 512) {
+	eoa = 0;
+	read_ret = 0;
+	while (eoa == 0) {
 		E_DEBUG("");
-		if (memcmp(buf, zeros_512, 512) == 0) {
-			if (uxfio_sfa_read(input_fd, buf, 512) == 512) {
-				if (!memcmp(buf, zeros_512, 512)) {
-					block_number++;
-					eoa = 1;
-					break;
-				} else {
-					ahsStaticDeleteFilehdr(file_hdr0);
-					return -1;
-				}
+		read_ret = taru_read_in_tar_header2(taru, file_hdr0, input_fd, NULL, &eoa, 0, -1);
+		if (read_ret < 0)
+			break;
+		if (!eoa)  {
+			E_DEBUG("");
+			retx = tarui_write_archive_member_
+				(taru, (struct stat*)NULL, file_hdr0,
+					hllist, defer, porinode,
+					out_file_des, 
+					input_fd, archive_format_out,
+					taru->taru_tarheaderflagsM /* Was 0 Apr2014*/);
+	
+			if (retx < 0) {
+				return -9;
+			}
+
+			retvalx += retx;
+			if ((retx=taru_tape_skip_padding(input_fd, file_hdr0->c_filesize, arf_ustar)) < 0) {
+				return -8;
+			}
+
+			if (ls_fd >= 0) {
+				taru_print_tar_ls_list(ls_tmp, file_hdr0, ls_verbose_level);
+				uxfio_write(ls_fd, strob_str(ls_tmp), strob_strlen(ls_tmp));
 			} else {
-				ahsStaticDeleteFilehdr(file_hdr0);
-				return -2;
+				;
 			}
 		}
 		E_DEBUG("");
-		if (taru_tarheader_check(buf)) {
-			fprintf(stderr, "tarheader error ifd=%d ofd=%d\n", input_fd, out_file_des);
-			ahsStaticDeleteFilehdr(file_hdr0);
-			return -4;
-		}
-
-		memcpy((char*)tar_hdr, buf, 512);
-		if (*(((char*)tar_hdr) + 156) == 'L') {
-			taru_otoumax(tar_hdr->size, &file_hdr0->c_filesize);
-			if (file_hdr0->c_filesize > 511) {
-				/* FIXME support arbitrary length name */
-				return -5;
-			}
-			ret = uxfio_sfa_read(input_fd, buf+512, 512);
-			if (ret != 512) return -6;
-			ret = uxfio_sfa_read(input_fd, buf+512+512, 512);
-			if (ret != 512) return -7;
-			header_length = TARRECORDSIZE+TARRECORDSIZE+TARRECORDSIZE;
-		} else {
-			/* FIXME, check for extended headers */
-			header_length = TARRECORDSIZE;
-		}
-
-		taru_read_in_tar_header2(taru, file_hdr0, -1,
-					buf, (int*)NULL, 0, header_length);
-
-
-		E_DEBUG("");
-		retx = tarui_write_archive_member_
-			(taru, (struct stat*)NULL, file_hdr0,
-				hllist, defer, porinode,
-				out_file_des, 
-				input_fd, archive_format_out,
-				/*tarheaderflags*/ 0);
-		/* was in the write member_data function */
-		taru_tape_skip_padding(input_fd,
-					file_hdr0->c_filesize,
-						arf_ustar);
-		if (retx < 0)
-			return -9;
-		retvalx += retx;
-
-		if (ls_fd >= 0) {
-			taru_print_tar_ls_list(ls_tmp, file_hdr0, ls_verbose_level);
-			uxfio_write(ls_fd, strob_str(ls_tmp), strob_strlen(ls_tmp));
-		} else {
-			;
-		}
-
-
-		E_DEBUG("");
-	}	/* while read 512-byte blocks */
-
+	} /* while not at EOA end of archive */
 	E_DEBUG("");
+	if (read_ret < 0) return -11;
+
 	/* The collection is complete; append the trailer.  */
 	ret = taru_write_archive_trailer(taru,
 				archive_format_out,
 				out_file_des, 512, 0, 0);
-
 	E_DEBUG("");
 	if (ret <= 0) return -10;
 	retvalx += (intmax_t)ret;
