@@ -46,14 +46,14 @@ def_create_deferment(struct new_cpio_header *file_hdr)
 	d = (struct deferment *) malloc(sizeof(struct deferment));
 	d->headerP = ahsStaticCreateFilehdr();
 	taru_filehdr2filehdr((d->headerP), file_hdr);	
-  	ahsStaticSetTarLinkname((d->headerP), NULL);
+  	ahsStaticSetPaxLinkname((d->headerP), NULL);
 	return d;
 }
 
 static void
 def_free_deferment(struct deferment *d)
 {
-  	ahsStaticSetTarLinkname((d->headerP), NULL);
+  	ahsStaticSetPaxLinkname((d->headerP), NULL);
   	ahsStaticSetTarFilename((d->headerP), NULL);
 	ahsStaticDeleteFilehdr(d->headerP);
 	swbis_free(d);
