@@ -30,16 +30,16 @@
 #include <ctype.h>
 #include "strob.h"
 #include "cpiohdr.h"
-#include "taru.h"
+/* #include "taru.h" */
 #include "to_oct.h"
 
 typedef struct {
-	TARU * taruM;
+	int etar_tarheaderflagsM;
 	char * tar_hdrM;
 	time_t timeM;
 } ETAR;
 
-ETAR * etar_open(TARU * taru);
+ETAR * etar_open(int tarheaderflags);
 void  etar_close(ETAR * itar);
 struct tar_header * etar_get_hdr (ETAR * etar);
 void etar_init_hdr (ETAR * etar);
