@@ -4,7 +4,7 @@
 /*
    Copyright (C) 1985, 1992, 1993, 1994, 1996, 1997, 1999, 2000, 2001, 
    2003, 2004 Free Software Foundation, Inc.
-   Copyright (C) 2003,2004,2005,2010 Jim Lowe
+   Copyright (C) 2003,2004,2005,2010,2014 Jim Lowe
 
    Portions of this code are derived from code 
    copyrighted by the Free Software Foundation.  Retention of their 
@@ -34,7 +34,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -52,6 +51,9 @@
 #include "taru.h"
 #include "uxfio.h"
 #include "swutilname.h"
+#include "swutillib.h"
+
+/*#include "debug_config.h" */
 
 #define TARRECORDSIZE 512
 #define FALSE 0
@@ -69,7 +71,7 @@
 
 #define NOBODY_ID		((uid_t)(07777777))
 #define CACHE_LEN		10
-#define SNAME_LEN		32
+/* #define SNAME_LEN		32  Now in taru.h */
 
 static int dbpair_user_nset = 0;
 static int dbpair_group_nset = 0;
@@ -77,29 +79,34 @@ static int dbpair_group_nset = 0;
 static char * g_pwent_msg[] = {"user", "group"};
 
 typedef struct {
-	int idM;
-	char nameM[TARU_SYSDBNAME_LEN];
+	long int idM;
+	STROB * snameM;
 	int in_sysM;
 } SYSDBPAIR;
 
-static SYSDBPAIR userCacheM[CACHE_LEN];
-static SYSDBPAIR groupCacheM[CACHE_LEN];
+static SYSDBPAIR userCacheM[CACHE_LEN] =  {
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1}} ;
+static SYSDBPAIR groupCacheM[CACHE_LEN] = {
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1},
+{0,NULL,-1}} ;
 
-#include "debug_config.h"
-/*
-#define TARNEEDDEBUG 1
-#undef TARNEEDDEBUG
-
-#ifdef TARNEEDDEBUG
-#define TAR_E_DEBUG(format) SWBISERROR("TAR DEBUG: ", format)
-#define TAR_E_DEBUG2(format, arg) SWBISERROR2("TAR DEBUG: ", format, arg)
-#define TAR_E_DEBUG3(format, arg, arg1) SWBISERROR3("TAR DEBUG: ", format, arg, arg1)
-#else
-#define TAR_E_DEBUG(arg)
-#define TAR_E_DEBUG2(arg, arg1)
-#define TAR_E_DEBUG3(arg, arg1, arg2)
-#endif
-*/
 
 static size_t
 split_long_name (const char *name, size_t length)
@@ -157,6 +164,8 @@ void
 dbcache_set(int * dbpair_plen, SYSDBPAIR * cache, char * name, int id, int in_sys)
 {
 	int cell;
+
+	E_DEBUG("");
 	if (*dbpair_plen >= CACHE_LEN) {
 		*dbpair_plen = 1;
 		cell = 0;
@@ -165,8 +174,14 @@ dbcache_set(int * dbpair_plen, SYSDBPAIR * cache, char * name, int id, int in_sy
 		(*dbpair_plen)++;
 	}	
 	cache[cell].idM = id;
-	strncpy(cache[cell].nameM, name, TARU_SYSDBNAME_LEN -1);
-	cache[cell].nameM[TARU_SYSDBNAME_LEN -1] = '\0';
+	if (cache[cell].snameM == NULL) {
+		cache[cell].snameM = strob_open(32);
+	}
+	E_DEBUG3("strcpy cell=[%d] id=[%d]", cell, (int)id);
+	E_DEBUG3("strcpy cell=[%d] name=[%s]", cell, name);
+	E_DEBUG3("strcpy cell=[%d] in_sys=[%d]", cell, in_sys);
+	strob_strcpy(cache[cell].snameM, name /*, TARU_SYSDBNAME_LEN -1*/);
+	/* cache[cell].nameM[TARU_SYSDBNAME_LEN -1] = '\0'; */
 	cache[cell].in_sysM = in_sys;
 }
 
@@ -179,10 +194,16 @@ dbcache_lookup(int * dbpair_len, SYSDBPAIR * cache, char * name, int id)
 
 	for(i=0; i < *dbpair_len; i++) {
 		pair = cache + i;
-		if (name && strcmp(name, pair->nameM) == 0) {
-			return pair;
+		if (name) {
+			if (pair->snameM == NULL)
+				pair->snameM = strob_open(32);
+			if (strob_strcmp(pair->snameM, name) == 0) {
+				E_DEBUG2("found by name: %s in cache", name);
+				return pair;
+			}
 		}
 		if (id >= 0 && pair->idM == id) {
+			E_DEBUG2("found by id: %d in cache", (int)id);
 			return pair;
 		}
 	}
@@ -197,7 +218,7 @@ return_name_by_cache(int * dbpair_len, SYSDBPAIR * cache, int id, int * is_in_sy
 	pair = dbcache_lookup(dbpair_len, cache, NULL, id);
 	if (!pair) return NULL;
 	*is_in_sysdb = pair->in_sysM;
-	return pair->nameM;
+	return strob_str(pair->snameM);
 }
 
 
@@ -214,22 +235,23 @@ return_id_by_cache(int * dbpair_len, SYSDBPAIR * cache, char * name, int * is_in
 
 static 
 char *
-l_tar_idcache_getuser(long uid, char *name)
+l_tar_sysdata_getuser(long uid, char *tarbuf, STROB * paxbuf)
 {
 	struct passwd *pwent;
 	pwent = getpwuid(uid);
 	if (!pwent) {
-		*name = '\0';
+		if (tarbuf) memset(tarbuf, '\0', SNAME_LEN);
+		if (paxbuf) strob_strcpy(paxbuf, "");
 		return (char *) NULL;
 	}
-	strncpy(name, pwent->pw_name, SNAME_LEN);
-	name[31]='\0';
-	return name;
+	if (tarbuf) swlib_strncpy(tarbuf, pwent->pw_name, SNAME_LEN);
+	if (paxbuf) strob_strcpy(paxbuf, pwent->pw_name);
+	return pwent->pw_name;
 }
 
 static 
 int 
-l_cache_getuidbyname(char *user, long *puid)
+l_cache_tar_getuidbyname(char *user, long *puid)
 {
 	int ret;
 	uid_t id;
@@ -253,22 +275,23 @@ l_getuidbyname(char *user, long *puid)
 
 static 
 char *
-l_tar_idcache_getgroup(long gid, char *name)
+l_tar_sysdata_getgroup(long gid, char *tarbuf, STROB * paxbuf)
 {
 	struct group *pwent;
 	pwent = getgrgid(gid);
 	if (!pwent) {
-		*name ='\0';
+		if (tarbuf) memset(tarbuf, '\0', SNAME_LEN);
+		if (paxbuf) strob_strcpy(paxbuf, "");
 		return (char *) NULL;
 	}
-	strncpy(name, pwent->gr_name, SNAME_LEN);
-	name[31]='\0';
-	return name;
+	if (tarbuf) swlib_strncpy(tarbuf, pwent->gr_name, SNAME_LEN);
+	if (paxbuf) strob_strcpy(paxbuf, pwent->gr_name);
+	return pwent->gr_name;
 }
 
 static 
 int 
-l_cache_getgidbyname(char *user, long *pgid)
+l_cache_tar_getgidbyname(char *user, long *pgid)
 {
 	int ret;
 	gid_t id;
@@ -295,17 +318,20 @@ get_pwent(
 		int idx,
 		long id, 
 		char *userkey, 
-		char *buf, 
-		long *pid, 
+		char * tarbuf,
+		STROB * paxbuf,
+		long *ppid, 
 		int  (*v_get_id)(char *, long *),
-		char * (*v_get_name)(long, char *),
+		char * (*v_get_name)(long, char *, STROB *),
 		int * dbpair_len, 
 		SYSDBPAIR * cache
 	)
 {
+	char * cname;
 	int is_in_sysdb = 0;
+	int n;
+	int retval = 0;
 	if (userkey) {
-		int n;
 		if (strlen(userkey) == 0) return -1;
 		n = return_id_by_cache(dbpair_len, cache, userkey, &is_in_sysdb);
 		if (n >= 0) {
@@ -315,51 +341,267 @@ get_pwent(
 			 * i.e. The user is looking up a username that
 			 * is not in the system data files.
 			 */
-			*pid = n;
+			*ppid = n;
 			return is_in_sysdb;
 		}	
-		n = (*(v_get_id))(userkey, pid);
+		n = (*(v_get_id))(userkey, ppid);
 		if (n == 0) {
 			/*
 			 * found in system database
 			 */
-			dbcache_set(dbpair_len, cache, userkey, *pid, 0 /*in sys db*/);
+			dbcache_set(dbpair_len, cache, userkey, *ppid, 0 /*in sys db*/);
 		} else {
 			/*
 			 * Not found
 			 */
-			*pid = AHS_UID_NOBODY;
+			*ppid = AHS_UID_NOBODY;
 			error_msg_control(userkey, -1, g_pwent_msg[idx]);
-			dbcache_set(dbpair_len, cache, userkey, *pid, -1 /*not in sys db*/);
+			dbcache_set(dbpair_len, cache, userkey, *ppid, -1 /*not in sys db*/);
 			return -1;
 		}
 		return n;
 	} else {
-		char * cname;
 		cname = return_name_by_cache(dbpair_len, cache, id, &is_in_sysdb);
 		if (cname) {
+			/*
+ 			 * Found in cache
+ 			 */
+
 			if (strlen(cname) >  TARU_SYSDBNAME_LEN - 1) {
 				fprintf(stderr, "%s: user name too long for ustar headers: %s\n", swlib_utilname_get(), cname);
 
 			}
-			strncpy(buf, cname, TARU_SYSDBNAME_LEN - 1);
-			buf[TARU_SYSDBNAME_LEN - 1] = '\0';
-			return is_in_sysdb;
-		}
-		if ((*(v_get_name))(id, buf)) {
-			/*
-			 * found in system database
-			 */
-			dbcache_set(dbpair_len, cache, buf, id, 0);
-			return 0;
+			
+			retval = is_in_sysdb;
 		} else {
 			/*
-			 * Not found
-			 */
-			strncpy(buf, AHS_USERNAME_NOBODY, 10);
-			dbcache_set(dbpair_len, cache, buf, id, -1);
-			error_msg_control((char*)NULL, id, g_pwent_msg[idx]);
-			return -1;
+ 			 * Not found in cache, check system database
+ 			 */ 
+			cname = (*(v_get_name))(id, NULL, NULL);
+			if (cname) {
+				/*
+				 * found in system database
+				 */
+				dbcache_set(dbpair_len, cache, cname, id, 0);
+				retval = 0;
+			} else {
+				/*
+				 * Not found
+				 */
+				/* strob_strcpy(buf, AHS_USERNAME_NOBODY); */
+				cname = AHS_USERNAME_NOBODY;
+				dbcache_set(dbpair_len, cache, AHS_USERNAME_NOBODY, id, -1);
+				error_msg_control((char*)NULL, id, g_pwent_msg[idx]);
+				retval = -1;
+			}
+		}
+
+		if (cname && tarbuf) {
+			strncpy(tarbuf, cname , TARU_SYSDBNAME_LEN - 1);
+			tarbuf[TARU_SYSDBNAME_LEN - 1] = '\0';
+		}
+
+		if (cname && paxbuf) {
+			strob_strcpy(paxbuf, cname);
+		}
+		return retval;
+	}
+}
+
+
+void
+taru_set_filehdr_sysdb_nameid_by_policy(char * vselect,
+		struct new_cpio_header *file_hdr,
+		int termch, int tar_iflags_numeric_uids)
+{
+	unsigned long psid;
+	unsigned long c_id;
+	unsigned char c_c;
+	gid_t pgid;
+	uid_t puid;
+	int G;
+	char * sysusername;
+
+	if (strncasecmp("G", vselect, 1) == 0) {
+		G = 1; /* Group */
+		c_id = file_hdr->c_gid;
+		c_c = file_hdr->c_cg;
+		sysusername = ahsStaticGetTarGroupname(file_hdr);
+	} else {
+		G = 0; /* User */
+		c_id = file_hdr->c_uid;
+		c_c = file_hdr->c_cu;
+		sysusername = ahsStaticGetTarUsername(file_hdr);
+	}
+
+	if ((sysusername == NULL || strlen(sysusername) == 0) &&
+				c_c != TARU_C_BY_UNONE) {
+		E_DEBUG("sysusername nil");
+		/*
+ 		 * Get name by id number
+ 		 */
+		if (G) {
+			l_tar_sysdata_getgroup(c_id, NULL, (STROB*)(file_hdr->c_groupname));
+		} else {
+			l_tar_sysdata_getuser(c_id, NULL, (STROB*)(file_hdr->c_username));
+		}
+		/* if (tar_hdr) swlib_strncpy(tar_hdr_dest, NULL, SNAME_LEN); */
+	} else {
+		if ( /* No need to abstract the TARU_C_BY_<> values because
+			user and group have the same value */
+			c_c == TARU_C_BY_USYS || 
+			c_c == TARU_C_BY_UNAME
+			) 
+			{
+			/*
+ 			 * Get id by name
+ 			 */
+			E_DEBUG2("finding id for sysusername = %s", sysusername);
+
+			if (G) {
+				if (taru_get_gid_by_name(sysusername, &pgid)) {
+					pgid = NOBODY_ID;
+					E_DEBUG3("user name [%s] not found setting uid to %d", sysusername, (int)pgid);
+				}
+				psid =  (unsigned long)pgid;
+			} else {
+				if (taru_get_uid_by_name(sysusername, &puid)) {
+					puid = NOBODY_ID;
+					E_DEBUG3("user name [%s] not found setting uid to %d", sysusername, (int)puid);
+				}
+				psid =  (unsigned long)puid;
+			}
+
+			/* if (tar_hdr) swlib_strncpy(tar_hdr_dest, sysusername, SNAME_LEN); */
+		} else if (
+			c_c == TARU_C_BY_UID /* same as TARU_C_BY_GID */
+			) 
+			{
+			psid = c_id;
+			E_DEBUG2("finding user for uid  %d", (int)psid);
+			if (tar_iflags_numeric_uids == 0) {
+				if (G) {
+					l_tar_sysdata_getgroup(c_id, NULL, (STROB*)(file_hdr->c_groupname));
+				} else {
+					l_tar_sysdata_getuser(c_id, NULL, (STROB*)(file_hdr->c_username));
+				}
+				/* if (tar_hdr) swlib_strncpy(tar_hdr_dest, tarbuf, SNAME_LEN); */
+			} else {
+				/* if (tar_hdr) memset(tar_hdr_dest, '\0', SNAME_LEN); */
+			}
+		} else {
+			E_DEBUG("Using user and uid with no lookups");
+			psid = c_id;
+			/* if (tar_hdr) swlib_strncpy(tar_hdr_dest, sysusername, SNAME_LEN); */
+		}	
+
+		if (G) {
+			/* if (tar_hdr) GID_TO_CHARS(psid, tar_hdr_id, termch);*/
+		} else {
+			/* if (tar_hdr) UID_TO_CHARS(psid, tar_hdr_id, termch);*/
+		}
+	}
+}
+
+
+void
+taru_set_sysdb_nameid_by_policy(struct tar_header *tar_hdr, char * vselect, char * sysusername,
+		char * tarbuf, STROB *paxbuf,
+		struct new_cpio_header *file_hdr,
+		int termch, int tar_iflags_numeric_uids)
+{
+	char * tar_hdr_dest;
+	char * tar_hdr_id;
+	unsigned long psid;
+	unsigned long c_id;
+	unsigned char c_c;
+	gid_t pgid;
+	uid_t puid;
+	int G;
+
+	tar_hdr_id = NULL;
+	tar_hdr_dest = NULL;
+	if (strncasecmp("G", vselect, 1) == 0) {
+		G = 1; /* Group */
+		if (tar_hdr) tar_hdr_dest = tar_hdr->gname;
+		if (tar_hdr) tar_hdr_id = tar_hdr->gid;
+		c_id = file_hdr->c_gid;
+		c_c = file_hdr->c_cg;
+	} else {
+		G = 0; /* User */
+		if (tar_hdr) tar_hdr_dest = tar_hdr->uname;
+		if (tar_hdr) tar_hdr_id = tar_hdr->uid;
+		c_id = file_hdr->c_uid;
+		c_c = file_hdr->c_cu;
+	}
+
+	if ((sysusername == NULL || strlen(sysusername) == 0) &&
+				c_c != TARU_C_BY_UNONE) {
+		E_DEBUG("sysusername nil");
+		/*
+ 		 * Get name by id number
+ 		 */
+		if (G) {
+			l_tar_sysdata_getgroup(c_id, tarbuf, paxbuf);
+		} else {
+			l_tar_sysdata_getuser(c_id, tarbuf, paxbuf);
+		}
+		if (tar_hdr) swlib_strncpy(tar_hdr_dest, tarbuf, SNAME_LEN);
+	} else {
+		if ( /* No need to abstract the TARU_C_BY_<> values because
+			user and group have the same value */
+			c_c == TARU_C_BY_USYS || 
+			c_c == TARU_C_BY_UNAME
+			) 
+			{
+			/*
+ 			 * Get id by name
+ 			 */
+			E_DEBUG2("finding id for sysusername = %s", sysusername);
+
+			if (G) {
+				if (taru_get_gid_by_name(sysusername, &pgid)) {
+					pgid = NOBODY_ID;
+					E_DEBUG3("user name [%s] not found setting uid to %d", sysusername, (int)pgid);
+				}
+				psid =  (unsigned long)pgid;
+			} else {
+				if (taru_get_uid_by_name(sysusername, &puid)) {
+					puid = NOBODY_ID;
+					E_DEBUG3("user name [%s] not found setting uid to %d", sysusername, (int)puid);
+				}
+				psid =  (unsigned long)puid;
+			}
+
+			if (tar_hdr) swlib_strncpy(tar_hdr_dest, sysusername, SNAME_LEN);
+		} else if (
+			c_c == TARU_C_BY_UID /* same as TARU_C_BY_GID */
+			) 
+			{
+			psid = c_id;
+			E_DEBUG2("finding user for uid  %d", (int)psid);
+			if (tar_iflags_numeric_uids == 0) {
+				if (G) {
+					taru_get_tar_group_by_gid(psid, tarbuf);
+					if (paxbuf) taru_get_pax_group_by_gid(psid, paxbuf);
+				} else {
+					taru_get_tar_user_by_uid(psid, tarbuf);
+					if (paxbuf) taru_get_pax_user_by_uid(psid, paxbuf);
+				}
+				if (tar_hdr) swlib_strncpy(tar_hdr_dest, tarbuf, SNAME_LEN);
+			} else {
+				if (tar_hdr) memset(tar_hdr_dest, '\0', SNAME_LEN);
+			}
+		} else {
+			E_DEBUG("Using user and uid with no lookups");
+			psid = c_id;
+			if (tar_hdr) swlib_strncpy(tar_hdr_dest, sysusername, SNAME_LEN);
+		}	
+
+		if (G) {
+			if (tar_hdr) GID_TO_CHARS(psid, tar_hdr_id, termch); /* these really are the same */
+		} else {
+			if (tar_hdr) UID_TO_CHARS(psid, tar_hdr_id, termch); /* these really are the same */
 		}
 	}
 }
@@ -380,10 +622,10 @@ taru_get_uid_by_name(char *username, uid_t *puid)
 	long pid;
 	int ret;
 	E_DEBUG("");
-	ret = get_pwent(0, 0, username, NULL,
+	ret = get_pwent(0, 0, username, NULL, NULL,
 			&pid, 
 			l_getuidbyname, 
-			l_tar_idcache_getuser,
+			l_tar_sysdata_getuser,
 			&dbpair_user_nset,
 			userCacheM
 		);
@@ -391,14 +633,31 @@ taru_get_uid_by_name(char *username, uid_t *puid)
 	return ret;
 }
 
+
 int 
-taru_get_user_by_uid(uid_t uid, char * buf)
+taru_get_pax_user_by_uid(uid_t uid, STROB * buf)
 {
 	int ret;
 	E_DEBUG("");
-	ret = get_pwent(0, uid, NULL, buf, NULL,
+	ret = get_pwent(0, uid, NULL, NULL, buf, NULL,
 			l_getuidbyname, 
-			l_tar_idcache_getuser,
+			l_tar_sysdata_getuser,
+			&dbpair_user_nset,
+			userCacheM
+		);
+	return ret;
+}
+
+
+
+int 
+taru_get_tar_user_by_uid(uid_t uid, char * buf)
+{
+	int ret;
+	E_DEBUG("");
+	ret = get_pwent(0, uid, NULL, buf, NULL, NULL,
+			l_getuidbyname, 
+			l_tar_sysdata_getuser,
 			&dbpair_user_nset,
 			userCacheM
 		);
@@ -411,10 +670,10 @@ taru_get_gid_by_name(char *groupname, gid_t *guid)
 	int ret;
 	long pid;
 	E_DEBUG("");
-	ret =  get_pwent(1, 0, groupname, NULL,
+	ret =  get_pwent(1, 0, groupname, NULL, NULL,
 			&pid,
 			l_getgidbyname, 
-			l_tar_idcache_getgroup,
+			l_tar_sysdata_getgroup,
 			&dbpair_group_nset,
 			groupCacheM
 		);
@@ -423,13 +682,27 @@ taru_get_gid_by_name(char *groupname, gid_t *guid)
 }
 
 int 
-taru_get_group_by_gid(gid_t gid, char * buf)
+taru_get_pax_group_by_gid(gid_t gid, STROB * buf)
 {
 	int ret;
 	E_DEBUG("");
-	ret = get_pwent(1, gid, NULL, buf, NULL,
+	ret = get_pwent(1, gid, NULL, NULL, buf, NULL,
 			l_getgidbyname, 
-			l_tar_idcache_getgroup,
+			l_tar_sysdata_getgroup,
+			&dbpair_group_nset,
+			groupCacheM
+		);
+	return ret;
+}
+
+int 
+taru_get_tar_group_by_gid(gid_t gid, char * tarbuf)
+{
+	int ret;
+	E_DEBUG("");
+	ret = get_pwent(1, gid, NULL, tarbuf, NULL, NULL,
+			l_getgidbyname, 
+			l_tar_sysdata_getgroup,
 			&dbpair_group_nset,
 			groupCacheM
 		);
@@ -437,8 +710,7 @@ taru_get_group_by_gid(gid_t gid, char * buf)
 }
 
 int
-taru_split_name_ustar(TARU * taru, struct tar_header *tar_hdr,
-			char * name, int tar_iflags)
+taru_split_name_ustar(struct tar_header *tar_hdr, char * name, int tar_iflags)
 {
 	size_t length = strlen(name);
 	size_t i;
@@ -455,7 +727,7 @@ taru_split_name_ustar(TARU * taru, struct tar_header *tar_hdr,
 	i = split_long_name(name, length);
 	if (length - i - 1 > NAME_FIELD_SIZE)
 	{
-		fprintf(stderr, "%s: name too long (cannot be split): (i=%d) [%s]\n", swlib_utilname_get(), i, name);
+		fprintf(stderr, "%s: name too long (cannot be split): (i=%d) [%s]\n", swlib_utilname_get(), (int)i, name);
 		return -1;
 	}
 	if (i == 0) {
@@ -527,7 +799,8 @@ taru_set_filetype_from_tartype(char ch, mode_t * mode, char * filename)
 	return 0;
 }
 
-static void
+static
+void
 tarui_get_filetypes (mode_t mode, int * cpio_mode, char *tarflag)
 {
 	if (S_ISREG(mode)) {
@@ -594,31 +867,38 @@ taru_get_cpio_filetype (mode_t mode)
 	return cm;
 }
 
-/* Read a tar header, including the file name, from file descriptor IN_DES
-   into FILE_HDR.  */
-int
-taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
-			int in_des, char * fheader_buffer,
-			int * eoa, int tarheaderflags, int buf_len)
+
+static int
+i_taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
+			int in_des, char * fsource_buffer,
+			int * eoa, int tarheaderflags, int fsource_buffer_len, int fp_retval)
 {
 	mode_t modet;
 	int retval = 0;
-	char * long_link_buffer;
-	int long_link_bytes = 0;
+	unsigned char * ext_data_buffer;
+	int ext_data_buffer_len;
 	long bytes_skipped = 0;
 	int warned = FALSE;
 	union tar_record tar_rec;
-	struct tar_header *tar_hdr = (struct tar_header *) &tar_rec;
+	struct tar_header *tar_hdr;
 	char * aregfilename;
 	char * tmpname;
-	char * header_buffer = fheader_buffer;
+	char * read_buffer;
 	int do_record_header = 0;
 	int tar_iflags_retain_header_id = 0;
 	int buffer_offset;
+	int xhd_errorcode;
+	int eoaret;
 	int tmpret;
+	int tmpret_ustar;
+	int tmpret_extdata;
+	int read_so_far = 0;
 	long uidp;
 	long gidp;
 
+	
+	E_DEBUG("BEGIN");
+
 	buffer_offset = 0;
 	if (taru) {
 		tar_iflags_retain_header_id = (tarheaderflags & TARU_TAR_RETAIN_HEADER_IDS);
@@ -626,161 +906,202 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 	if (eoa) {
 		*eoa = 0;
 	}
-	long_link_buffer = NULL;
+	ext_data_buffer = NULL;
 	
 	if (taru) {
 		do_record_header = taru->do_record_headerM;
 	}
-	if (do_record_header) {
-		taru->header_lengthM = 0;	
-	}
 
-	E_DEBUG("");	
-	GNU_L_REREAD:	
+	E_DEBUG("");
+
+	strob_set_length(taru->read_bufferM, taru->read_buffer_posM + TARRECORDSIZE);
+	read_buffer = strob_str(taru->read_bufferM);
 	
-	if (header_buffer == NULL) {
-		tmpret = taru_tape_buffered_read(in_des, (void *)(&tar_rec), TARRECORDSIZE);
-		if (tmpret > 0 && tmpret != TARRECORDSIZE) {
-			fprintf(stderr, "%s: otar.c: short read in read_in_tar_header2 return=%d\n", swlib_utilname_get(), tmpret);
+	if (fsource_buffer == NULL) {
+		tmpret_ustar = taru_tape_buffered_read(in_des, (void *)(read_buffer), TARRECORDSIZE);
+		if (tmpret_ustar > 0 && tmpret_ustar != TARRECORDSIZE) {
+			fprintf(stderr,
+				"%s: otar.c: short read in read_in_tar_header2 return=%d\n",
+					swlib_utilname_get(), tmpret_ustar);
 			return -1;
-		} else if (tmpret == 0) {
+		} else if (tmpret_ustar == 0) {
 			return 0;
-		} else if (tmpret < 0) {
-			fprintf(stderr, "%s: otar.c: read error from read_in_tar_header2 return=%d\n", swlib_utilname_get(), tmpret);
+		} else if (tmpret_ustar < 0) {
+			fprintf(stderr,
+				"%s: otar.c: read error from read_in_tar_header2 return=%d\n",
+					swlib_utilname_get(), tmpret_ustar);
 			return -1;
 		} else {
 			; /* Ok */
 		}
 	} else {
-		memcpy((void *) (&tar_rec), header_buffer+buffer_offset, TARRECORDSIZE);
+		memcpy((void *)(read_buffer), fsource_buffer + taru->read_buffer_posM, TARRECORDSIZE);
+		fsource_buffer_len -= TARRECORDSIZE;
+		tmpret_ustar = TARRECORDSIZE;
 	}
+	taru->read_buffer_posM += TARRECORDSIZE;
+	strob_set_length(taru->read_bufferM, taru->read_buffer_posM + TARRECORDSIZE);
+	read_buffer = strob_str(taru->read_bufferM);
 
 	E_DEBUG("");	
 	if (do_record_header) {
 		strob_set_memlength(taru->headerM, taru->header_lengthM + 512);
 		memcpy((void*)(strob_str(taru->headerM) + taru->header_lengthM),
-				 (void*)(&tar_rec), 512);
+				 (void*)(read_buffer), 512);
 		taru->header_lengthM += 512;
 	}
 
 	retval += 512;
 
 	E_DEBUG("");	
-	/* Check for a block of 0's.  */
-	if (null_block((long *) &tar_rec, TARRECORDSIZE)) {
-#if 0
-		/* Found one block of 512 0's. If the next block is also all 0's
-		   then this is the end of the archive.  If not, assume the
-		   previous block was all corruption and continue reading
-		   the archive.  */
-		/* Commented out because GNU tar sometimes creates archives with
-		   only one block of 0's at the end.  This happened for the
-		   cpio 2.0 distribution!  */
-		if (header_buffer)
-			return 0;
-		taru_tape_buffered_read(in_des,
-					(void *)&tar_rec, TARRECORDSIZE);
-		if (null_block((long *) &tar_rec, TARRECORDSIZE))
-#endif
-		{
-			ahsStaticSetTarFilename(file_hdr,
-					CPIO_INBAND_EOA_FILENAME);
-			if (eoa) *eoa = 1;
-			return 1024;
+	/* 
+ 	 * Check for a block of 0's.
+ 	 */
+	if (null_block((long *) read_buffer, TARRECORDSIZE)) {
+		ahsStaticSetTarFilename(file_hdr, CPIO_INBAND_EOA_FILENAME);
+
+		/*
+		 * Allow one (1) null block to signal EOA, this is wrong as two
+		 * should be required. Check all remaining blocks to determine length
+		 * of trailer blocks
+		 */
+		if (fsource_buffer == NULL) {
+			eoaret = taru_tape_buffered_read(in_des, (void *)read_buffer, TARRECORDSIZE);
+		} else {
+			eoaret = TARRECORDSIZE;
+			memset(read_buffer, '\0', TARRECORDSIZE);
 		}
-#if 0
-		bytes_skipped = TARRECORDSIZE;
-#endif
+		if (
+			eoaret != TARRECORDSIZE ||
+			null_block((long *) read_buffer, TARRECORDSIZE) == 0
+		) {
+			/*
+			 * This is unexpected as it indicates a possible corrupt
+			 * archive.
+			 */
+
+			fprintf(stderr,
+			"%s: possible corrupt archive, non-null block found after one null block\n",
+							swlib_utilname_get());
+			retval = -1;
+		} else {
+			/*
+			 * normal exit, two (2) null blocks
+			 */
+			retval = 1024;
+		}
+		taru->read_buffer_posM += TARRECORDSIZE;
+		if (eoa) *eoa = retval;
+		return retval;  /* End of Archive */
 	}
 
+	tar_hdr = (struct tar_header*)(read_buffer);
 	E_DEBUG("");	
-	if (*(((char*)tar_hdr) + 156) == 'L') {
-		if (header_buffer && buf_len == TARRECORDSIZE) {
-			/*
-			* your screwed.
-			*/
-			fprintf(stderr,
-			"fatal error, can not read GNU type L extension\n");
-			exit(24);
-		} else if (header_buffer && buf_len > TARRECORDSIZE) {
-			taru_otoumax(tar_hdr->size, &file_hdr->c_filesize);
-			if (file_hdr->c_filesize > 511) {
-				/*
-				* pretend can't be done. At the moment
-				* filenames this long are not supported.
-				*/
-				fprintf(stderr,
-	"fatal error, can not read GNU type L extension because file name\n"
-	"is too long.\n");
-				exit(24);
-			}
-			long_link_bytes += 512;
-			retval += 512;
+	if (
+		tar_hdr->typeflag == GNUTYPE_LONGNAME ||
+		tar_hdr->typeflag == GNUTYPE_LONGLINK ||
+		tar_hdr->typeflag == XHDTYPE ||
+		tar_hdr->typeflag == XGLTYPE
+	  ) {
+		E_DEBUG2("GOT Extened Header data type:[%c]", tar_hdr->typeflag);
 
-			E_DEBUG("");	
-			/* Sanity check */
-			if (retval + long_link_bytes != buf_len) {
-				fprintf(stderr, "fatal error in taru_in_tar_header2: type L error\n");
-				exit(24);
-			}
-			long_link_buffer = malloc(TARRECORDSIZE + 1);
-			memset(long_link_buffer, '\0', TARRECORDSIZE + 1);
-			memcpy(long_link_buffer, header_buffer+512, 512);
-			long_link_buffer[TARRECORDSIZE] = '\0';
+		/*
+ 		 * Determine the size of the exteneded header data
+ 		 */
 
-			if (do_record_header) {
-				strob_set_memlength(taru->headerM,
-						taru->header_lengthM + 512);
-				memcpy((void*)(strob_str(taru->headerM) +
-							 taru->header_lengthM),
-					(void*)(long_link_buffer), 512);
-				taru->header_lengthM += 512;
+		taru_otoumax(tar_hdr->size, &file_hdr->c_filesize);
+
+		/*
+ 		 * Read the Extended Header Data blocks
+ 		 */
+
+		if (fsource_buffer) {
+			E_DEBUG("in fsource_buffer");
+			if ((unsigned long)fsource_buffer_len < file_hdr->c_filesize) {
+				E_DEBUG3("fsource_buffer_len < file_hdr->c_filesize: %d < %d", (int) fsource_buffer_len, (int)file_hdr->c_filesize);
+				tmpret_extdata = -1;
+			} else {
+				E_DEBUG("reading pax data blocks");
+				tmpret_extdata = taru_read_pax_data_blocks(taru, -1,
+					fsource_buffer,
+					file_hdr->c_filesize,
+					taru_tape_buffered_read);
+			}
+			if (tmpret_extdata > 0) {
+				fsource_buffer_len -= tmpret_extdata;
 			}
-			buffer_offset = TARRECORDSIZE + TARRECORDSIZE;
-			goto GNU_L_REREAD;
 		} else {
-			/* 
-			* read the data block which contains the filename.
-			*/
-			E_DEBUG("");	
-			taru_otoumax(tar_hdr->size, &file_hdr->c_filesize);
-
-			if (file_hdr->c_filesize > 511) {
-				/*
-				* pretend can't be done. At the moment
-				* filenames this long are not supported.
-				*/
-				fprintf(stderr,
-	"fatal error, can not read GNU type L extension because file name\n"
-	"is too long.\n");
-				exit(24);
-
-			}
-			long_link_buffer = malloc(TARRECORDSIZE + 1);
-			memset(long_link_buffer, '\0', TARRECORDSIZE + 1);
-			SWLIB_ALLOC_ASSERT(long_link_buffer != NULL);
-
-			if (taru_tape_buffered_read(in_des,
-						(void *)long_link_buffer,
-						TARRECORDSIZE) !=
-							TARRECORDSIZE) {
-				fprintf(stderr, 
-				"error reading long link data block, fatal.\n");
-				exit(25);
-			}
-			long_link_bytes += 512;
-			long_link_buffer[TARRECORDSIZE] = '\0';
-
-			if (do_record_header) {
-				strob_set_memlength(taru->headerM,
-						taru->header_lengthM + 512);
-				memcpy((void*)(strob_str(taru->headerM) +
-							 taru->header_lengthM),
-					(void*)(long_link_buffer), 512);
-				taru->header_lengthM += 512;
-			}
-			goto GNU_L_REREAD;
+			E_DEBUG("fsource_buffer is NULL");
+			tmpret_extdata = taru_read_pax_data_blocks(taru, in_des,
+					NULL,
+					file_hdr->c_filesize,
+					taru_tape_buffered_read);
 		}
+
+		if (tmpret_extdata < 0) {
+			fprintf(stderr, "fatal error reading extended header data blocks\n");
+			exit(24);
+		}
+
+		/*
+ 		 * Now interpret the exteneded header data blocks and assign the
+ 		 * values to the file_hdr fields setting the usage mask to mark
+ 		 * the field as set (thus overriding the value in the regular ustar header)
+ 		 * to be read next.
+		 */
+
+		ext_data_buffer = (unsigned char*)strob_str(taru->read_bufferM)
+						+ taru->read_buffer_posM 
+						- tmpret_extdata;
+		ext_data_buffer_len = file_hdr->c_filesize;
+
+		switch (tar_hdr->typeflag) {
+			case GNUTYPE_LONGNAME:
+				/* Null terminated by taru_read_pax_data_blocks */
+				ahsStaticSetTarFilename(file_hdr, (char*)ext_data_buffer);
+				E_DEBUG2("Setting LongName Data: [%s]", ahsStaticGetTarFilename(file_hdr));	
+				file_hdr->extHeader_usage_maskM |= TARU_EHUM_PATH;
+			break;
+			case GNUTYPE_LONGLINK:
+				/* Null terminated by taru_read_pax_data_blocks */
+				ahsStaticSetPaxLinkname(file_hdr, (char*)ext_data_buffer);
+				E_DEBUG2("Setting LongLink Data: [%s]", ahsStaticGetTarLinkname(file_hdr));	
+				file_hdr->extHeader_usage_maskM |= TARU_EHUM_LINKPATH;
+			break;
+			case XGLTYPE:
+				fprintf(stderr, "type XGLTYPE (Pax Global Header) not supported\n");
+				exit(26);
+				break;
+			case XHDTYPE:
+				tmpret = taru_read_all_ext_header_records (file_hdr,
+									(char*)ext_data_buffer,
+									ext_data_buffer_len,
+									&xhd_errorcode);
+				if (tmpret < 0 ) {
+					fprintf(stderr, "%s: error reading Pax extended header, errorcode=%d\n",
+						swlib_utilname_get(),  xhd_errorcode);
+					return -11;
+				}
+				break;
+			default:
+				/* error never should happen */
+				fprintf(stderr, "fatal internal error\n");
+				exit(26);
+
+			break;
+		}
+
+		/*
+		 * Now make recursive call with the extened header info and
+		 * the header_buffer pointer moved ahead 'read_so_far' bytes ahead
+ 		 */ 
+
+		E_DEBUG("Calling i_taru_read_in_tar_header2");	
+		read_so_far = tmpret_ustar + tmpret_extdata;
+		retval = i_taru_read_in_tar_header2(taru, file_hdr, in_des,
+					fsource_buffer ? fsource_buffer + read_so_far : (char*)NULL,
+					eoa, tarheaderflags, fsource_buffer_len, fp_retval + read_so_far);
+		return retval;
 	}
 
 	E_DEBUG("");	
@@ -789,7 +1110,7 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 		taru_otoul(tar_hdr->chksum, &file_hdr->c_chksum);
 
 		if (file_hdr->c_chksum != taru_tar_checksum(tar_hdr)) {
-			if (!header_buffer) {
+			if (!fsource_buffer) {
 
 		/* If the checksum is bad, skip 1 byte and try again.  When
 		   we try again we do not look for an EOF record (all zeros),
@@ -824,28 +1145,26 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			}
 
 		}	/* bad check sum */
+				
 
-		if (long_link_buffer == NULL) {
-			/*
-			* not GNU type L.
-			*/
+		if (((file_hdr->extHeader_usage_maskM) & TARU_EHUM_PATH) == 0) {
 			tmpname = taru_dup_tar_name((void*)tar_hdr);
 			ahsStaticSetTarFilename(file_hdr, tmpname);
 		} else {
-			/*
-			* Type 'L' long link.
-			*/
-			ahsStaticSetTarFilename(file_hdr, long_link_buffer);
-			tmpname = ahsStaticGetTarFilename(file_hdr);
-			free(long_link_buffer);
-			long_link_buffer = NULL;
+			;
+			tmpname = NULL;
+			E_DEBUG("Path set by extended headers");	
 		}
+
 		E_DEBUG("");	
-		file_hdr->c_namesize = strlen(tmpname) + 1;
+		file_hdr->c_namesize = strlen(ahsStaticGetTarFilename(file_hdr)) + 1;
 
 		if ((char)(tar_hdr->typeflag) == LNKTYPE) {
+			/* hard link */
+			E_DEBUG("setting file_hdr->c_nlink = 2");
 			file_hdr->c_nlink = 2;
 		} else {
+			E_DEBUG("setting file_hdr->c_nlink = 1");	
 			file_hdr->c_nlink = 1;
 		}
 
@@ -856,7 +1175,7 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 		ahsStaticSetTarUsername(file_hdr, tar_hdr->uname);
 		if (
 			tar_iflags_retain_header_id == 0 &&
-			l_cache_getuidbyname(tar_hdr->uname, &uidp) == 0
+			l_cache_tar_getuidbyname(tar_hdr->uname, &uidp) == 0
 		) {
 			E_DEBUG("");	
 			file_hdr->c_uid = uidp;
@@ -866,8 +1185,7 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 		}
 		E_DEBUG("");	
 		taru_set_filetype_from_tartype((char)(tar_hdr->typeflag),
-			(modet=(mode_t)(file_hdr->c_mode), &modet),
-				tmpname);
+			(modet=(mode_t)(file_hdr->c_mode), &modet), tmpname);
 	
 		E_DEBUG("");	
 		switch ((int)(tar_hdr->typeflag)) {
@@ -882,7 +1200,7 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 		ahsStaticSetTarGroupname(file_hdr, tar_hdr->gname);
 		if (
 			tar_iflags_retain_header_id == 0 &&
-			l_cache_getgidbyname(tar_hdr->gname, &gidp) == 0
+			l_cache_tar_getgidbyname(tar_hdr->gname, &gidp) == 0
 		) {
 			file_hdr->c_gid = gidp;
 		} else {
@@ -896,7 +1214,12 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 		taru_otoul(tar_hdr->devminor,
 				(unsigned long *) &file_hdr->c_rdev_min);
 		E_DEBUG("");	
+
+
+		/* This closes (frees) the buffer, don't know why to do this
+ 		 * in retrospect
 		ahsStaticSetTarLinkname(file_hdr, (char *)(NULL));
+ 		 */
 
 		switch (tar_hdr->typeflag) {
 		case REGTYPE:
@@ -941,8 +1264,17 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 #endif
 		case SYMTYPE:
 #ifdef CP_IFLNK
+			E_DEBUG("at SYMLINK");
 			file_hdr->c_mode |= CP_IFLNK;
-			ahsStaticSetTarLinkname(file_hdr, tar_hdr->linkname);
+
+			if (((file_hdr->extHeader_usage_maskM) & TARU_EHUM_LINKPATH) == 0) {
+				E_DEBUG2("linkname=[%s]", tar_hdr->linkname);
+				ahsStaticSetTarLinkname(file_hdr, tar_hdr->linkname);
+				E_DEBUG2("ahs linkname=[%s]", ahsStaticGetTarLinkname(file_hdr));
+			} else {
+				;
+				E_DEBUG("Path set by extended headers");	
+			}
 			file_hdr->c_filesize = 0;
 			break;
 			/* Else fall through.  */
@@ -971,9 +1303,69 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 				swlib_utilname_get(), bytes_skipped);
 		if (tarheaderflags & TARU_TAR_FRAGILE_FORMAT) return -4;
         }
-	return retval + long_link_bytes;
+	return retval + fp_retval;
 }
 
+
+
+/* Read a tar header, including the file name, from file descriptor IN_DES
+   into FILE_HDR.  */
+int
+taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
+			int in_des, char * fsource_buffer,
+			int * eoa, int tarheaderflags, int fsource_buffer_len)
+{
+	EXATT * p;
+	CPLOB * extlist;
+	int index;
+	int ret;
+
+	E_DEBUG("");
+
+	if (taru) taru->read_buffer_posM = 0;
+	if (taru && taru->do_record_headerM) {
+		taru->header_lengthM = 0;	
+	}
+	file_hdr->extHeader_usage_maskM = 0;
+
+	/*
+ 	 * Initialize the list of extended header records to
+ 	 * a status of not used
+ 	 */
+	taru_exattlist_init(file_hdr);
+
+	ahsStaticSetTarFilename(file_hdr, "");
+	ahsStaticSetPaxLinkname(file_hdr, "");
+
+	E_DEBUG2("fsource_buffer=%p", fsource_buffer);
+	E_DEBUG2("fsource_buffer_len=%d", fsource_buffer_len);
+
+	ret = i_taru_read_in_tar_header2(taru, file_hdr, in_des, fsource_buffer,
+			eoa, tarheaderflags, fsource_buffer_len, 0);
+
+	/*
+ 	 * Now write the values from the extended headers
+ 	 */
+
+	if (file_hdr->extHeader_usage_maskM) {
+		extlist = (CPLOB*)(file_hdr->extattlistM);
+		if (extlist) {
+			/*
+ 			 * Loop over the extended header records
+ 			 * to override the the values from the ustar header
+ 			 * that have extended header records.
+ 			 */
+			index = 0;
+			while((p=(EXATT*) cplob_val(file_hdr->extattlistM, index))) {
+				taru_exatt_override_ustar (file_hdr, p);
+				index++;
+			}
+		}
+	}
+	return ret;
+}
+
+
 int
 taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			int out_des, char *header_buffer,
@@ -1001,7 +1393,7 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 		return -1;
 	}
 	
-	if (*((char*)(taru)) != 'A') {
+	if (*((char*)(taru)) != 'A' /* magic bytes */) {
 		fprintf(stderr,
 	"%s: fatal error: taru is uninitialized, this needs to be fixed\n",
 		swlib_utilname_get());
@@ -1012,7 +1404,6 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 	E_DEBUG3("username=[%s] groupname=[%s]", username, groupname);
 	E_DEBUG3("file_hdr: uid=[%d] gid=[%d]", (int)(file_hdr->c_uid), (int)(file_hdr->c_gid));
 
-
 	bzero((char *) &tar_rec, TARRECORDSIZE);
 
 	tar_iflags_like_star = (tar_iflags & TARU_TAR_BE_LIKE_STAR);
@@ -1035,11 +1426,10 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			ahsStaticGetTarFilename(file_hdr));
 
 	if (strob_strlen(taru->u_name_bufferM) == 0) {
-		fprintf(stderr, "internal error\n");
+		fprintf(stderr, "%s: internal error: taru->u_name_bufferM zero length\n", swlib_utilname_get());
 		return -1;
 	}
 
-
 	/*
 	 * Add a trailing slash if a directory
 	 */
@@ -1070,7 +1460,7 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 	 * the pathname is now in (char*)u_name_buffer
 	 */
 
-	if (taru_set_new_name(taru, tar_hdr, -1, u_name_buffer)) {
+	if (taru_set_new_name(/*taru, */ tar_hdr, -1, u_name_buffer, taru->taru_tarheaderflagsM)) {
 		fprintf(stderr, "%s: %s not dumped\n", swlib_utilname_get(), u_name_buffer);
 		return -13;
 	}
@@ -1102,7 +1492,7 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			 */
 			if (strlen(tmpp) > TARLINKNAMESIZE) {
 				fprintf(stderr,
-					"%s, link name [%s] too long for tar.\n",
+					"%s, link name [%s] too long for tar (returning -4).\n",
 						swlib_utilname_get(), tmpp);
 				E_DEBUG("LEAVING");
 				return -4;
@@ -1133,17 +1523,28 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 #endif				/* CP_IFIFO */
 #ifdef CP_IFLNK
 	case CP_IFLNK:
-		tar_hdr->typeflag = SYMTYPE;
-
 		tmpp = ahsStaticGetTarLinkname(file_hdr);
-		if (strlen(tmpp) > TARLINKNAMESIZE) {
-			fprintf(stderr, "%s: link name [%s] too long for tar.\n",
+		if (
+			strcmp(ahsStaticGetTarFilename(file_hdr), GNU_LONG_LINK) == 0 
+		) {
+			if (strlen(tmpp) > 510) {
+				fprintf(stderr, "%s: link name [%s] too long this implementation at this time.\n",
 					swlib_utilname_get(), tmpp);
-			E_DEBUG("LEAVING");
-			return -5;
+				return -5;
+			}
+			tar_hdr->typeflag = GNUTYPE_LONGLINK;
+			SIZE_TO_CHARS(strlen(tmpp),  tar_hdr->size, termch);
+		} else { 
+			tar_hdr->typeflag = SYMTYPE;
+			if (strlen(tmpp) > TARLINKNAMESIZE) {
+				fprintf(stderr, "%s: link name [%s] too long for tar.\n",
+						swlib_utilname_get(), tmpp);
+				E_DEBUG("LEAVING");
+				return -5;
+			}
+			SIZE_TO_CHARS(0,  tar_hdr->size, termch);
 		}
 		strncpy(tar_hdr->linkname, tmpp, TARLINKNAMESIZE);
-		SIZE_TO_CHARS(0,  tar_hdr->size, termch);
 		break;
 #endif				/* CP_IFLNK */
 
@@ -1151,7 +1552,18 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 		if (file_hdr->c_mode == 0 &&
 			strcmp(ahsStaticGetTarFilename(file_hdr),
 						GNU_LONG_LINK) == 0) {
-			tar_hdr->typeflag = 'L';
+			/*
+ 			 * Special Case for GN_LONG_LINK file types
+ 			 */
+
+			if (strlen(ahsStaticGetTarLinkname(file_hdr)) == 0) {
+				tar_hdr->typeflag = GNUTYPE_LONGNAME;
+			} else {
+				/*
+ 				 * this is the code path for hard links
+ 				 */
+				tar_hdr->typeflag = GNUTYPE_LONGLINK;
+			}
 		} else {
 			fprintf(stderr, 
 	"bad CP_?? filetype in taru_write_out_tar_header2: mode = %d\n",
@@ -1175,11 +1587,14 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 		strncpy(tar_hdr->magic + 6, " ", 2);
 	}
 
+	/* -------------------------------------------------------------------- */
+
+
 	if ((username == NULL || strlen(username) == 0) &&
 				file_hdr->c_cu != TARU_C_BY_UNONE) {
 		E_DEBUG("username nil");
-		l_tar_idcache_getuser(file_hdr->c_uid, u_ent_buffer);
-		strncpy(tar_hdr->uname, u_ent_buffer, SNAME_LEN);
+		l_tar_sysdata_getuser(file_hdr->c_uid, u_ent_buffer, NULL);
+		swlib_strncpy(tar_hdr->uname, u_ent_buffer, SNAME_LEN);
 	} else {
 		uid_t puid;
 		if (
@@ -1200,7 +1615,7 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			puid = file_hdr->c_uid;
 			E_DEBUG2("finding user for uid  %d", (int)puid);
 			if (tar_iflags_numeric_uids == 0) {
-				taru_get_user_by_uid(puid, u_ent_buffer);
+				taru_get_tar_user_by_uid(puid, u_ent_buffer);
 				swlib_strncpy(tar_hdr->uname, u_ent_buffer, SNAME_LEN);
 			} else {
 				swlib_strncpy(tar_hdr->uname, "", SNAME_LEN);
@@ -1216,7 +1631,7 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 	if ((groupname == NULL || strlen(groupname) == 0)
 				&& file_hdr->c_cg != TARU_C_BY_GNONE) {
 		E_DEBUG("groupname nil");
-		l_tar_idcache_getgroup(file_hdr->c_gid, u_ent_buffer);
+		l_tar_sysdata_getgroup(file_hdr->c_gid, u_ent_buffer, NULL);
 		strncpy(tar_hdr->gname, u_ent_buffer, SNAME_LEN);
 	} else {
 		gid_t pgid;
@@ -1239,7 +1654,7 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			pgid = file_hdr->c_gid;
 			E_DEBUG2("finding gname for gid  %d", (int)pgid);
 			if (tar_iflags_numeric_uids == 0) {
-				taru_get_group_by_gid(pgid, u_ent_buffer);
+				taru_get_tar_group_by_gid(pgid, u_ent_buffer);
 				swlib_strncpy(tar_hdr->gname, u_ent_buffer, SNAME_LEN);
 			} else {
 				swlib_strncpy(tar_hdr->gname, "", SNAME_LEN);
@@ -1252,6 +1667,7 @@ taru_write_out_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 		GID_TO_CHARS(pgid, tar_hdr->gid, termch);
 	}
 
+
 	if (tar_iflags_numeric_uids) {
 		memset((void*)tar_hdr->uname, '\0', SNAME_LEN);
 		memset((void*)tar_hdr->gname, '\0', SNAME_LEN);
