@@ -46,21 +46,26 @@
 /* These macros work even on ones'-complement hosts (!).
    The extra casts work around common compiler bugs.  */
 #define TYPE_SIGNED(t) (! ((t) 0 < (t) -1))
+
+/* Removed by jhl on 17MAR2013 to eliminate compiler warning.
+   They don't appear anywhere.
 #define TYPE_MINIMUM(t) (TYPE_SIGNED (t) \
 			 ? ~ (t) 0 << (sizeof (t) * CHAR_BIT - 1) \
 			 : (t) 0)
 #define TYPE_MAXIMUM(t) ((t) (~ (t) 0 - TYPE_MINIMUM (t)))
+*/
+
 
 /* Bound on length of the string representing an integer value of type t.
    Subtract one for the sign bit if t is signed;
    302 / 1000 is log10 (2) rounded up;
    add one for integer division truncation;
    add one more for a minus sign if t is signed.  */
-#define INT_STRLEN_BOUND(t) \
+#define x_INT_STRLEN_BOUND(t) \
   ((sizeof (t) * CHAR_BIT - TYPE_SIGNED (t)) * 302 / 1000 \
    + 1 + TYPE_SIGNED (t))
 
-#define UINTMAX_STRSIZE_BOUND (INT_STRLEN_BOUND (uintmax_t) + 1)
+#define UINTMAX_STRSIZE_BOUND (x_INT_STRLEN_BOUND (uintmax_t) + 1)
 
 
 enum gnutar_archive_format
@@ -130,7 +135,7 @@ void uintmax_to_chars PARAMS ((uintmax_t, char *, size_t, int OSIX_FORMAT, int t
 
 char *stringify_uintmax_t_backwards PARAMS ((uintmax_t, char *));
 
-char const *tartime PARAMS ((time_t));
+/* char const *tartime PARAMS ((time_t)); */
 
 #define GID_FROM_HEADER(where) gid_from_header (where, sizeof (where))
 #define MAJOR_FROM_HEADER(where) major_from_header (where, sizeof (where))
