@@ -131,6 +131,8 @@ common_setup_write_header(XFORMAT * xux, struct new_cpio_header **p_hdr1,
                 // only lstat the source if name is null.
 	        */ 
 		*strob_str(xux->linkname2M) = '\0';
+		/* taru_exattlist_init(xformat_vfile_hdr(xux)); */
+		/* taru_exattlist_init(hdr0); */
 		*p_hdr1=(struct new_cpio_header*)(xformat_vfile_hdr(xux));
 		if (!t_name ) {
                     	char * p1_name = ahsStaticGetTarFilename(*p_hdr1);
@@ -203,7 +205,7 @@ common_setup_write_header(XFORMAT * xux, struct new_cpio_header **p_hdr1,
 			XFORMAT_E_DEBUG2("[1 0 0] %s", taru_header_dump_string_s(hdr0, "[1 0 0] hdr0 "));
 			#endif
 			ahsStaticSetTarFilename(hdr0, strob_str(xux->name2M));
-			ahsStaticSetTarLinkname(hdr0, strob_str(xux->linkname2M));
+			ahsStaticSetPaxLinkname(hdr0, strob_str(xux->linkname2M));
 			*p_hdr1 = hdr0;
 		 } else if ( (t_stat && t_name && t_source == NULL) ||
 					(t_stat && t_name && t_source) ) {
@@ -456,6 +458,13 @@ xformat_get_output_format(XFORMAT * xux)
 	return (int)(xux->output_format_codeM);
 }
 
+void
+xformat_init_vfile_header(XFORMAT * xux)
+{
+	taru_init_header(xformat_vfile_hdr(xux));
+}
+
+
 struct new_cpio_header *
 xformat_vfile_hdr(XFORMAT * xux)
 {
@@ -765,6 +774,7 @@ xformat_get_next_dirent(XFORMAT * xux, struct stat * st)
 	char * name;
 	XFORMAT_E_DEBUG("");
 	name = swvarfs_get_next_dirent(xux->swvarfsM, st);
+	xux->eoaM = xux->swvarfsM->eoaM;
 	return name;
 }
 
@@ -888,6 +898,7 @@ xformat_write_file_by_fd (XFORMAT * xux,
 	}
 	if((ret=common_setup_write_header(xux, &hdr1,
 				hdr0, t, name, (char*)NULL)) == 0) {
+		XFORMAT_E_DEBUG("");
 		if ((ret=taru_write_archive_member_header(xux->taruM, NULL, 
 					hdr1, 
 					xux->link_recordM, 
@@ -898,6 +909,7 @@ xformat_write_file_by_fd (XFORMAT * xux,
 					hdr0, 
 					xux->taruM->taru_tarheaderflagsM)) > 0)
 		{
+			XFORMAT_E_DEBUG("");
 			ret1 = taru_write_archive_member_data(xux->taruM,
 					hdr0, xux->ofdM, source_fd,
 					(int(*)(int))fout,
@@ -1010,6 +1022,7 @@ xformat_write_file(XFORMAT * xux, struct stat *t, char * name, char * source)
 	        }
 	}
 
+	XFORMAT_E_DEBUG("calling taru_write_archive_member_header");
 	ret=taru_write_archive_member_header(xux->taruM, NULL, 
 					hdr1, 
 					xux->link_recordM, 
@@ -1019,8 +1032,9 @@ xformat_write_file(XFORMAT * xux, struct stat *t, char * name, char * source)
 					xux->output_format_codeM, 
 					hdr0, 
 					xux->taruM->taru_tarheaderflagsM);
+	XFORMAT_E_DEBUG2("taru_write_archive_member_header ret=%d", ret);
         if (ret > 0 && fd >= 0) {
-		XFORMAT_E_DEBUG("");
+		XFORMAT_E_DEBUG("calling taru_write_archive_member_data");
 		ret1 = taru_write_archive_member_data(xux->taruM, hdr0,
 						xux->ofdM, fd,
 						(int(*)(int))NULL,
@@ -1029,9 +1043,9 @@ xformat_write_file(XFORMAT * xux, struct stat *t, char * name, char * source)
 		ret += ret1;
 		local_v_close(xux, fd); 
 	}
-	xux->bytes_writtenM += ret;
+	if (ret > 0) xux->bytes_writtenM += ret;
 	ahsStaticDeleteFilehdr(hdr0);
-	XFORMAT_E_DEBUG("LEAVING");
+	XFORMAT_E_DEBUG2("LEAVING with ret=%d", ret);
 	return ret;
 }
 
@@ -1311,6 +1325,7 @@ xformat_copy_pass_md5(XFORMAT * xux, int dst_fd, int src_fd, char * md5buf)
 	}
      
 	if (dst_fd < 0) {
+		digs = NULL;
 		if (taru_read_amount(src_fd, file_hdr0->c_filesize) !=
 					(intmax_t)(file_hdr0->c_filesize)) {
 			XFORMAT_E_FAIL("");
@@ -1334,11 +1349,11 @@ xformat_copy_pass_md5(XFORMAT * xux, int dst_fd, int src_fd, char * md5buf)
 		taru_tape_skip_padding(xux->ifdM,
 				file_hdr0->c_filesize, xux->format_codeM);
 	}
-	if (md5buf) {
+	if (md5buf && digs) {
 		strcpy(md5buf, digs->md5);
 	}
 	if (retval > 0) xux->bytes_writtenM += retval;
-	taru_digs_delete(digs);
+	if (digs) taru_digs_delete(digs);
 	return retval;
 }
 
@@ -1594,6 +1609,7 @@ xformat_get_source_filename (XFORMAT * xux, char * buf) {
 int
 xformat_is_end_of_archive(XFORMAT * xux) {
 	int ret;
+	int trailer_blocks_read_return;
 	char * name;
 		
 	XFORMAT_E_DEBUG("");
@@ -1612,8 +1628,13 @@ xformat_is_end_of_archive(XFORMAT * xux) {
 		* // read the null trailer blocks so the they get put in
 		* // the taruib buffer object.
 		*/
+
+		/* the following is now superfulous since the entirety of the trailers
+		 * blocks are being read.
+		 */
 		if (xformat_get_pass_fd(xux)) {
-			taru_read_amount(xformat_get_ifd(xux), -1);
+			trailer_blocks_read_return = taru_read_amount(xformat_get_ifd(xux), -1);
+			XFORMAT_E_DEBUG2("unread trailer blocks = [%d]", trailer_blocks_read_return);
 		}
 	}
 	return ret;
