@@ -1171,8 +1171,14 @@ LABEL_REREAD:
 					   and determine if it is the same for every file. */
 					UINFILE_E_DEBUG2("||||||| [%s]", name);
 					/* Look for the slackware control directory "install/", if we find
-					   one assume it is slackware */
-					if (strcmp(name, UINFILE_MAGIC_slack "/") == 0) {
+					   one assume it is slackware.
+					   Also, need to look for ./install/ since some packagers
+					   prepend a ./	
+				  	*/
+					if (
+						strcmp(name, UINFILE_MAGIC_slack "/") == 0 ||
+						strcmp(name, "./" UINFILE_MAGIC_slack "/") == 0
+					) {
 						UINFILE_E_DEBUG("found install directory");
 						found = 1;
 						break;
