@@ -8,9 +8,9 @@
 #include <typeinfo>
 #include "stream_config.h"
 #include "swparser.h"
-#include "swparser_global.h"
 #include "swpackagefile.h"
 #include "swcatalogfile.h"
+#include "swmain.h"
 
 extern "C" {
 #include "swlib.h"
