@@ -5,7 +5,7 @@
 
 #include "swptrlist.h"
 #include "swpackagefile.h"
-#include "swparser_global.h"
+#include "swmain.h"
 
 int
 main (int argc, char ** argv)
